function lambda2cdf( x, p, q, a2, delta, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Calculates the probability that a random variable distributed
!     according to the lambda-square distribution with P and Q
!     degrees of freedom and A2 eccentricity parameter, is less than
!     or equal to X
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . First degrees of freedom       (P >  0) - Real
!     Q     - Input . Second   "    "     "          (Q >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     lambda2cdf (stopping criterion)
!                     (eps < delta < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then lambda2cdf = zero)
!                     2 = maximum number of iterations reached
!                         (then lambda2cdf = value at last iteration,
!                                        or zero)
!                     3 = required accuracy cannot be reached
!                         (then lambda2cdf = value at last iteration)
!                     4 = error in auxiliary function
!                         (chi2cdf or phi2cdf)
!
!     External functions called:
!       CHI2CDF  PHI2CDF
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  LOG  MAX  NINT  (and DLGAMA if available)
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: lambda2cdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q, a2 , delta
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: chi2cdf, phi2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: qlimit=1.0e5_8, cinf=1.0e-20_8
   real(kind=8), parameter :: dflimit=1.0e6_8
   real(kind=8), parameter :: eps=0.223e-15_8, tinyr=1.0e-307_8, explower=-706.893_8
   real(kind=8), parameter :: relerr=1.0e-14_8 ! Relative error assumed
                                               ! in recurrence calculations
   real(kind=8), parameter :: xp=tinyr/relerr
   integer, parameter :: jmax=200

   !     dflimit = limit for approximation in chi2cdf function
   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     tinyr = the smallest positive real
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))

   real(kind=8) :: aqal, bl, dj, dj1, erp, err, ga, gl, g0,  &
                   kgj, kgl, kx,                             &
                   px2, px2l,                                &
                   p2, qqal, q2, q2l, rl,                    &
                   sum, sumg
   integer :: j, ja, jjj, jz, j0

   !-----------------------------------------------------------------

   lambda2cdf = zero
   ier = 0

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero .or. p <= zero .or. q <= zero  &
      .or. delta >= one .or. delta <= eps ) then
      ier = 1
      return
   end if

   !  Case x = 0

   if ( x < eps ) return

   !  Case a2 = 0

   if ( a2 < eps ) then
      lambda2cdf = chi2cdf( p*x, p, dflimit, ier )
      if ( ier /= 0 ) ier = 4
      return
   end if

   !  If q is large enough use limiting distribution

   if ( q > qlimit ) then
      lambda2cdf = phi2cdf( x, p, a2, delta, maxitr, ier )
      if ( ier /= 0 ) ier = 4
      return
   end if

   !  General case (iterations)
   !  kx's are decreasing for all j's
   !  gl's are decreasing for j >= jjj = a2/2 - a2/q - 1
   !  kgj, sumg are only used for stopping rule
   !  Logs are used to avoid underflows

   err  = delta * half
   erp  = err / relerr
   p2   = p * half
   px2  = p2 * x
   px2l = log( px2 )
   q2   = q * half
   qqal = q2 * log( q/(q+a2) )
   aqal = log( a2/(q+a2) )
   q2l  = dlgama( q2 )
   jjj  = nint( a2*half - a2/q - half )
   jjj  = max( jjj, 0 )

   !  Examine rate of convergence:
   !  find starting index for iterations (j0)
   !  cinf is a limit under which sum of g coefficients is
   !  considered as negligible
   !  g0 is gl at j0

   j0 = 0
   g0 = qqal
   if ( jjj >= jmax ) then
      if ( g0 < log(cinf) ) then
         ga = qqal - q2l + dlgama(jjj+q2) - dlgama(jjj+one) + jjj*aqal
         if ( ga < explower )  goto 5   !  Cannot be computed
         if ( exp(ga) < cinf ) goto 5   !  Cannot be computed
         ja = 0
         jz = jjj
         ga = g0
         do
            j0 = (ja+jz) / 2
            g0 = qqal - q2l + dlgama(j0+q2) - dlgama(j0+one) + j0*aqal
            if ( g0 >= explower ) then
               if ( (j0+1)*exp(g0) < cinf ) then
                  ja = j0
                  ga = g0
               else
                  jz = j0
               end if
            else
               ja = j0
               ga = g0
            end if
            if ( ja+1 >= jz ) exit
         end do
         j0 = ja
         g0 = ga
      end if
   end if
   5 continue

   !  Initialize

   !  when j0 > 0, it is assumed sumg at j0 is negligible,
   !  and the upper bound is taken for kgj:
   !  kgj = sum( j*g*kx ) <= sum( j*g )  (as kx <= 1)
   !                      <= exp(g0)*sum( j )    (as j <= j0 < jjj)
   !                      <= exp(G0)*J0*(J0+1)/2

   dj = j0
   gl = g0
   kx = chi2cdf( p*x, p+dj+dj, dflimit, ier )
   if ( ier /= 0 ) then
      ier = 4
      return
   end if
   rl  = (p2+dj) * px2l - ( px2 + dlgama(p2+dj+one) )
   kgj = zero
   if ( j0 > 0 ) then
      bl = dlgama(q2+dj) - q2l - dlgama(dj+one)
      if ( g0 >= explower ) kgj = (dj+one) * dj * half * exp(g0)
   else
      bl = zero
   end if
   sum   = zero
   sumg  = zero

   !  Iteration loop

   do j = j0, j0+maxitr

      if ( kx > zero ) then
         kgl = log(kx) + gl
         if ( kgl >= explower ) then
            kgl = exp( kgl )
            sum = sum + kgl
            kgj = kgj + kgl*dj
            !  check loss of accuracy
            if ( kgj >= erp ) then
               ier = 3
               lambda2cdf = sum
               return
            end if
         end if
      end if

      lambda2cdf = sum

      !  Check accuracy (stopping rule)
      !  xp is used to prevent possible underflow

      if ( gl >= explower ) sumg = sumg + exp(gl)
      if ( kgj >= xp ) then
         if ( relerr*kgj+kx*(one-sumg) < err ) return
      else
         if (            kx*(one-sumg) < err ) return
      end if

      !  Prepare next iteration

      dj1   = j + 1
      bl    = bl + log( (q2+dj)/dj1 )
      gl    = bl + qqal + dj1*aqal
      if ( rl >= explower ) kx = kx - exp( rl )
      rl = rl - log(p2+dj1) + px2l
      dj = dj1
      !!*  autre solution: sans bl: recurrence "directe" sur gl
      !!*  inconvenient: si aqal tres faible devant les autres termes
      !!*  (tel qu'ayant contribution exactement 0), ca ne peux pas etre
      !!*  compense par dj1*aqal
      !**      dj1   = j + 1
      !**      gl    = gl + log( (q2+dj)/dj1 ) + aqal
      !**      if ( rl >= explower ) kx = kx - exp( rl )
      !**      rl = rl - log(p2+dj1) + px2l
      !**      dj = dj1

   end do

   !  Maximum number of iterations is reached

   ier = 2

end function lambda2cdf
