function phi2cdf( x, p, a2, delta, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Calculates the probability that a random variable distributed
!     according to the phi-square distribution with P degrees of
!     freedom and A2 eccentricity parameter, is less than or equal to X
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . Degrees of freedom             (P >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     phi2cdf (stopping criterion)
!                     (eps < DELTA < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then phi2cdf = zero)
!                     2 = maximum number of iterations reached
!                         (then phi2cdf = value at last iteration,
!                                      or zero)
!                     3 = required accuracy cannot be reached
!                         (then phi2cdf = value at last iteration)
!                     4 = error in auxiliary function (chi2cdf)
!
!     External functions called:
!       CHI2CDF
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  LOG  INT  (and DLGAMA if available)
!
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: phi2cdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, a2 , delta
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: chi2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: erraux=1.0e-14_8, dflimit=1.0e6_8
   real(kind=8), parameter :: eps=0.223e-15_8, tinyr=1.0e-307_8, explower=-706.893_8
   real(kind=8), parameter :: relerr=1.0e-14_8 ! Relative error assumed
                                               ! in recurrence calculations
   real(kind=8), parameter :: xp=tinyr/relerr

   !     erraux = relative error in chi2cdf function
   !     dflimit = limit for approximation in chi2cdf function
   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     tinyr = the smallest positive real
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))

   real(kind=8) :: a22, a22l, dj, erp, err, gl,  &
                   kgj, kgl, kx,                 &
                   px2, px2l,                    &
                   p2, rl,                       &
                   sum, sumg
   integer :: j

   !-----------------------------------------------------------------

   phi2cdf = zero
   ier = 0

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero .or. p <= zero     &
      .or. delta >= one .or. delta <= eps ) then
      ier = 1
      return
   end if

   !  Case x = 0

   if ( x < eps ) return

   !  Case a2 = 0

   if ( a2 < eps ) then
      phi2cdf = chi2cdf( p*x, p, dflimit, ier )
      if ( ier /= 0 ) then
         ier = 4
      else
         if ( phi2cdf*erraux > delta ) ier = 3
      end if
      return
   end if

   !  General case (iterations)
   !  kx's are decreasing for all j's
   !  gl's are decreasing for j > a2/2
   !  kgj, sumg are only used for stopping rule
   !  Logs are used to avoid underflows

   err  = delta * half
   erp  = err / relerr
   p2   = p * half
   px2  = p2 * x
   px2l = log( px2 )
   a22  = a2 * half
   a22l = log( a22 )

   !  Initialize

   gl = -a22
   kx = chi2cdf( p*x, p, dflimit, ier )
   if ( ier /= 0 ) then
      ier = 4
      return
   else if ( kx*erraux > delta ) then
      ier = 3
      return
   end if
   rl   = p2*px2l - ( px2 + dlgama(p2+one) )
   kgj  = zero
   sum  = zero
   sumg = zero
   dj   = zero

   !  Iteration loop

   do j = 0, maxitr

      if ( kx > zero ) then
         kgl = log(kx) + gl
         if ( kgl >= explower ) then
            kgl = exp( kgl )
            sum = sum + kgl
            kgj = kgj + kgl*dj
            !  check loss of accuracy
            if ( kgj >= erp ) then
               ier = 3
               phi2cdf = sum
               return
            end if
         end if
      end if

      phi2cdf = sum

      !  Check accuracy (stopping rule)
      !  xp is used to prevent possible underflow

      if ( gl >= explower ) sumg = sumg + exp(gl)
      if ( kgj >= xp ) then
         if ( relerr*kgj+kx*(one-sumg) < err ) return
      else
         if (            kx*(one-sumg) < err ) return
      end if

      !  Prepare next iteration

      dj = j + 1
      gl = gl + a22l - log(dj)
      if ( rl >= explower ) kx = kx - exp( rl )
      rl = rl - log(p2+dj) + px2l

   end do

   !  Maximum number of iterations is reached

   ier = 2

end function phi2cdf
