SUBROUTINE basort( n, m, metr1, a, c )

!***********************************************************************
!     - basort -
!     computes N-1 orthogonal (with respect to the unit or A**-1 metric)
!     contrasts of same norm in a N-dimensional space
!***********************************************************************
!
!     N     - INPUT . Number of dimensions                    - INTEGER
!     M     - INPUT . Number of rows used to dimension matrix - INTEGER
!                     C in calling module (must be >= N)
!     METR1 - INPUT . If .true. the unit metric is used,      - LOGICAL
!                     if .false. the A**(-1) metric is used
!     A     - INPUT . Unused if METR1 is .true.               - REAL*8
!                     Vector of reciprocal of the diagonal    - REAL*8
!                     terms of the metric-matrix, of length N
!     C     - OUTPUT. Matrix of contrast coefficients         - REAL*8
!                     2-dimensioned (M,max(N-1,1)). The N-1
!                     columns are the contrast-vectors.
!                     Note that
!                     if N = 1 then C(1,1) = 1,
!                     if N = 2 then C(1,1) = 1, C(2,1) = -1
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  SUM  SQRT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n, m
   LOGICAL, INTENT(IN) :: metr1
   REAL(KIND=8), INTENT(IN) :: a(n)
   REAL(KIND=8), INTENT(OUT) :: c(m,max(n-1,1))

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, two=2.0_8
   REAL(KIND=8) :: anorm, p, suma
   INTEGER :: i, i1

   !--------------------------------------------------------------------

   if ( n <= 2 ) then
      c(n,1) = -one
      c(1,1) = one
      return
   end if

   if ( metr1 ) then

      ! Use unit metric
      ! ---------------
      c(1,1) = one
      c(2,1) = -one
      c(3:n,1) = zero
      do i = 2, n-1
         p = i*(i+1)/2
         p = one / sqrt(p)
         c(1:i,i) = p
         c(i+1,i) = - i*p
         c(i+2:n,i) = zero
      end do

   else

      ! Use a**(-1) metric
      ! ------------------
         ! Compute the square of the norm of the contrasts (anorm)
      p = n
      suma  = zero
      anorm = zero
      do i = 1, n
         suma  = suma + a(i)
         anorm = anorm + a(i)*a(i)
      end do
      anorm = (anorm - suma*suma/p) / (p-one)
      anorm = two / (suma/p - anorm/suma)
         ! Compute the n-1 orthogonal contrasts
      do i = 1, n-1
         i1  = i + 1
         suma = sum( a(1:i) )
         p = suma/(suma+a(i1)) * a(i1) * anorm
         p = sqrt(p)
         c(1:i,i) = a(1:i)/suma * p
         c(i1,i) = -p
         c(i1+1:n,i) = zero
      end do

   end if

END SUBROUTINE basort
