SUBROUTINE mpower( xmat, pow, n, tolz, ierr )

!***********************************************************************
!     - mpower -
!     raises a symmetric matrix (positive or semi-positive definite)
!     to a real power (if the power is negative the matrix must be
!     positive definite, and semi-positive definite otherwise)
!***********************************************************************
!
!     XMAT  - INPUT . The concerned matrix, of order N        - REAL*8
!                     (in triangular form)
!             OUTPUT. Resultant matrix
!                     (in triangular form)
!     POW   - INPUT . The desired power (non zero)            - REAL*8
!     N     - INPUT . Order of XMAT                           - INTEGER
!     TOLZ  - INPUT . Zero-tolerance for eigenvalues          - REAL*8
!     IERR  - OUTPUT. 0 if no error                           - INTEGER
!                     <0 : the power desired was negative and
!                          XMAT was not positive definite:
!                          eigenvalues in DIAG(-IERR)..DIAG(N)
!                          are lower than or equal to zero
!                     >0 : error in diagonalization :
!                          eigenvalues in DIAG(IERR+1)..DIAG(N)
!                          (and corresponding eigenvectors)
!                          may be presumed accurate
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        TQL2X   (adapted from EISPACK library)
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SUM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: pow, tolz
   INTEGER, INTENT(IN) :: n
   REAL(KIND=8), INTENT(INOUT) :: xmat(n*(n+1)/2)
   INTEGER, INTENT(OUT) :: ierr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ij, irank, j
   REAL(KIND=8) :: diag(n), dumm(n), xvec(n,n)

   !--------------------------------------------------------------------

   ierr = 0

   if ( n == 1 ) then
      if ( xmat(1) > zero ) then
         xmat(1) = xmat(1)**pow
      else
         if ( pow <= zero ) ierr = -1
      end if
      return
   end if

   ! Diagonalize xmat
   ! ----------------
      ! Eigenvectors are returned in xvec, eigenvalues in diag
   call tql2x( n, xmat, diag, dumm, xvec, .true., ierr )
   if ( ierr /= 0 ) return

   ! Raise the matrix to given power ( xvec*(diag**pow)*xvec')
   ! -------------------------------
   irank = n
   do i = 1, n
      if ( diag(i) > tolz ) then
         diag(i) = diag(i)**pow
      else
            ! The input matrix is singular
         if ( pow > zero ) then
            irank = i - 1
            exit
         else
            ierr = -i
            return
         end if
      end if
   end do
   ij = 0
   do i = 1, n
      do j = 1, i
         sumx = sum( diag(1:irank) * xvec(i,1:irank) * xvec(j,1:irank) )
         ij = ij + 1
         xmat(ij) = sumx
      end do
   end do

END SUBROUTINE mpower
