      SUBROUTINE PLOTF ( FPLT  , NLDX  , IO    , X     , Y     , N     , &
                         YMIN  , YMAX  , NLYAX , MXABS , MARKP , FILLC , &
                         LOCAX )
!***********************************************************************
!     - PLOTF -
!     builds up the PLOT of the graph of a Function, (in "text" mode).
!     The plot is stored in an array, and printed if requested.
!     Abscissas correspond to the horizontal axis and ordinates to
!     the vertical axis.
!     Some values in abscissas are displayed.
!     For ordinates, extreme and middle points are displayed.
!***********************************************************************
!
!     FPLT  - OUTPUT. Array containing the plot, of minimal   - CHAR
!                     length NLYAX-LOCAX+3, here dimensioned
!                     as (LOCAX-2:NLYAX):
!                     FPLT(NLYAX) contains the upper line,
!                     corresponding to YMAX,
!                     FPLT(1) contains the line corresponding
!                     to YMIN,
!                     FPLT(LOCAX) contains the X-axis,
!                     FPLT(LOCAX-1) and possibly FPLT(LOCAX-2)
!                     contain displayed X-values, if any
!                     The needed width is NCY+1+N+NCX/2 (see
!                     parameter statement below for NCY & NCX)
!                     and it must be compatible with width of
!                     line-printer if printing is requested
!     NLDX  - OUTPUT. Number of lines used to display the     - INTEGER
!                     X-values: 0, 1 or 2 at most
!                     (remember these lines are stored under
!                     FPLT(LOCAX))
!     IO    - INPUT . Output unit number. If negative the     - INTEGER
!                     plot will be built and stored in FPLT
!                     but will not be printed
!     X     - INPUT . Vector of the abscissas, of length N    - REAL
!     Y     - INPUT . Vector of the ordinates, of length N    - REAL
!     N     - INPUT . Dimension of X and Y: number of points  - INTEGER
!                     to be plotted
!     YMIN  - INPUT . Minimum value to be taken on Y-axis     - REAL
!                     YMIN can be any value such that
!                     YMIN <= Min( Y(1),...,Y(N) )
!     YMAX  - INPUT . Maximum value to be taken on Y-axis     - REAL
!                     YMAX can be any value such that
!                     YMAX >= Max( Y(1),...,Y(N) )
!     NLYAX - INPUT . Number of lines to be used for Y-axis   - INTEGER
!     MXABS - INPUT . Maximum number of abscissa values to be - INTEGER
!                     printed under the X-axis.
!                     The actual number will be taken as the
!                     greatest divisor of N-1 that is
!                     <= Min( (N-1)/(NCX/2+1), MXABS-1, 9 )
!                     where NCX is number of printed digits
!                     (defined below in a parameter statement)
!                     If no such a divisor can be found, only
!                     extreme and middle points will be printed
!                     Thus a good choice would be to take N
!                     and MXABS such that MXABS-1 is a divisor
!                     of N-1  and  (N-1)/(MXABS-1) > NCX/2
!     MARKP - INPUT . The character to be used for the plot   - CHAR
!     FILLC - INPUT . If .true., the whole area between the   - LOGICAL
!                     curve and the X-axis will be filled
!                     with MARKP symbol
!                     If .false., only the curve-line will be
!                     marked by MARKP
!     LOCAX - INPUT . Location of the X-axis:                 - INTEGER
!                     0: the X-axis will be displayed under
!                        the line corresponding to YMIN
!                     1: the X-axis will correspond to YMIN
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        RFMTA  RFMTB
!     Fortran  functions called:
!        MIN  MOD  NINT  REAL
!
!-----------------------------------------------------------------------
!
!
!        ARGUMENTS
!        ---------
      REAL              YMIN  , YMAX
      INTEGER           NLDX  , IO    , N     , NLYAX , MXABS , LOCAX
      LOGICAL           FILLC
      CHARACTER*(*)     MARKP
      REAL              X (N) , Y (N)
      CHARACTER*(*)     FPLT (LOCAX-2:NLYAX)
!
!        LOCAL DECLARATIONS
!        ------------------
      CHARACTER*6       RFMTA , RFMTB
      REAL              A     , DELTA , ONE   , Y2    , ZERO  , ZERTOL
      INTEGER           I     , II    , J     , JAX   , JJ    , J1
      INTEGER           K     , K6    , M     , M0    , M2    , NCX
      INTEGER           NCX1  , NCX2  , NCY   , NCYX2 , NCY1  , NCY4
      INTEGER           NN    , NNN   , NX    , N2
      CHARACTER*121     FMTX
      CHARACTER*15      BLANK , FMTY
!
      PARAMETER ( NCX=13, NCX1=NCX+1, NCX2=NCX/2,  &
                  NCY=14, NCY1=NCY+1, NCY4=NCY-4, NCYX2=NCY-NCX2 )
!
!        NCX = width of printed X-values (7<NCX<15)
!        NCY = width of printed Y-values (7<NCY<15) -> format FMTY
!
      PARAMETER (ZERTOL=1.0E-35, ZERO=0.0E0, ONE=1.0E0)
!
!        ZERTOL = a tolerance for zero
!
      DATA  BLANK / '               ' /
      DATA  FMTY  / '(G14. 7,T15,A) ' /
!
!     ------------------------------------------------------------------
!
!
      JAX = LOCAX
      NN  = N
      NNN = NN + NCY1
!
!        Get scaling slope
      DELTA = YMAX - YMIN
      IF( DELTA .LE. ZERO ) DELTA = ONE
      A = REAL(NLYAX-1) / DELTA
!
!        Set FPLT to blank and put YMAX and (YMIN+YMAX)/2
      DO 10 I = 1, NLYAX
         FPLT(I) = BLANK(:NCY) // '|'
   10 CONTINUE
      Y2 = YMIN + (NLYAX/2)*DELTA/(NLYAX-1)
      FMTY(2:7) = RFMTB( NCY4, -4, Y2, ZERTOL )
      WRITE(FPLT(NLYAX/2+1)(:NCY1),FMTY) Y2, '+'
      FMTY(2:7) = RFMTB( NCY4, -4, YMAX, ZERTOL )
      WRITE(FPLT(NLYAX)(:NCY1),FMTY) YMAX, '+'
!
!     BUILD THE PLOT IN FPLT
!     ----------------------
      IF( FILLC )THEN
!           The whole area under the curve is to be filled in
         DO 30 J = 1, NN
            JJ = NCY1 + J
            I  = NINT( ONE+A*(Y(J)-YMIN) )
            DO 20 II = JAX+1, I
               FPLT(II)(JJ:JJ) = MARKP(1:1)
   20       CONTINUE
   30    CONTINUE
      else
!           Only the curve-line is to be marked
         DO 40 J = 1, NN
            JJ = NCY1 + J
            I  = NINT( ONE+A*(Y(J)-YMIN) )
            FPLT(I)(JJ:JJ) = MARKP(1:1)
   40    CONTINUE
      ENDIF
!        Prepare Line 1 and X-axis
      FPLT(JAX)(:NCY1+1) = BLANK(:NCY1) // '+'
      DO 50 J = NCY1+2, NN+NCY
         FPLT(JAX)(J:J) = '-'
   50 CONTINUE
      FPLT(JAX)(NNN:) = '+'
      FMTY(2:7) = RFMTB( NCY4, -4, YMIN, ZERTOL )
      WRITE(FPLT(1)(:NCY1),FMTY) YMIN, '+'
!
!     DISPLAY X-AXIS (=FPLT(JAX)) AND ABSCISSAS
!     -----------------------------------------
      M0 = MIN( MXABS-1, (N-1)/(NCX2+1), 9 )
      IF( MXABS .NE. 3 .AND. M0 .NE. 2 )THEN
!
         DO 60 I = M0, 1, -1
            M = I
            IF( MOD(N-1,M) .EQ. 0 ) GOTO 65
   60    CONTINUE
!              No X-value displayed
            NLDX = 0
            GOTO 100
   65    CONTINUE
         NX = (N-1) / M
         M2 = MOD( M+1, 2 )
         DO 70 J = 2+NCY+NX, NNN, NX
            FPLT(JAX)(J:J) = '+'
   70    CONTINUE
         NLDX = 1
         IF( NX .LE. NCX ) NLDX = 2
         DO 90 I = 1, NLDX
!              Get format
            DO 75 J = 1, (M+1)/NLDX+M2*(NLDX-1)/I
               WRITE(FMTX(12*J-11:12*J-6),222)   &
                  2+NCYX2+NX*(I-NLDX-1+NLDX*J)
   75       CONTINUE
            K = -5
            DO 80 J = NX*I-NX+1, N, NX*NLDX
               K = K + 12
               FMTX(K:K+5) = RFMTA( NCX, -4, X(J), ZERTOL )
   80       CONTINUE
            K6 = K + 6
            FMTX(NLDX+1-NLDX:1) = '('
            FMTX(K6:K6)         = ')'
!              Get line
            FPLT(JAX-I) = ' '
            J1 = NX*(I-1) + 1
            WRITE(FPLT(JAX-I),FMTX(:K6)) (X(J),J=J1,N,NX*NLDX)
   90    CONTINUE
!
      else
!           Only 2 or 3 X-values displayed
!           Display X-axis and X(1), X(NN), X(NN/2)
         WRITE(FMTX(1:24),333) NCYX2+3
         FMTX(  7:12)  = RFMTA( NCX, -4, X(1), ZERTOL )
         FMTX( 25:120) = '    '
         FMTX(121:121) = ')'
         NLDX = 2
         FPLT(JAX-1) = ' '
         IF( NN .LT. NCX )THEN
            WRITE(FPLT(JAX-1),FMTX) X(1)
            WRITE(FMTX(1:24),333) NCYX2+2+NN
            FMTX( 7:12) = RFMTA( NCX, -4, X(NN), ZERTOL )
            FPLT(JAX-2) = ' '
            WRITE(FPLT(JAX-2),FMTX) X(NN)
         else
            N2 = NN/2 + 1
            FPLT(JAX)(NCY1+N2:NCY1+N2) = '+'
            IF( NCX .LE. NN-NCX-1 )THEN
               NLDX = 1
               WRITE(FMTX(13:18),222) NCYX2+2+N2
               FMTX(19:24) = RFMTA( NCX, -4, X(NN/2), ZERTOL )
               WRITE(FMTX(25:30),222) NCYX2+2+NN
               FMTX(31:36) = RFMTA( NCX, -4, X(NN), ZERTOL )
               WRITE(FPLT(JAX-1),FMTX) X(1), X(N2), X(NN)
            else
               WRITE(FMTX(13:18),222) NCYX2+2+NN
               FMTX(19:24) = RFMTA( NCX, -4, X(NN), ZERTOL )
               WRITE(FPLT(JAX-1),FMTX) X(1), X(NN)
               WRITE(FMTX(1:24),333) NCYX2+2+N2
               FMTX( 7:12) = RFMTA( NCX, -4, X(NN/2), ZERTOL )
               FPLT(JAX-2) = ' '
               WRITE(FPLT(JAX-2),FMTX) X(N2)
            ENDIF
         ENDIF
!
      ENDIF
!
!     PRINT PLOT
!     ----------
  100 CONTINUE
      IF( IO .GE. 0 )THEN
         NNN = NNN + NCX2
         DO 110 I = NLYAX, JAX-NLDX, -1
            WRITE(IO,111) FPLT(I)(:NNN)
  110    CONTINUE
      ENDIF
!
!
  111 FORMAT(1X,A)
  222 FORMAT(',T',I3,',')
  333 FORMAT('(T',I3,',                  ')
!
      END
