SUBROUTINE prtsym( iout, lchk, sym, nrow, namv, lstd, txt, ntxt )

!***********************************************************************
!     - prtsym -
!     PRinTs the lower triangle of a SYMmetrical matrix, with possible
!     standardization by diagonal elements (e.g. a variance-covariance
!     matrix or the corresponding correlation matrix)
!***********************************************************************
!
!     IOUT  - INPUT . Output unit number                      - INTEGER
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!     SYM   - INPUT . Symmetrical matrix of order NROW,       - REAL*8
!                     stored in triangular form
!     NROW  - INPUT . Order of SYM (must be > 1)              - INTEGER
!     NAMV  - INPUT . Names of rows/columns in SYM            - CHAR
!     LSTD  - INPUT . If .false. SYM is printed               - LOGICAL
!                     If .true. the standardized matrix is
!                     printed:
!                     SYM(i,j) / SQRT(SYM(i,i)*SYM(j,j))
!     TXT   - INPUT . A title printed at top of the matrix    - CHAR
!     NTXT  - INPUT . Length of TXT (at most the n first      - INTEGER
!                     characters will be underlined, where
!                     n is parameter NUNDL below)
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        DFMTC  DIVID0
!     Fortran  functions called:
!        ABS  MAXVAL  MIN  SQRT  TINY
!
!-----------------------------------------------------------------------

   USE gmpint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iout, nrow, ntxt
   LOGICAL, INTENT(IN) :: lchk, lstd
   REAL(KIND=8), INTENT(IN) :: sym(nrow*(nrow+1)/2)
   CHARACTER(LEN=*), INTENT(IN) :: txt
   CHARACTER(LEN=*), INTENT(IN) :: namv(nrow)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: divid0
   CHARACTER(LEN=6), EXTERNAL :: dfmtc

   REAL(KIND=8) :: xxx
   INTEGER, PARAMETER :: nundl=50
   INTEGER :: i, l, linpp, ll, lll, llll, lt
   INTEGER :: maxil, nbeg, nend, nl, nladd, nlast, nlr
   LOGICAL :: newp
   CHARACTER(LEN=nundl) :: undln="=================================================="
   CHARACTER(LEN=16) :: fmt="(1x,a8,20      )"

   !--------------------------------------------------------------------

   if ( lchk ) then
      call chkpg( 6, .false., newp )
      linpp = linep
   else
      linpp = 99999
   end if
   write(iout,111) txt(:ntxt), undln(:min(ntxt,nundl))
   lines = lines + 3

      ! If not standardized matrix, get nice format
      ! (if standardization holds, the maximum value is assumed to be 1.)
   if ( .not. lstd ) then
         ! First find maximum absolute value
      xxx = maxval( abs(sym) )
         ! Get format (with provision for possible minus sign)
      if ( xxx > 1.0e-20_8 ) then
         fmt(10:15) = dfmtc( 9, -5, -xxx, 1.0e-30_8 )
      else
         fmt(10:15) = dfmtc( 9, -5, -xxx, tiny(xxx) )
      end if
   end if

      ! maxil is maximum number of items printed per line
      ! (9 characters are reserved for printing of values,
      ! see formats fmt and 333)
   maxil = (lrecp-(1+8)) / 9
   nl    = nrow
   lll   = 0
   nend  = 0

   do      ! Loop while nend < nrow
      nbeg = nend + 1
      nend = min( nend+maxil, nrow )

         ! nlr   = no. of lines still to be printed
         ! nladd = no. of lines in current page
      nlr = nl
      ll  = lll
      lt  = ll*(ll+1)/2
      do      ! Loop while nlr > 0
         write(iout,222)  (namv(i),i=nbeg,nend)
         lines = lines + 1
         nladd = min( nlr, linpp-lines )
         do l = 1, nladd
            ll    = ll + 1
            llll  = ll*(ll+1) / 2
            nlast = min( nend, ll )
            if ( lstd ) then
               write(iout,333) namv(ll),   &
                  (divid0( sym(lt+i), sqrt(sym(llll)*sym(i*(i+1)/2)),9.9999_8 ), i=nbeg,nlast)
            else
               write(iout,fmt) namv(ll), (sym(lt+i),i=nbeg,nlast)
            end if
            lt = lt + ll
         end do
         lines = lines + nladd
         nlr   = nlr   - nladd
         if ( nlr <= 0 ) exit
         if ( lchk ) call chkpg( 0, .true., newp )
         write(iout,333) '  '
         lines = lines + 1
      end do

      nl  = nl  - maxil
      lll = lll + maxil
      if ( nend >= nrow ) exit
      if ( lchk ) call chkpg( 4, .false., newp )
      write(iout,333) '  '
      lines = lines + 1
   end do

   111 format('  '/(1x,a))
   222 format(9x,20(1x,a8))
   333 format(1x,a8,20f9.4)

END SUBROUTINE prtsym
