SUBROUTINE fmt_type( fmt, nitem, type )

!***********************************************************************
!     - fmt_type -
!     identifies the type (character, integer or real) of each item to
!     be associated with a given Fortran format
!     (Caution: this is a very crude analysis of fmt; in particular,
!     no syntax analysis is provided)
!***********************************************************************
!
!     fmt   - INPUT . Fortran format specification            - CHAR
!                     (supposed enclosed in parentheses)
!     nitem - INPUT . Number of items associated with fmt     - INTEGER
!                     (nitem > 0)
!     type  - OUTPUT. Array of type (of length nitem):        - INTEGER
!                     0 = undefined (error)
!                     1 = character type
!                     2 = integer type
!                     3 = real type
!
!-----------------------------------------------------------------------
!
!     Subroutines called :
!        COMPRS  CVUCT  INUMBER  REPLA1
!     External functions called :
!        NONE
!     Fortran  functions called :
!        INDEX  LEN_TRIM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nitem
   CHARACTER(LEN=*), INTENT(IN) :: fmt
   INTEGER, INTENT(OUT) :: type(nitem)

   !  Local declarations
   !  ------------------

   INTEGER, PARAMETER :: maxlvl=7   ! Maximum parenthesis level in format

   INTEGER :: i, item, j, k, lfmt, n, plvl
   LOGICAL :: grp, pend
   CHARACTER(LEN=LEN_TRIM(fmt)) :: xfmt
   INTEGER :: g1st(maxlvl), glst(maxlvl), nrep(maxlvl)

   !--------------------------------------------------------------------

   type = 0   ! Undefined
   lfmt = len_trim(fmt)
   if ( lfmt < 3 .or. nitem <= 0 ) return
   if ( fmt(1:1) /= "(" .or. fmt(lfmt:lfmt) /= ")" ) return

   xfmt = fmt(2:lfmt-1)
   call cvuct( xfmt )

      ! If no character nor integer specification, analysis is simplified
   if ( index(xfmt,"A") == 0 .and. index(xfmt,"I") == 0 ) then
!!    if ( index(xfmt,"F") > 0 .or. index(xfmt,"G") > 0 .or. index(xfmt,"E") > 0 .or. index(xfmt,"D") > 0 )  &
      type = 3   ! Real type assumed
      return
   end if

   call repla1( "/", ",", xfmt, .true. )
   call repla1( "T", ",", xfmt, .true. )
   call repla1( "X", ",", xfmt, .true. )
   call repla1( ".", ",", xfmt, .true. )
   call comprs( xfmt, " ", ",", i )
   lfmt = len_trim(xfmt)
   if ( lfmt < 2 ) return

   plvl = 0   ! Parenthesis level
   nrep = 1   ! Repetition factor
   g1st(1) = 1   ! Index of the 1st item of the repetition group
      ! N.B. The repetition group (if any) begins at the last "(" with level 1

   grp = .false.  ! Flag for at least 1 parenthesis group
   pend = .false. ! Flag for repetition is pending
   item = 0   ! Index of current item
   n = 1      ! Repetition factor for a single item
   i = 0      ! Position of character in xfmt

   do                  ! Loop on characters in xfmt
      if ( item >= nitem ) return
      i = i + 1
      if ( i > lfmt ) exit

      select case( xfmt(i:i) )
         case ( "," )
            pend = .false.
            n = 1
         case ( "(" )
            grp = .true.
            plvl = plvl + 1
            if ( plvl > maxlvl ) return   ! Error: terminate
            g1st(plvl) = item + 1
            if ( pend ) then   ! The group has a repetition factor
               nrep(plvl) = n
               pend = .false.
            end if
            n = 1
         case ( ")" )
            if ( plvl < 1 ) return        ! Error: terminate
            glst(plvl) = item
            do k = 2, nrep(plvl)
               do j = g1st(plvl), glst(plvl)
                  item = item + 1
                  type(item) = type(j)
                  if ( item >= nitem ) return
               end do
            end do
            plvl = plvl - 1
            pend = .false.
            n = 1
         case ( "0":"9" )
            call inumber( xfmt(i:), j, n )
            i = i + j - 1
            pend = .true.
         case ( "A" )
            do j = 1, n
               item = item + 1
               type(item) = 1
               if ( item >= nitem ) return
            end do
            pend = .false.
            n = 1
         case ( "I" )
            do j = 1, n
               item = item + 1
               type(item) = 2
               if ( item >= nitem ) return
            end do
            pend = .false.
            n = 1
         case ( "D":"G" )
            do j = 1, n
               item = item + 1
               type(item) = 3
               if ( item >= nitem ) return
            end do
            pend = .false.
            n = 1
      end select

   end do

   if ( plvl /= 0 ) return       ! Error

   if ( item < nitem ) then      ! More items than format specifications
      if ( .not. grp ) then
         g1st(1) = 1
         glst(1) = item
      end if
      do
         do j = g1st(1), glst(1)
            item = item + 1
            type(item) = type(j)
            if ( item >= nitem ) return
         end do
      end do
   end if

END SUBROUTINE fmt_type
