REAL(KIND=8) FUNCTION foper( kodop, x, y, xvar, nvar, kase, nof, nerd )

!***********************************************************************
!     - foper -
!     executes operator number kodop on x (and y if relevant) for the
!     current case and current formula
!***********************************************************************
!
!     KODOP - INPUT . Operator code number                    - INTEGER
!     X     - INPUT . First  operand                          - REAL*8
!     Y     - INPUT . Second operand                          - REAL*8
!     XVAR  - INPUT . Vector of variables for current case    - REAL*8
!     NVAR  - INPUT . Length of XVAR                          - INTEGER
!     KASE  - INPUT . Sequential case number                  - INTEGER
!     NOF   - INPUT . Sequential formula number               - INTEGER
!     NERD  - INPUT . Number of errors already detected       - INTEGER
!             OUTPUT. Updated value
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE  (DERF  DERFC  DLGAMA if not in Fortran library)
!     Fortran  functions called:
!        many of them
!
!-----------------------------------------------------------------------

   USE opprm
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: kodop, nvar, kase, nof
   REAL(KIND=8), INTENT(IN) :: x, y
   REAL(KIND=8), DIMENSION(nvar), INTENT(IN) :: xvar
   INTEGER, INTENT(INOUT) :: nerd

   !  Local declarations
   !  ------------------

!! REAL(KIND=8), EXTERNAL :: derf               ! Error function
!! REAL(KIND=8), EXTERNAL :: derfc              ! Error function (complement)
!! REAL(KIND=8), EXTERNAL :: dgamma             ! gamma(x)
!! REAL(KIND=8), EXTERNAL :: dlgama             ! Log(gamma(x))
   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8), PARAMETER :: twopi=6.2831853071795865_8
   INTEGER, PARAMETER :: mxmsg=10 ! Maximum number of error messages printed
   REAL(KIND=8) :: xx, yy
   INTEGER :: k

   INCLUDE 'foperp.'

   !--------------------------------------------------------------------


   ! Note that xmaxim, rather than xout, is used here as the
   ! out-of-range value because scanning for missing or out-of-range
   ! values is not yet performed when processing transformations

   foper = xmaxim
      ! Check the operand is within range
   if ( x >= xmaxim .and. kodop /= itxop ) return

   select case (kodop)

      ! Logical operators
      ! -----------------

      case ( 8)         !... eqv
         if ( y < xmaxim ) then
            xx = min( one, max( zero, x) )
            yy = min( one, max( zero, y) )
            if ( xx == yy ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case ( 9)         !... neqv
         if ( y < xmaxim ) then
            xx = min( one, max( zero, x) )
            yy = min( one, max( zero, y) )
            if ( xx /= yy ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case (10)         !... or
         if ( y < xmaxim ) then
            xx = min( one, max( zero, x) )
            yy = min( one, max( zero, y) )
            foper = min( one, xx+yy )
         end if

      case (11)         !... and
         if ( y < xmaxim ) then
            xx = min( one, max( zero, x) )
            yy = min( one, max( zero, y) )
            foper = xx * yy
         end if

      case (1,12)       !... not
         foper = one - min( one, max( zero, x) )

      case (2,13)       !... gt
         if ( y < xmaxim ) then
            if ( x > y ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case (3,14)       !... ge
         if ( y < xmaxim ) then
            if ( x >= y ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case (4,15)       !... lt
         if ( y < xmaxim ) then
            if ( x < y ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case (5,16)       !... le
         if ( y < xmaxim ) then
            if ( x <= y ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case (6,17)       !... eq
         if ( y < xmaxim ) then
            if ( x == y ) then
               foper = one
            else
               foper = zero
            end if
         end if

      case (7,18)       !... ne
         if ( y < xmaxim ) then
            if ( x /= y ) then
               foper = one
            else
               foper = zero
            end if
         end if

      ! Algebraic operators
      ! -------------------

      case (19)         !... addition
         if ( y < xmaxim ) foper = x + y

      case (20)         !... substraction
         if ( y < xmaxim ) foper = x - y

      case (21)         !... division
         if ( y < xmaxim ) then
            if ( abs(y) >= xminim ) then
               foper = x / y
            else
               nerd = nerd + 1
               if ( nerd <= mxmsg ) then
                  if ( nerd == 1 ) write(io,*) ewmrk1
                  write(io,111) kase, nof, zdiv
               end if
            end if
         end if

      case (22)         !... product
         if ( y < xmaxim ) foper = x * y

      case (23)         !... power
         if ( y < xmaxim ) then
            if ( mod(y,one) <= xminim ) then
               foper = x ** nint(y)
            else
               foper = x ** y
            end if
         end if

      case (24)         !... square root
         if ( x >= zero ) then
            foper = sqrt (x)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, zsqr, x
            end if
         end if

      case (25)         !... natural logarithm
         if ( x > zero ) then
            foper = log (x)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, zlogn, x
            end if
         end if

      case (26)         !... decimal logarithm
         if ( x > zero ) then
            foper = log10 (x)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, zlogd, x
            end if
         end if

      case (27)         !... exponential
         if ( explow <= x .and. x <= expupp ) then
            foper = exp (x)
         else
            if ( x < explow ) foper = zero
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, zexp, x
            end if
         end if

      case (28)         !... absolute value
         foper = abs (x)

      case (29)         !... integer part
         foper = int (x)

      case (30)         !... nearest integer
         foper = nint (x)

      case (31)         !... sign
         if ( y < xmaxim ) foper = sign (x, y)

      case (32)         !... modulo
         if ( y < xmaxim ) then
            if ( abs(y) >= xminim ) then
               foper = mod (x, y)
            else
               nerd = nerd + 1
               if ( nerd <= mxmsg ) then
                  if ( nerd == 1 ) write(io,*) ewmrk1
                  write(io,111) kase, nof, zmod, y
               end if
            end if
         end if

      case (33)         !... minimum
         if ( y < xmaxim ) foper = min (x, y)

      case (34)         !... maximum
         if ( y < xmaxim ) foper = max (x, y)

      case (35)         !... sine
         foper = sin (mod(x,twopi))

      case (36)         !... hyperbolic sine
         if ( abs(x) <= hsincos ) then
            foper = sinh (x)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, 'sinh', x
            end if
         end if

      case (37)         !... cosine
         foper = cos (mod(x,twopi))

      case (38)         !... hyperbolic cosine
         if ( abs(x) <= hsincos ) then
            foper = cosh (x)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, 'cosh', x
            end if
         end if

      case (39)         !... tangent
         foper = tan (mod(x,twopi))

      case (40)         !... hyperbolic tangent
         foper = tanh (x)

      case (41)         !... cotangent
         foper = cotan (mod(x,twopi)) ! Lahey
         ! If not part of the Fortran intrinsic functions library:
      !! if ( abs(tan(x)) >= xminim ) then
      !!    foper = one / tan (x)
      !! else
      !!    nerd = nerd + 1
      !!    if ( nerd <= mxmsg ) then
      !!       if ( nerd == 1 ) write(io,*) ewmrk1
      !!       write(io,111) kase, nof, zdiv
      !!    end if
      !! end if

      case (42)         !... arc sine
         foper = asin (x)

      case (43)         !... arc cosine
         foper = acos (x)

      case (44)         !... arc tangent
         foper = atan (x)

      case (45)         !... arc tangent "2"
         if ( y < xmaxim ) then
            if ( abs(y) >= xminim ) then
               foper = atan2 (x, y)
            else
               nerd = nerd + 1
               if ( nerd <= mxmsg ) then
                  if ( nerd == 1 ) write(io,*) ewmrk1
                  write(io,111) kase, nof, 'atan2', y
               end if
            end if
         end if

      case (46)         !... gamma function
         if ( gamlow <= x .and. x <= gamupp ) then
            foper = gamma (x)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, 'gamma', x
            end if
         end if

      case (47)         !... log(gamma function)
         foper = dlgama (x)

      case (48)         !... "error function" on (zero,x)
         foper = derf (x)

      case (49)         !... "error function" on (x,infinity)
         foper = derfc (x)

      case (50)         !... positive difference
         if ( y < xmaxim ) foper = dim (x, y)

      case (51)         !... convert radians to degrees
         foper = x * 57.2957795130823229_8

      case (52)         !... convert degrees to radians
         foper = x * 0.174532925199432946e-01_8

      case (53)         !... assign the value of a variable given its index
         ! x is the index of the variable, and so is k
         ! (note that an epsilon is added in ordrer to avoid such things
         ! as int(0.9999999999...) = 0)
         k = int (x + 1.0e-10_8)
         if ( 0 < k .and. k <= nvar ) then
            foper = xvar(k)
         else
            nerd = nerd + 1
            if ( nerd <= mxmsg ) then
               if ( nerd == 1 ) write(io,*) ewmrk1
               write(io,111) kase, nof, zind, k
            end if
         end if

      ! Sign operators (unary + -) and text operator
      ! --------------------------------------------

      case (54,56)      !... unary + (code number nbreop)
         foper = x      !    and text operator (code number itxop)
                        !    act as an assignement

      case (55)         !... unary - (code number nbreop-1)
         foper = -x


      case default
         nerd = nerd + 1
         if ( nerd <= mxmsg ) then
            if ( nerd == 1 ) write(io,*) ewmrk1
            write(io,111) kase, nof
            write(io,222) kodop
         end if

   end select

   if ( nerd > 0 ) write(io,*) ewmrk2

   INCLUDE 'foperf.'

END FUNCTION foper
