SUBROUTINE transa( car, com, lastk, sep, to, ncto, by, ncby, nomvar, nvar, nvart, nvatp, lerr )

!***********************************************************************
!     - transa -
!     analyzes "transformations": identifies statements (algebraic
!     formulas, if-blocks, do-loops) and determines the sequence of
!     executions
!***********************************************************************
!
!     CAR   - INPUT . Characters string containing the        - CHAR
!                     transformations (with delimiting
!                     apostrophs removed)
!             OUTPUT. Some characters modified
!     COM   - INPUT . A logical vector such that COM(I) is    - LOGICAL
!                     .true. if CAR(I:I) was inbeded in
!                     apostrophs, and .false. otherwise
!     LASTK - INPUT . Length of CAR and COM                   - INTEGER
!     SEP   - INPUT . End-of-formula character                - CHAR
!     TO    - INPUT . Separator between initial and test      - CHAR
!                     values of a DO-loop
!     NCTO  - INPUT . Number of significant characters in TO  - INTEGER
!     BY    - INPUT . Separator between test value and        - CHAR
!                     increment of a DO-loop
!     NCBY  - INPUT . Number of significant characters in BY  - INTEGER
!     NOMVAR- INPUT . Names of variables (assumed at most     - CHAR
!                     NOCW-byte long, see local variables
!                     DOSTR and DOINC)
!             OUTPUT. Updated list if temporary variables
!     NVAR  - INPUT . Number of variables                     - INTEGER
!     NVART - INPUT . Dimension of NOMVAR and LONVAR          - INTEGER
!                     If NVART > NVAR, then the difference
!                     is the maximum number of temporary
!                     variables that can be used during the
!                     transformations
!     NVATP - OUTPUT. NVAR + number of temporary variables    - INTEGER
!                     used
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements explicitly modified:
!     trancb : nform and arrays
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        FORMEV  TRANSM  VARID3
!     External functions called:
!        NONE
!     Fortran  functions called:
!        INDEX  LEN  MIN
!
!-----------------------------------------------------------------------

   USE gmpint   ! For parameters nocw and lgu
   USE oplhn
   USE opprm
   USE trancb

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lastk, ncto, ncby, nvar, nvart
   CHARACTER(LEN=*), INTENT(IN) :: to, by
   CHARACTER(LEN=1), INTENT(IN) :: sep
   LOGICAL, DIMENSION(lastk), INTENT(IN) :: com
   CHARACTER(LEN=*), DIMENSION(nvart), INTENT(INOUT) :: nomvar
   CHARACTER(LEN=*), INTENT(INOUT) :: car
   INTEGER, INTENT(OUT) :: nvatp
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: foper   ! The function executing operations

   INTEGER, PARAMETER :: maxlvl=5, siztp=8, sizdo=nocw+siztp+siztp+18, siznc=nocw+siztp+2

   ! maxlvl = maximum level of if-block or do-loop nesting
   ! sizdo  = size of character string for storage of
   !          loop-test formula and dimension of associated
   !          logical array docom
   ! siznc  = size of character string for storage of
   !          do-variable incrementation formula
   ! N.B. Names of variables are assumed at most nocw-byte long
   !      Be sure format 222 corresponds to a siztp-byte long name

   REAL(KIND=8) :: resul
   INTEGER :: i, iform, ipile, j, kol, koleq, kolf, kolm, kolt, kol3
   INTEGER :: lenv, lvldo, lvlif, ndo, nelse, nendf, nendo, nif, nstat, numv
   LOGICAL :: lelse
   CHARACTER(LEN=sizdo) :: dostr
   CHARACTER(LEN=siznc) :: doinc
   CHARACTER(LEN=siztp) :: ctemp
   INTEGER, DIMENSION(maxlvl) :: doif, ifdo
   INTEGER, DIMENSION(nvart) :: lonvar
   INTEGER, DIMENSION(nvart+nvart) :: iwrkb
   LOGICAL, DIMENSION(sizdo) :: docom=(/ (.false.,i=1,sizdo) /)
   INCLUDE 'transap.'

   ! text are error messages

   !--------------------------------------------------------------------

   ! DO-LOOPS analysis
   ! For DO-loops processing, 5 formulas are added:
   ! - the  DO_ i = a > b : c;  statement is replaced by:
   !   i=a;                       (initialize DO-variable)
   !   xxx=b;                     (create test-value)
   !   yyy=c;                     (create increment-value)
   !   (i-xxx)*SIGN(1,yyy)_LE_0;  (loop-test)
   !   this last formula is treated as a condition: the do-loop is
   !   continued if true and is halted otherwise
   ! - the  corresponding  ENDDO  statement is replaced by:
   !   i=i+yyy;                   (increment DO-variable)
   !   and the next statement to be executed will be the loop-test
   ! xxx and yyy are auxiliary variable names, respectively
   ! '&&&&do'//2n-1 and '&&&&do'//2n, where n is DO-nesting-level
   ! of the DO-block
   ! These auxiliary variables are used, rather than "b" and "c",
   ! to avoid possible modification within the do-loop
   ! Note that no test for zero increment is actually performed
   !
   ! DOIF(I) is IF-nesting-level of a DO-loop  of DO-nesting-level I
   ! IFDO(I) is DO-nesting-level of a block-IF of IF-nesting-level I
   !
   ! NEXT1 contains, for every formula:
   ! - if the formula is not a condition: the index of next formula
   !   to be executed
   ! - if the formula is a condition    : the index of next formula
   !   to be executed when the condition is true
   ! NEXT2 contains, for every formula:
   ! - if the formula is not a condition: the index of the involved
   !   variable (the one on left side of "=" sign)
   ! - if the formula is a condition    : the index of next formula
   !   to be executed when the condition is false
   !
   ! PILIF and PILEL are 2 stacks used to build up NEXT1 and NEXT2:
   ! For a given IF(A)THEN... sequence, PILIF contains the index of
   ! formula "A", and PILEL contains the index of the formula
   ! preceding the "ELSE" (or 0 if there is no "ELSE")
   ! LCOND is a flag, set to .TRUE. when the formula is a condition
   ! and set to .FALSE. otherwise
   ! For a given DO_... sequence, PILIF contains the index of the
   ! loop-test formula, and PILEL contains the number of the
   ! DO-variable
   ! For the end-of-loop condition formula generated by the DO_
   ! statement, LCOND is set to .TRUE.

   !--------------------------------------------------------------------

   lerr = .false.

   ! Compute lonvar (no. of significant characters of nomvar elements)
   ! -----------------------------------------------------------------
   lonvar = LEN_TRIM( nomvar )

   !  NSTAT  is a counter for statements
   !  NFORM  is a counter for formulas (in TRANCB )
   !  NIF    is a counter for "IF()THEN"
   !  NENDF  is a counter for "ENDIF"
   !  NELSE  is a counter for "ELSE"
   !  LELSE  checks no two consecutive "ELSE"
   !  LVLIF  codes the level of IF-blocks
   !  NDO    is a counter for "DO_"
   !  NENDO  is a counter for "ENDDO"
   !  LVLDO  codes the level of DO-blocks
   !  IPILE  is a pointer for PILIF and PILEL stacks
   !  KOL    is the first column of the zone scanned
   !  TRANSM is the routine printing error message

   lenv  = len( nomvar(1) )
   dostr(1:1)     = '('
   dostr(lenv+2:) = '-        )*SIGN(1,        )_LE_0' // sep
   nvatp = nvar
   nstat = 0
   nform = 0
   nif   = 0
   nendf = 0
   nelse = 0
   lvlif = 0
   lelse = .false.
   ndo   = 0
   nendo = 0
   lvldo = 0
   ipile = 0
   kol   = 1

   ! Loop on the zone analyzed
   ! -------------------------
   do
      nstat = nstat + 1

      if ( car(kol:kol) == sep .or. car(kol:kol) == ' ' ) then
            ! It's a separator or a blank (skip it)
            ! ---------------------------
         kol = kol + 1

      else

         kol3 = kol + 3
         kolm = min( kol+4, lastk )
         if ( lastk < kol3 ) then
            call transm( io, ewmrk1, ewmrk2, text( 4), stmnt, nstat, car(kol:lastk) )
            lerr = .true.
            return
         end if

         if ( car(kol:kol+2) == 'IF(' .and. .not. com(kol) ) then
               ! It's a condition ( IF(...)THEN )
               ! --------------------------------
            nif   = nif + 1
            lvlif = lvlif + 1
            nform = nform + 1
            ipile = ipile + 1
            kolt  = index( car(kol3:lastk), ')THEN' ) + kol3-1
            if ( kolt <= kol3 ) then
               call transm( io, ewmrk1, ewmrk2, text( 5), 'if()then', nif, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( nform > nbrefo ) then
               call transm( io, ewmrk1, ewmrk2, text( 1), stmnt, nstat, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( lvlif > maxlvl ) then
               call transm( io, ewmrk1, ewmrk2, text( 9), 'if()then', nif, car(kol:lastk) )
               lerr = .true.
               return
            end if
            lcond(nform) = .true.
            next1(nform) = nform + 1
            pilif(ipile) = nform
            pilel(ipile) = 0
            ifdo(lvlif)  = lvldo
            lelse = .false.
            kol   = kolt + 5
            car(kolt:kolt) = sep
               ! Analyze formula
            call formev( foper, nvatp, car(kol3:kolt), com(kol3:kolt), &
                         nomvar, lonvar, xtra(-1:,nform),              &
                         adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
            if ( lerr ) return
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
            car(kolt:kolt) = ')'

         else if ( car(kol:kol+3) == 'ELSE' .and. .not. com(kol) ) then
               ! It's an "ELSE"
               ! --------------
            nelse = nelse + 1
            if ( lelse ) then
               call transm( io, ewmrk1, ewmrk2, text( 6), 'else', nelse, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( ipile <= 0 ) then
               call transm( io, ewmrk1, ewmrk2, text( 7), 'else', nelse, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( lvldo /= ifdo(lvlif) ) then
               call transm( io, ewmrk1, ewmrk2, text(10), 'else', nelse, car(kol:lastk) )
               lerr = .true.
               return
            end if
            iform        = pilif(ipile)
            next2(iform) = nform + 1
            pilel(ipile) = nform
            lelse = .true.
            kol   = kol + 4

         else if ( lastk >= kol+4 .and. car(kol:kolm) == 'ENDIF' .and. .not. com(kol) ) then
               ! It's an "ENDIF"
               ! ---------------
            nendf = nendf + 1
            if ( ipile <= 0 ) then
               call transm( io, ewmrk1, ewmrk2, text( 8), 'endif', nendf, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( lvldo /= ifdo(lvlif) ) then
               call transm( io, ewmrk1, ewmrk2, text(10), 'endif', nendf, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( pilel (ipile) > 0 ) then
                  ! Update branch to skip the ELSE block
               iform        = pilel(ipile)
               next1(iform) = nform + 1
            else
                  ! Update branch when the condition is false
               iform        = pilif(ipile)
               next2(iform) = nform + 1
            end if
            lelse = .false.
            ipile = ipile - 1
            lvlif = lvlif - 1
            kol   = kol + 5

         else if ( car(kol:kol+2) == 'DO_' .and. .not. com(kol) ) then
               ! It's a doo-loop ( DO_... )
               ! --------------------------
            ndo   = ndo + 1
            lvldo = lvldo + 1
            nform = nform + 1
            ipile = ipile + 1
            kolf  = index( car(kol3:lastk), sep ) + kol3-1
            if ( kolf <= kol3 ) then
               call transm( io, ewmrk1, ewmrk2, text( 3), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( nform+4 > nbrefo ) then
               call transm( io, ewmrk1, ewmrk2, text( 1), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( lvldo > maxlvl ) then
               call transm( io, ewmrk1, ewmrk2, text( 9), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
               ! Get the number of the do-variable (numv)
            koleq = index( car(kol3:kolf), '=' ) + kol3-1
            if ( koleq <= kol3 ) then
               call transm( io, ewmrk1, ewmrk2, text( 2), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            call varid3( nvar, nvart, nvatp, nomvar, lonvar, car(kol3:koleq-1), koleq-kol3, numv, lerr )
            if ( lerr ) return
            if ( numv <= nvar ) then
               call transm( io, ewmrk1, ewmrk2, text(12), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
               ! ...and store it
            pilel(ipile) = numv
               ! Do-variable initialization
            i = index( car(koleq+1:kolf), to(:ncto) ) + koleq
            if ( i <= koleq+1 ) then
               call transm( io, ewmrk1, ewmrk2, text(11), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            car(i:i)     = sep
            lcond(nform) = .false.
            next1(nform) = nform + 1
            next2(nform) = numv
            call formev( foper, nvatp, car(koleq+1:i), com(koleq+1:i), &
                         nomvar, lonvar, xtra(-1:,nform),              &
                         adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
            if ( lerr ) return
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
            car(i:i)    = to(1:1)
               ! Generate and analyze test-value assignement
            nform = nform + 1
            i     = i + ncto
            j     = index( car(i:kolf), by(:ncby) ) + i-1
            if ( j == i .or. j+ncby == kolf .or. i == kolf ) then
               call transm( io, ewmrk1, ewmrk2, text(11), 'do_', ndo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            write(ctemp,222) lvldo+lvldo-1
            call varid3( nvar, nvart, nvatp, nomvar, lonvar, ctemp, siztp, numv, lerr )
            if ( lerr ) return
            if ( j < i ) then
               j = kolf
            else
               car(j:j) = sep
            end if
            lcond(nform) = .false.
            next1(nform) = nform + 1
            next2(nform) = numv
            call formev( foper, nvatp, car(i:j), com(i:j),       &
                         nomvar, lonvar, xtra(-1:,nform),        &
                         adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
            if ( lerr ) return
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
               ! Generate and analyze increment value assignement
            nform = nform + 1
            write(ctemp,222) lvldo+lvldo
            call varid3( nvar, nvart, nvatp, nomvar, lonvar, ctemp, siztp, numv, lerr )
            if ( lerr ) return
            lcond(nform) = .false.
            next1(nform) = nform + 1
            next2(nform) = numv
            if ( j < kolf ) then
               car(j:j) = by(1:1)
               call formev( foper, nvatp, car(j+1:kolf), com(j+1:kolf),  &
                            nomvar, lonvar, xtra(-1:,nform),             &
                            adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
               if ( lerr ) return
            else
               ctemp = '1' // sep
               call formev( foper, nvatp, ctemp(:2), docom,         &
                            nomvar, lonvar, xtra(-1:,nform),        &
                            adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
               if ( lerr ) return
            end if
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
               ! Generate and analyze loop-test formula
            nform = nform + 1
            lcond(nform) = .true.
            next1(nform) = nform + 1
            pilif(ipile) = nform
            doif(lvldo)  = lvlif
            dostr(2:lenv+1) = nomvar(pilel(ipile))
            j = lenv + siztp
            write(dostr(lenv+3:j+2       ),222) lvldo+lvldo-1
            write(dostr(j+12  :j+siztp+11),222) lvldo+lvldo
            call formev( foper, nvatp, dostr, docom,             &
                         nomvar, lonvar, xtra(-1:,nform),        &
                         adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
            if ( lerr ) return
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
            kol = kolf + 1

         else if ( lastk >= kol+4 .and. car(kol:kolm) == 'ENDDO' .and. .not. com(kol) ) then
               ! It's an "ENDDO"
               ! ---------------
            nendo = nendo + 1
            nform = nform + 1
            if ( ipile <= 0 ) then
               call transm( io, ewmrk1, ewmrk2, text(13), 'enddo', nendo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( nform > nbrefo ) then
               call transm( io, ewmrk1, ewmrk2, text( 1), 'enddo', nendo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( lvlif /= doif(lvldo) ) then
               call transm( io, ewmrk1, ewmrk2, text(10), 'enddo', nendo, car(kol:lastk) )
               lerr = .true.
               return
            end if
            iform        = pilif(ipile)
            numv         = pilel(ipile)
            lcond(nform) = .false.
            next1(nform) = iform
            next2(nform) = numv
               ! Update end of do-loop location
            next2(iform) = nform + 1
               ! Generate and analyze incrementation formula
            doinc = nomvar(numv) // '+'
            write(doinc(lenv+2:siznc),222) lvldo+lvldo, sep
            call formev( foper, nvatp, doinc, docom,             &
                         nomvar, lonvar, xtra(-1:,nform),        &
                         adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
            if ( lerr ) return
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
            ipile = ipile - 1
            lvldo = lvldo - 1
            kol   = kolm + 1

         else
               ! It's an algebraic formula (well, it should be...)
               ! -------------------------
            nform = nform + 1
               ! Find "=" sign
            koleq = index( car(kol:lastk), '=' ) + kol-1
            if ( koleq <= kol ) then
               call transm( io, ewmrk1, ewmrk2, text( 2), stmnt, nstat, car(kol:lastk) )
               lerr = .true.
               return
            end if
               ! See if variable operator on left hand side of "=" sign
            if ( car(kol:kol+lonop(ivrop)-1) == nomop(ivrop)(:lonop(ivrop)) ) then
                  ! Analyze the formula for the variable index
               if ( nform > nbrefo ) then
                  call transm( io, ewmrk1, ewmrk2, text( 1), stmnt, nstat, car(kol:lastk) )
                  lerr = .true.
                  return
               end if
               lcond(nform) = .false.
               next1(nform) = nform + 1
               next2(nform) = -1
               i = kol + lonop(ivrop)
               car(koleq:koleq) = sep
               call formev( foper, nvatp, car(i:koleq), com(i:koleq),  &
                            nomvar, lonvar, xtra(-1:,nform),           &
                            adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
               if ( lerr ) return
               nato(nform) = natom
               nope(nform) = noper
               nvaf(nform) = nvarf
               numv  = 0
               nform = nform + 1
            else
                  ! Find the number of the variable (numv)
               call varid3( nvar, nvart, nvatp, nomvar, lonvar, car(kol:koleq-1), koleq-kol, numv, lerr )
               if ( lerr ) return
            end if
               ! Find the end of formula on right hand side of "=" sign
            kolf = index( car(koleq+1:lastk), sep ) + koleq
            if ( kolf <= koleq ) then
               call transm( io, ewmrk1, ewmrk2, text( 3), stmnt, nstat, car(kol:lastk) )
               lerr = .true.
               return
            end if
            if ( nform > nbrefo ) then
               call transm( io, ewmrk1, ewmrk2, text( 1), stmnt, nstat, car(kol:lastk) )
               lerr = .true.
               return
            end if
            lcond(nform) = .false.
            next1(nform) = nform + 1
            next2(nform) = numv
               ! Analyze formula
            call formev( foper, nvatp, car(koleq+1:kolf), com(koleq+1:kolf),  &
                         nomvar, lonvar, xtra(-1:,nform),                     &
                         adr(1:,1:,nform), adrv(1:,1:,nform), resul, iwrkb, lerr )
            if ( lerr ) return
            nato(nform) = natom
            nope(nform) = noper
            nvaf(nform) = nvarf
            kol = kolf + 1

         end if

      end if
         ! Go back to analyze next element, starting from column kol
      if ( kol > lastk ) exit
   end do

   ! End of process

   if ( ipile /= 0 .or. lelse .or. lvldo /= 0 .or. lvlif /= 0 ) then
      call transm( io, ewmrk1, ewmrk2, text(14), stmnt, nstat, car(kol:lastk) )
      lerr = .true.
      return
   end if

   if ( nvatp > nvar ) then
      write(lgu,111) nvatp-nvar, (nomvar(nvar+i),i=1,nvatp-nvar)
   end if

   INCLUDE 'transaf.'

END SUBROUTINE transa
