SUBROUTINE fannp( effct, scal, df1, df2, direct, xlim, plev,    &
                  prob, fmin, dff, kode, xneg, xnot, iapp, ierr )
!***********************************************************************
!     - fannp -
!     Frequentist Approach to Negligible/Notable Problem
!***********************************************************************
!
!     EFFCT - INPUT . Effect (or magnitude of a multivariate  - REAL*8
!                     effect)
!     SCAL  - INPUT . Scale index of associated Bayesian      - REAL*8
!                     distribution
!     DF1   - INPUT . Df of the comparison                    - REAL*8
!     DF2   - INPUT . Df of the error term                    - REAL*8
!     DIRECT- INPUT . .true. if the problem is direct:        - LOGICAL
!                     find the probability for a given limit
!                     .false. otherwise:
!                     find both negligible and notable limits
!                     given a probability
!     XLIM  - INPUT . The limit entered                       - REAL*8
!                     Irrelevant if DIRECT is .false.
!     PLEV  - INPUT . The probability level entered           - REAL*8
!                     Irrelevant if DIRECT is .true.
!     PROB  - OUTPUT. The probability returned                - REAL*8
!                     Irrelevant if DIRECT is .false.
!     FMIN  - OUTPUT. F-statistic associated to PROB          - REAL*8
!                     Irrelevant if DIRECT is .false.
!     DFF   - OUTPUT. Numerator df corresponding to FMIN      - REAL*8
!                     Irrelevant if DIRECT is .false.
!     KODE  - OUTPUT. Type of response to XLIM                - INTEGER
!                       0 : negligible statement
!                       1 : notable statement
!                     Irrelevant if DIRECT is .false.
!     XNEG  - OUTPUT. Negligible limit returned               - REAL*8
!                     Irrelevant if DIRECT is .true.
!     XNOT  - OUTPUT. Notable limit returned                  - REAL*8
!                     Irrelevant if DIRECT is .true.
!     IAPP  - OUTPUT. Approximation code:                     - INTEGER
!                       0 : no approximation used
!                       1 : approximation used
!     IERR  - OUTPUT. Error code:                             - INTEGER
!                     0 if no error, otherwise the error code
!                     returned by betacdf or ksi2i function
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        BETACDF  KSI2I
!     Fortran  functions called:
!        ABS  MAX  SQRT
!
!-----------------------------------------------------------------------

   USE pacdtr

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: effct, scal, df1, df2, xlim, plev
   LOGICAL, INTENT(IN) :: direct
   REAL(KIND=8), INTENT(OUT) :: prob, fmin, dff, xneg, xnot
   INTEGER, INTENT(OUT) :: kode, iapp, ierr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: betacdf, ksi2i
   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8
   REAL(KIND=8) :: effa, test, xlima
   INTEGER, PARAMETER :: mxit=2000
   LOGICAL :: hpl

   !--------------------------------------------------------------------

   ierr = 0
   iapp = 0
   effa = abs( effct )

   if ( direct ) then

         ! Find probability, given a limit
      xlima = abs( xlim )
      fmin  = ( (xlima-effa) / scal )**2
      if ( xlima >= effa ) then
         kode = 0
         dff  = one
      else
         kode = 1
         dff  = df1
      end if
      test = df1*fmin / (df1*fmin+df2)
      prob = betacdf( test, half*dff, half*df2, ierr )
      if ( ierr == 0 ) then
         prob = one - prob
         if ( kode == 0 ) prob = half * prob
      else
         prob = -one
      end if

   else

         ! Find negligible limit
         ! (The t distribution is obtained via the ksi-square one)
      hpl  = plev < half
      test = plev + plev - one
      if ( hpl ) test = -test
      xneg = ksi2i( test, one, df2, zero, delta, mxit, iapp, ierr )
      if ( ierr == 0 ) then
         if ( hpl ) xneg = -xneg
         xneg = effa + scal*sqrt(xneg/df1)
      else
         xneg = -one
      end if
         ! Find notable limit
      xnot = ksi2i( plev, df1, df2, zero, delta, mxit, iapp, ierr )
      if ( ierr == 0 ) then
         xnot = effa - scal*sqrt(xnot/df1)
      else
         xnot = -one
      end if

   end if

END SUBROUTINE fannp
