SUBROUTINE idntfy( part, npart, ng, nt, gag, tat, type, ngpar, k1, pwgh,  &
                   ntpar, nvp, nwp, i1, gpar, tpar )
!***********************************************************************
!     - idntfy -
!     identifies the type of a comparison with no interaction sign
!***********************************************************************
!
!     PART  - INPUT . Partition-matrix, 2-dimensioned NT, NG  - INTEGER
!     NPART - INPUT . Number of parts in the partition        - INTEGER
!     NG    - INPUT . Number of columns (groups)  in PART     - INTEGER
!     NT    - INPUT . Number of rows (treatments) in PART     - INTEGER
!     GAG   - INPUT . Vector of group weights of length NG    - REAL*8
!     TAT   - INPUT . Vector of group weights of length NT    - REAL*8
!     TYPE  - OUTPUT. Type of the comparison:                 - INTEGER
!                        1 = W/G1 type    (product form)
!                        2 = V/T1 type    (product form)
!                        3 = V.W  type    (product form)
!                        4 = general type (non product form)
!     NGPAR - OUTPUT. Number of columns (groups)  in PART     - INTEGER
!                     with at least one non-zero cell
!     K1    - OUTPUT. Index of first group with at least one  - INTEGER
!                     non-zero coefficient
!     PWGH  - OUTPUT. Vector of part-weights, of length NPART - REAL*8
!                     Irrelevant if type is not 4
!                     --- N.B. ALL FOLLOWING ARGUMENTS
!                     --- ARE IRRELEVANT IF TYPE IS 4
!     NTPAR - OUTPUT. Number of rows (treatments) in PART     - INTEGER
!                     with at least one non-zero cell
!     NVP   - OUTPUT. Number of parts in the partition GPAR   - INTEGER
!     NWP   - OUTPUT. Number of parts in the partition TPAR   - INTEGER
!     I1    - OUTPUT. Index of first treatment with non-zero  - INTEGER
!                     coefficient for group K1
!     GPAR  - OUTPUT. The marginal partition on groups,       - INTEGER
!                     of length NG
!     TPAR  - OUTPUT. The marginal partition on treatments,   - INTEGER
!                     of length NT
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: npart, ng, nt
   INTEGER, INTENT(IN) :: part(nt,ng)
   REAL(KIND=8), INTENT(IN) :: gag(ng), tat(nt)
   INTEGER, INTENT(OUT) :: type, ngpar, k1
   INTEGER, INTENT(OUT) :: ntpar, nvp, nwp, i1
   INTEGER, INTENT(OUT) :: gpar(ng), tpar(nt)
   REAL(KIND=8), INTENT(OUT) :: pwgh(npart)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, delta=1.0e-10_8
                                        ! delta is a tolerance for zero
   REAL(KIND=8) :: p
   INTEGER :: difa, diff, i, ip1, ip2, j, k, pik
   LOGICAL :: no0, no01, sama, samd, samt

   !--------------------------------------------------------------------
   !  Here we are only concerned with comparison formulas with no
   !  interaction sign, so that the only possible V.W type of
   !  interaction is implicit 1 df type (e.g.: g1t1_g2t2,g1t2_g2t1)
   !
   !  A necessary (but not sufficient) condition for a comparison
   !  to be product form is that the partition matrix may be obtained
   !  by the product of 2 marginal partitions, thus exhibiting the
   !  following structure:
   !  all occurrences of a same part-number form "rectangles"
   !  (up to rows and/or columns reordering) such that:
   !  1) given any non-zero cell, all rectangles crossed by the
   !  corresponding row have same height (number of rows) and
   !  all rectangles crossed by the corresponding column have same
   !  width (number of columns);
   !  2) given any zero cell, either its corresponding row or column
   !  (or both) is entirely made up of zeroes. For example:
   !        . . . . . . . . . .
   !        . 4 4 4 . 2 . . 2 .
   !        . 4 4 4 . 2 . . 2 .
   !        . . . . . . . . . .
   !        . 3 3 3 . 1 . . 1 .
   !        . 3 3 3 . 1 . . 1 .
   !        . . . . . . . . . .
   !        . 4 4 4 . 2 . . 2 .
   !        . . . . . . . . . .
   !
   !  To identify such a property of the matrix, it is sufficient
   !  to verify that, for all n and m, indices of non-zero columns,
   !  and for all i's such that Part(i,n) and Part(i,m) are not
   !  simultaneously zero:
   !  Part(i,n) - Part(i,m) = Knm   where Knm is a constant for the
   !  given pair of columns.
   !  (Obviously this is symmetric for rows.)
   !  Moreover:
   !  . if all non-zero numbers in a non-zero row are identical,
   !  then type is 1
   !  . if all non-zero numbers in a non-zero column are identical,
   !  then type is 2
   !  . otherwise type is 4 (as explicit interactions are excluded)
   !
   !  The difference property is not a necessary condition:
   !  it's not met when 1 df V.W interactions (type 3) are implicit
   !  (developped), e.g.: g1t1_g2t2,g1t2_g2t1
   !  To identify such cases then, in the above algorithm absolute
   !  difference replaces signed difference and constants Knm are
   !  restricted to be 0 or 1.
   !  Furthermore, columns of the weighted partition matrix (each cell
   !  weighted by its part-weight) must be proportional.
   !  If these two conditions hold type is 3 and is 4 otherwise.

   ! Find the 1st non-zero group
   ! ---------------------------
   no0   = .false.
   samt  = .true.
   nvp   = 1
   nwp   = 1
   ngpar = 1
   ntpar = 0
   k1    = 0
   do
      k1 = k1 + 1
      gpar(k1) = 0
      do i = 1, nt
         tpar(i) = part(i,k1)
         if ( part(i,k1) > 0 ) then
            if ( tpar(i) > nwp ) nwp = tpar(i)
            ntpar = ntpar + 1
            if ( no0 ) then
               if ( part(i,k1) /= pik ) samt = .false.
            else
               no0 = .true.
               i1  = i
               pik = part(i,k1)
               gpar(k1) = pik
               if ( gpar(k1) > nvp ) nvp = gpar(k1)
            end if
         end if
      end do
      if ( no0 ) exit
   end do

   if ( ng == 1 ) then
      type    = 1
      nvp     = 1
      gpar(1) = 1
      return
   end if

   ! Proceede on next groups
   ! -----------------------
      ! samd checks the column constant difference property
      ! sama checks the column constant absolute difference property
   samd = .true.
   sama = .true.
   do k = k1+1, ng
      no0  = .false.
      no01 = .false.
      gpar(k) = part(i1,k)
      if ( gpar(k) > nvp ) nvp = gpar(k)

      do i = 1, nt
         if ( part(i,k) > 0 ) then
            if ( part(i,k1) == 0 ) goto 10
               pik = part(i,k1) - part(i,k)
               if ( no0 ) then
                  if ( pik      /= diff ) samd = .false.
                  if ( abs(pik) /= difa ) sama = .false.
                  samt  = samt .and. part(i1,k) == part(i,k)
               else
                  no0   = .true.
                  ngpar = ngpar + 1
                  diff  = part(i,k1) - part(i,k)
                  difa  = abs(diff)
                  if ( difa > 1 ) sama = .false.
               end if
         else
            if ( part(i,k1) > 0 ) no01 = .true.
         end if
      end do

      if ( no0 .and. no01 ) goto 10
   end do

         ! If we go here, it means the non-zero cells in the
         ! partition matrix have a rectangular shape, so that
         ! (i1,k1) is the 1st non-zero cell.
         ! Also note that gpar(.) = part(i1,.) and tpar(.) = part(.,k1)

   if ( samd ) then

      ! Complete identification
      ! -----------------------
      if ( samt .or. nt == 1 ) then
         type = 2      ! V/T1 type
         nwp  = 1
         where ( tpar(i1:nt) > 1 ) tpar(i1:nt) = 1
      else
            ! W/G1 or general type
         pik = part(i1,k1)
         do k = k1+1, ng
            if ( part(i1,k) > 0 .and. part(i1,k) /= pik ) goto 10
         end do
         type = 1      ! W/G1 type
         nvp  = 1
         where ( gpar(k1:ng) > 1 ) gpar(k1:ng) = 1
      end if

   else

         ! Remember the only possible V.W interactions are 1 df ones,
         ! thus involving exactly 2 parts
      if ( .not. sama .or. npart /= 2 ) goto 10

      ! Type may be V.W or general, check proportionality property
      ! ----------------------------------------------------------
         ! First compute part-weights
      pwgh = zero
      do k = k1, ng
         do i = i1, nt
            j = part(i,k)
            if ( j > 0 ) pwgh(j) = pwgh(j) + gag(k)*tat(i)
         end do
      end do
         ! Check proportionality
      pik = part(i1,k1)
      do k = k1+1, ng
         if ( part(i1,k) > 0 ) then
            ip2 = part(i1,k)
            p   = pwgh(pik) / pwgh(ip2)
            do i = i1+1, nt
               ip1 = part(i,k1)
               if ( ip1 > 0 ) then
                  ip2 = part(i,k)
                  if ( abs(pwgh(ip1)/pwgh(ip2)-p) > delta ) then
                     type = 4      ! General 1 df type
                     return
                  end if
               end if
            end do
         end if
      end do
      type = 3      ! 1 df V.W type

   end if

   return

10 continue

   ! Type is general, compute part-weights
   ! -------------------------------------
   type = 4
   pwgh = zero
   do k = k1, ng
      do i = 1, nt
         j = part(i,k)
         if ( j > 0 ) pwgh(j) = pwgh(j) + gag(k)*tat(i)
      end do
   end do

END SUBROUTINE idntfy
