SUBROUTINE pacdeg( form, lf, kol, rtyp, kdeg, helm, intcp, prop )

!***********************************************************************
!     - pacdeg -
!     returns the DEGree and type of a polynomial regression requested
!     in a comparison-formula, if any, or a flag if an Helmert's
!     contrast is requested
!***********************************************************************
!
!     FORM  - INPUT . The formula currently processed         - CHAR
!     LF    - INPUT . Length of FORM                          - INTEGER
!     KOL   - INPUT . The column in FORM currently pointed to - INTEGER
!             OUTPUT. The column in FORM following the keyword
!                     for regression (and the degree if any),
!                     unchanged if no regression request
!     RTYP  - OUTPUT. The requested type of regression        - INTEGER
!                     Absolute value of RTYP may be:
!                     0 = no regression requested
!                     1 = degree type (KDEG only, thus 1 df)
!                         (including case of proportionality).
!                         Keyword is KWGEN(1) possibly followed
!                         by degree (if no degree specified,
!                         maximum degree will be assumed)
!                         or KWPRO for proportionality.
!                         ALIAS(0:3) below, are aliases for
!                         DEG0, DEG1, DEG2, DEG3
!                     2 = cumulative type (cumulative degrees
!                         up to KDEG, thus KDEG df).
!                         Keyword is KWGEN(2) possibly followed
!                         by degree (if no degree specified,
!                         maximum degree will be assumed)
!                     A negative type codes for the residual
!                     comparison
!     KDEG  - OUTPUT. The degree requested                    - INTEGER
!                     Any negative value will indicate that
!                     no explicit degree was mentioned, thus
!                     that maximum degree is assumed
!     HELM  - OUTPUT. .true. if an Helmert's contrast is      - LOGICAL
!                     requested, .false. otherwise
!     INTCP - OUTPUT. .true. if intercept is requested,       - LOGICAL
!                     .false. otherwise.
!                     When intercept is requested, KDEG
!                     returns 0 and RTYP returns 1 or -1
!     PROP  - OUTPUT. .true. if proportionality is requested, - LOGICAL
!                     .false. otherwise.
!                     When proportionality is requested, KDEG
!                     returns 1 and RTYP returns 1 or -1
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CVUCT
!     External functions called:
!        NONE
!     Fortran  functions called:
!        IACHAR
!
!-----------------------------------------------------------------------

   USE is_digit
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lf
   CHARACTER(LEN=*), INTENT(IN) :: form
   INTEGER, INTENT(INOUT) :: kol
   INTEGER, INTENT(OUT) :: rtyp, kdeg
   LOGICAL, INTENT(OUT) :: helm, intcp, prop

   !  Local declarations
   !  ------------------

   INTEGER :: k, k3, sgn
   CHARACTER(LEN=3) :: fupp
   INCLUDE 'pacdegp.'

   !--------------------------------------------------------------------

   helm  = .false.
   intcp = .false.
   prop  = .false.
   kdeg  = 0
   rtyp  = 0
   sgn   = 1
      ! Check for a residual comparaison
      ! (denoted by a minus sign preceeding the keyword)
   if ( form(kol:kol) == '-' ) then
      sgn = -1
      kol = kol + 1
   end if
   k3 = kol + 3
   if ( lf < k3 ) return
   k  = kol
      ! Convert first 3 characters to upper case
   fupp = form(k:k+2)
   call cvuct( fupp(1:3) )
   kol  = k3

   ! Scan for Helmert's contrast
   ! ---------------------------
   if ( fupp == kwhel ) then
      helm = .true.
      return
   end if

   rtyp = sgn

   ! Scan for special degree
   ! -----------------------
   if ( fupp == kwint ) then
      intcp = .true.
      kdeg  = 0
      return
   end if
   if ( fupp == kwpro ) then
      prop  = .true.
      kdeg  = 1
      return
   end if

   ! Scan for degree aliases
   ! -----------------------
   if ( fupp == alias(0) ) then
      kdeg = 0
      return
   end if
   if ( fupp == alias(1) ) then
      kdeg = 1
      return
   end if
   if ( fupp == alias(2) ) then
      kdeg = 2
      return
   end if
   if ( fupp == alias(3) ) then
      kdeg = 3
      return
   end if

   ! Scan for generic keyword
   ! ------------------------
   if ( fupp /= kwgen(1) ) then
      if ( fupp == kwgen(2) ) then
         rtyp = sgn + sgn
      else        ! No regression requested
         rtyp = 0
         kol  = k
         return
      end if
   end if

   ! Get the degree
   ! --------------
   k = iachar( form(kol:kol) )
   if ( .not. ldigit(k) ) then
      kdeg = -9999
      return
   end if
   kdeg = k - krnk0
   kol  = kol + 1
   do
      k = iachar( form(kol:kol) )
      if ( ldigit(k) ) then
         kdeg = 10*kdeg + k - krnk0
         kol  = kol + 1
         if ( kol <= lf ) cycle
      end if
      exit
   end do
      ! Note. By convention +/-Reg 0 is equivalent to +/-Deg 0
      ! but no specific statement such as
      !      if ( kdeg == 0 ) rtyp = sgn
      ! is needed because this property is implied by the algorithms used

END SUBROUTINE pacdeg
