SUBROUTINE pacdir( card, lcmdl, lcmd1, lprt, lfltr, kstop )

!***********************************************************************
!     - pacdir -
!     directs reading of paragraphs specific to program PAC and calls
!     to computational subroutines
!***********************************************************************
!
!     CARD  - INPUT . A record on LEC, already read if LCMD1  - CHAR
!                     is .true.
!             OUTPUT. Last record read on LEC
!     LCMDL - INPUT . .true. if a complete set of commands    - LOGICAL
!                     has already been read (in CAR)
!             OUTPUT. Idem for the next set of commands
!     LCMD1 - INPUT . .true. if the 1st record from a set of  - LOGICAL
!                     commands has already been read (in CARD)
!             OUTPUT. .false.
!     LPRT  - INPUT . If .true. command records read are      - LOGICAL
!                     printed
!     LFLTR - INPUT . If .true. commands are checked for      - LOGICAL
!                     unused or unrecognized paragraphs
!     kstop - OUTPUT. Error code, also  indicating if the     - INTEGER
!                     program is to be stopped immediately on
!                     return from this subroutine.
!                      0 = no error
!                     >0 = no error, stop execution
!                     <0 = error, stop execution
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car  ncar
!     gmpint : lines
!     memst2 : vec1
!     pacint : nuvar  ncomp  nszu
!     paclgl : lrwdb  lderiv
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CVUCS  GMPADS  GMPCMD  GMPCOM  GMP999  LOCAL  PACBAY
!        PACCOM  PACFOR  PACLID  PACLVO  PACPLA  PACX  PACY  PAC999
!     External functions called:
!        NAMAB
!     Fortran  functions called:
!        ALL  ALLOCATE  ALLOCATED  DEALLOCATE  LEN  LEN_TRIM  MAX  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst2
   USE memsta
   USE memstb
   USE pacchr
   USE pacint
   USE paclgl
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   LOGICAL, INTENT(IN) :: lprt, lfltr
   LOGICAL, INTENT(INOUT) :: lcmdl, lcmd1
   CHARACTER(LEN=*), INTENT(INOUT) :: card
   INTEGER, INTENT(OUT) :: kstop

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: namab

   REAL(KIND=8), PARAMETER :: one=1.0_8
   INTEGER, PARAMETER :: maxou=9, maxrc=10000
   CHARACTER(LEN=72), PARAMETER :: blank='                                                                        '
   CHARACTER(LEN=1), PARAMETER :: bsymc='&', bsymp='P'
      !  bsymc and bsymp are comparison and problem symbols used
      !  in Bayes save file

   REAL(KIND=8) :: baylev
   INTEGER :: cmptyp, i, ikend, ioc, irep, iulid, j, k, ka, kard, karp, karn, kar1
   INTEGER :: kb, kdeg, kompn, komp1, kn, k1, k3, last, maxr
   INTEGER :: ndf1, nend, nladd, noldc, noldf
   INTEGER :: noutlv, npart, nrco, nrrd, nrsavd, nwold, rtyp, sizusv, status
   LOGICAL :: allgt, balang, balant, lablg, lablt, lcomp
   LOGICAL :: leof, lerb, lerp, lerr, lerx, lform
   LOGICAL :: loldc, loldf, lonce, lplan, lprtb, lprti
   LOGICAL :: newp, newpb, none, planb, planc, pnew, prompt, prvldp, prvpla, samew
   CHARACTER(LEN=32) :: lidfil
   CHARACTER(LEN=maxcf) :: oldcnd, oldfor

   REAL(KIND=8) :: rcoef(maxrc)
   REAL(KIND=8), POINTER :: gweight(:), tweight(:)
   INTEGER :: loca(10)
   INTEGER, ALLOCATABLE :: part(:,:), usv(:)
   CHARACTER(LEN=1) :: gsym(26), tsym(26)
   CHARACTER(LEN=8) :: outlev(maxou)
   CHARACTER(LEN=nocw) :: templab(nguser)

   INCLUDE 'pacdirp.'

   !--------------------------------------------------------------------

   !     Some local elements of importance:
   !
   !  GSYM   is a character array containing the letters symbolizing
   !         the G-factors
   !  TSYM   is same for T-factors
   !  LPRTB  and LPRTI flag a command printing request, along with
   !         run-mode (respectively batch and interactive)
   !  LPLAN  flags a PLAN paragraph was ever entered
   !  PLANB  flags the first formula analyzed for a given PLAN paragraph
   !  PLANC  flags a PLAN paragraph is entered in current batch of
   !         paragraphs (thus overridding previous one if any)
   !  PRVPLA flags the PLAN paragraph is same as for previous comparison
   !         and that comparison was product form
   !  LABLG  is .true. if G-labels have been entered by user through
   !         a PLAN paragraph
   !  LABLT  is identical to LABLG but for T-labels
   !  NPART  is the number of parts in the G*T partition matrix (PART)
   !         which resulted from formula analysis (0 codes that matrix
   !         is not available: whether because of an entry by vectors or
   !         a nested comparison)
   !  ALLGT  is .true. when no cell in the groups*treatments cross-table
   !         design is empty
   !  LONCE  is .true. when covariates are constant over treatments and
   !         given only once, and thus are replicated over treatments
   !  LIDFIL is the name of a save file used as input by Eyelid program
   !         and is given by user, if desired, in PLAN paragraph
   !  IULID  is the unit number assigned to the above Eyelid save file
   !  LCOMP  flags a COMP paragraph was entered within a batch of
   !         paragraphs
   !  SAMEW  flags some results from previous comparison can be used;
   !         this is the case when the following conditions on current
   !         and previous comparisons hold:
   !           . they refer to the same design
   !           . they are both product form
   !           . they have same G-comparison-support
   !           . they have same W-contrast
   !  LFORM  flags an entry by formula in the COMP paragraph
   !  CMPTYP is code-number for type of comparison if entered by formula
   !  LOLDC  flags a condition was specified in previous formula
   !  OLDCND stores the formula-substring corresponding to the condtion
   !  NOLDC  is the length of the relevant substring in OLDCND
   !  LOLDF, OLDFOR, NOLDF are analogous to LOLDC, OLDCND, NOLDC but
   !         for the unconditional part (left part) of the formula
   !  RCOEF  is a vector for regression coefficients, possibly entered
   !         by user in COMP paragraph
   !  MAXRC  is the maximum length of this vector
   !  BAYLEV is Bayes-probability-level entered in COMP paragraph
   !  OUTLEV is vector of desired level of outputs on derived protocol,
   !         and is entered in COMP paragraph
   !  MAXOU  is its maximal length
   !  NOUTLV is the number of entries in OUTLEV

   !--------------------------------------------------------------------

   kstop = 0

      ! maxr is maximum record length of input command file
   maxr = len( card )

      ! newpb flags new problem and is used to control
      ! problem-title writing on bayes save file
   newpb = .true.

      ! Copy gmpint elements to avoid double argument transmission
      ! with module when calling gmpcmd
   nend = nfin
   ikend = ikfin

   ! Get rid of previous selection of variables in PLAN (if any)
   ! -----------------------------------------------------------
   if ( allocated(usev) ) deallocate(usev)
   if ( allocated(nomv) ) deallocate(nomv)

   ! Initialize
   ! ----------
      ! N.B. Some module elements were initialized in pacipb

      ! Set outlev to default according to lbref (in gmplgl)
   if ( lbref ) then
      outlev(1) = kwbrf
   else
      outlev(1) = kwnrm
   end if
   noutlv = 1
      ! Set Bayes-probability level to default value
   baylev = 0.90_8

      ! Default option for the subset of variables used (same subset as vuse)
      ! Actual length of usev will be known in pacpla, thus usv is used
      ! as temporary storage and usev will be allocated and filled in in pacpla
   nuvar = nvaru
   sizusv = 100*nvaru   ! Take provision
   if ( allocated(usv) ) deallocate(usv)
   allocate ( usv(sizusv), stat=status )
   if ( status /= 0 ) then   ! fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacDir)."
      write(lgu,*) ermrk2
      call gmp999            ! release memory space and files
      call pac999( 99 )
      stop
   end if
   do i = 1, nvaru
      usv(i) = i
   end do
   usv(nvaru+1:) = 0

      ! Clear auxiliary array of labels
   templab = ' '

      ! Default regression coefficients: equally spaced points
   do i = 1, maxrc
      rcoef(i) = i
   end do

      ! Set type of comparison to unknown-type code
   cmptyp = 0
      ! Set previous number of W-contrasts to 0
   nwold  = 0

   lprtb  = lprt .and. .not. linter
   lprti  = lprt .and. linter
   lerr   = .false.
   prvpla = .false.
   lplan  = .false.
   lcomp  = .false.
   lform  = .false.
   loldc  = .false.
   loldf  = .false.
   lablg  = .false.
   lablt  = .false.
   prompt = .true.

   paragr_loop: do   ! Loop on paragraphs reading
                     ! --------------------------

      if ( lcmdl ) then   ! If commands already read, skip reading
         lcmdl = .false.
      else
         if ( prompt ) then   ! Clear screen and prompt user
            call chkpg( 0, .true., newp )
            if ( linter .and. lec == lec1 ) then
               write(imp,2222) pfin(:nfin)
               lines = lines + 1
            end if
         else
            prompt = .true.
         end if

         ! Read paragraphs entered
         ! -----------------------
         lerr = .false.
            ! Set cfin as 1st character (for searching for cfin//pargr.name)
         ncar = 1
         car(1:1) = cfin
         call gmpcmd( card, lcmd1, lprtb, pfin, nend, ikend, kard, nrrd, leof, lerr )
         if ( lerr ) goto 20
         if ( leof ) then
            write(lgu,*) ermrk1
            write(lgu,888)
            write(lgu,*) ermrk2
            kstop = -1
            goto 30                              !--- stop e-o-f ---!
         end if

      end if

      lcmd1 = .false.

      k1 = 1
      do
         if ( car(k1:k1) /= cfin ) exit
         k1 = k1 + 1
      end do

         ! Add last end-of-sentence symbol in paragraphs if missing
      call gmpads( cfin, sep2, k1 )

         ! Selective conversion of commands (keywords) to capitals
         ! throughout all paragraphs (keyword values not converted)
      call cvucs( car(k1:ncar), com(k1:ncar), '=', sep2 )

      ! Special COMM paragraph ?
      ! ------------------------
      k3 = k1 + mpcomm - 1
      if ( car(k1:k3) == pcomm(:mpcomm) ) then
         kn = ncar - nfin

            ! Print commands read if requested and interactive run
         if ( lprti ) then
            nladd = 3 + (kn-k1)/lrecp
            call chkpg( nladd, .false., newp )
            write(imp,777) pcomm(:mpcomm), msginp
            write(imp,*) car(k1:kn)
            lines = lines + nladd
         else
!!          call chkpg( 2, .false., newp )
            write(lgu,777) pcomm(:mpcomm)
!!          lines = lines + 2
         end if

         car(k1:k3) = blank(:mpcomm-1) // sep2
            ! Analyze comm paragraph
         kn = kn + 1
         call gmpcom( k3, kn, lerr, maxr )
         lprtb = lprt .and. .not. linter
         lprti = lprt .and. linter
         if ( lerr ) then
            if ( linter .and. lec == lec1 ) then
               call chkpg( 0, .true., newp )
               write(imp,666)
               write(imp,444) pfin(:nfin)
               lines = lines + 2
               prompt = .false.
               cycle paragr_loop
            end if
            goto 20                              !--- stop error ---!
         end if

            ! Go back to read the paragraphs on appropriate unit
         if ( linter .and. lec == lec1 ) then
               ! Prompt user
            call chkpg( 0, .true., newp )
            write(imp,2222) pfin(:nfin)
            lines = lines + 1
         end if
         prompt = .false.
         cycle paragr_loop

      end if

         ! karp is starting location in car for search of paragraphs
         ! komp1/kompn are starting/ending locations in car of a COMP
         ! paragraph given within a batch of other paragraphs
      karp  = max( k1-1, 1 )
      komp1 = 0
      kompn = 0

      lerp  = .false.
      lerb  = .false.

      ! Loop in case of error in PAC paragraphs and interactive run
      ! ----
   10 continue
      none  = .true.
      planc = .false.

      ! No request ?
      ! ------------
      if ( car(k1:k1+nfin-2) == pfin(2:nfin) ) cycle paragr_loop

      ! New problem ?
      ! -------------
      if ( car(k1:k1+mpprob-1) == pprob(:mpprob) ) then
         lcmdl = .true.
         goto 30
      end if

      ! End of run ?
      ! ------------
      if ( car(k1:k1+mpstop-1) == pstop(:mpstop) ) then
         kstop = 1
         goto 30                                 !=== stop normal ===!
      end if

      ! COMP paragraph ?
      ! ----------------
      if ( car(k1:k1+mpcomp-1) /= pcomp(:mpcomp) ) then

         ! Analysis of PLAN paragraph
         ! --------------------------
         call local( cfin//pplan(:mpplan), 1+mpplan, karp, ncar, .false., 1, ioc, loca )
         if ( ioc > 0 ) then
            prvpla = .false.
            prvldp = .false.
            none   = .false.
            k1     = loca(1)
            kar1   = namab( car, pplan, npplan, mpplan, k1+1 )
               ! Search for the end of the paragraph
            call local( cfin, 1, kar1+1, ncar, .false., 1, ioc, loca )
            karn = loca(1)

               ! Print commands read if requested and interactive run
            if ( lprti ) then
               nladd = 3 + (karn-(k1+1))/lrecp
               call chkpg( nladd, .false., newp )
               write(imp,777) pplan(:mpplan), msginp
               write(imp,*) car(k1+1:karn)
               lines = lines + nladd
            else
!!             call chkpg( 2, .false., newp )
               write(lgu,777) pplan(:mpplan)
!!             lines = lines + 2
            end if

               ! Analysis with filtering
               ! lonce (corresponding to whether covariates were given only
               ! once for all treatments) and nuvar are known on return
            car(k1:kar1) = blank(:kar1-k1) // sep2
            call pacpla( k1, karn, lerp, unom, gsym, tsym, usv, sizusv, &
                         templab, lonce, lablg, lablt, iulid, lidfil )
            lplan = .true.
            planc = .true.
         end if

         ! Analysis of BAY paragraph
         ! -------------------------
         call local( cfin//pbaye(:mpbaye), 1+mpbaye, karp, ncar, .false., 1, ioc, loca )
         if ( ioc > 0 ) then
            none = .false.
            k1   = loca(1)
            kar1 = namab( car, pbaye, npbaye, mpbaye, k1+1 )
               ! Search for the end of the paragraph
            call local( cfin, 1, kar1+1, ncar, .false., 1, ioc, loca )
            karn = loca(1)

               ! Print commands read if requested and interactive run
            if ( lprti ) then
               nladd = 3 + (karn-(k1+1))/lrecp
               call chkpg( nladd, .false., newp )
               write(imp,777) pbaye(:mpbaye), msginp
               write(imp,*) car(k1+1:karn)
               lines = lines + nladd
            else
!!             call chkpg( 2, .false., newp )
               write(lgu,777) pbaye(:mpbaye)
!!             lines = lines + 2
            end if

               ! Analysis with filtering
            car(k1:kar1) = blank(:kar1-k1) // sep2
            call pacbay( k1, karn, lerb )

               ! N.B. Problem-record will be written on Bayes save file
               ! just before the analysis of comparisons. The reason for
               ! this is that saving can be done on a save file defined in
               ! a previous problem (thus without any Bayes paragraph in
               ! current problem)

         end if

         ! Is a COMP paragraph given within a batch of other paragraphs ?
         ! --------------------------------------------------------------
            ! If so the paragraph is only located and will be analyzed
            ! later on, after basic computations by pacx if PLAN is entered
         call local( cfin//pcomp(:mpcomp), 1+mpcomp, karp, ncar, .false., 1, ioc, loca )
         if ( ioc > 0 ) then
            none  = .false.
            lcomp = .true.
            komp1 = loca(1)
               ! Search for the end of the paragraph
            call local( cfin, 1, komp1+1, ncar, .false., 1, ioc, loca )
            kompn = loca(1)
         end if

         ! Filter unused or unrecognized paragraphs
         ! ----------------------------------------
         if ( lfltr .and. ncar-nfin-karp > kompn-komp1 ) then
            call local( cfin, 1, karp, ncar-nfin, .false., 9, ioc, loca )
               ! COMP paragraph is excluded from filtering, as not yet analyzed
            if ( lcomp ) then
               i = ioc - 1
            else
               i = ioc
            end if
            if ( i > 0 ) then
               nladd = 5 + i
               call chkpg( nladd, .false., newp )
               write(imp,222)
               write(lgu,*) ermrk1//ermrk1
               write(lgu,222)
               loca(ioc+1) = ncar - nfin + 1
               do i = 1, ioc
                  if ( .not. lcomp .or. loca(i) /= komp1 ) then
                     ka = loca(i)
                     kb = min( ka+10, loca(i+1)-1 )
                     write(imp,111) car(ka:kb)
                     write(lgu,111) car(ka:kb)
                  end if
               end do
               write(lgu,*) ermrk2//ermrk2
               lines = lines + nladd
            end if
         end if

         ! Reenter paragraph if error and interactive run
         ! ----------------------------------------------
         if ( lerb .or. lerp ) then
            if ( .not. linter .or. lec /= lec1 ) goto 20   !--- stop error ---!
            call chkpg( 0, .true., newp )
            write(imp,333)
            write(imp,444) pfin(:nfin)
            lines = lines + 2
            karp  = kompn + 1
            ncar  = karp
            car(karp:karp) = cfin
               ! The corrected parahraphs are stored AFTER the COMP paragraph
               ! (if given within others) so that its location is unchanged
               ! N.B. cfin must be stored just before the new paragraph
               ! name because searching is done for the name preceded by cfin
               ! but user may begin directly with paragraph name
            call gmpcmd( card,.false., lprtb, pfin, nend, ikend, kard, nrrd, leof, lerr )
            if ( lerr ) goto 20                  !--- stop error ---!
            if ( leof ) then
               write(imp,888)
               kstop = -1
               goto 30                           !--- stop e-o-f ---!
            end if

            k1 = karp + 1
            if ( car(k1:k1) == cfin ) then
               karp = k1
               k1   = k1 + 1
            end if

               ! Add last end-of-sentence symbol in paragraphs if missing
            call gmpads( cfin, sep2, k1 )

               ! Selective conversion of commands (keywords) to capitals
               ! throughout all paragraphs (keyword values not converted)
            call cvucs( car(k1:ncar), com(k1:ncar), '=', sep2 )

            goto 10         ! Back to paragraphs reading

         end if

         ! At least one paragraph is required
         ! ----------------------------------
         if ( none ) cycle paragr_loop

         if ( planc ) then
            ! A new PLAN paragraph was entered
            ! --------------------------------
            ! Analyze basic data set "X" (ngrp known on return)
            ! --------------------------
            call pacx( lonce, lablg, lablt, gsym(1:ngfac), tsym(1:ntfac), templab, allgt, lerx )
            planb = .true.
            if ( lerx ) then
               if ( linter ) then
                  call chkpg( 1, .false., newp )
                  write(imp,22222)
                  lines = lines + 1
                  goto 30
               end if
               goto 20                           !--- stop error ---!
            end if

               ! Allocate and fill in vec1
            if ( allocated(vec1) ) deallocate(vec1)
            allocate ( vec1(max(ngrp,ntrt)), stat=status )
            if ( status /= 0 ) then   ! fatal error
               write(lgu,*) ermrk1
               write(lgu,101) " (PacDir)."
               write(lgu,*) ermrk2
               if ( allocated(usv) ) deallocate(usv)
               if ( allocated(part) ) deallocate(part)
               call gmp999            ! release memory space and files
               call pac999( 99 )
               stop
            end if
            vec1 = one

               ! See if design is completely balanced
               ! ------------------------------------
            balang = .true.
            balant = .true.
            if ( ngrp > 1 ) balang = all( ag(2:ngrp) == ag(1) )
            if ( ntrt > 1 ) balant = all( at(2:ntrt) == at(1) )

               ! Create an Eyelid save file if requested
               ! ---------------------------------------
            if ( lidfil(1:1) /= ' ' ) then
               call paclid( iulid, lidfil, gsym(1:ngfac), tsym(1:ntfac),  &
                            nomv, gnum, gmod, tmod, gstr, tstr )
            end if

         end if

         ! Read next request (unless COMP paragraph already given)
         ! -----------------
         if ( .not. lcomp ) cycle paragr_loop

                            !---------------!

      end if

      ! Analysis of COMP paragraph
      ! --------------------------
         ! First, check a PLAN paragraph was given
      if ( .not. lplan ) then
         write(lgu,*) ermrk1
         write(lgu,999)
         write(lgu,*) ermrk2
!!       lines = lines + 2
         if ( linter .and. lec == lec1 ) then
            call chkpg( 0, .true., newp )
            write(imp,1111)
            write(imp,444) pfin(:nfin)
            lines = lines + 2
            prompt = .false.
            cycle paragr_loop
         end if
         goto 20                                 !--- stop error ---!
      end if

      if ( lcomp ) then
         k1    = komp1 + 1
         kar1  = namab( car, pcomp, npcomp, mpcomp, k1 )
         karn  = kompn
         komp1 = 0
         kompn = 0
         lcomp = .false.
         pnew  = .true.
      else
         kar1  = namab( car, pcomp, npcomp, mpcomp, k1 )
            ! Search for the end of the paragraph
         call local( cfin, 1, kar1+1, ncar, .false., 1, ioc, loca )
         karn  = loca(1)
         pnew  = .false.
      end if

         ! Print commands read if requested and interactive run
      if ( lprti ) then
         nladd = 3 + (karn-k1)/lrecp
         call chkpg( nladd, pnew, newp )
         write(imp,777) pcomp(:mpcomp), msginp
         write(imp,*) car(k1:karn)
         lines = lines + nladd
      else
   !!!   call chkpg( 2+10, pnew, newp )
!!       call chkpg( 2+10, .false., newp )
         write(lgu,777) pcomp(:mpcomp)
!!       lines = lines + 2
      end if

         ! Analysis with filtering
         ! In case of entry by vectors, v, w/u are allocated in paccom
      car(k1-1:kar1) = blank(1:kar1-k1+1) // sep2
      call paccom( k1, karn, lerr, ndf1, lform, baylev, outlev, maxou, noutlv, rcoef, maxrc, nrco )

      ! Reenter paragraph if error and interactive run
      ! ----------------------------------------------
      if ( lerr ) then
         if ( .not. linter .or. lec /= lec1 ) goto 20   !--- stop error ---!
         call chkpg( 0, .true., newp )
         write(imp,333)
         write(imp,444) pfin(:nfin)
         lines = lines + 2
         ncar = 1
         karp = 1
         car(1:1) = cfin
            ! N.B. cfin must be stored just before the new paragraph
            ! name because searching is done for the name preceded by cfin
            ! while user may begin directly with paragraph name
         call gmpcmd( card,.false., lprtb, pfin, nend, ikend, kard, nrrd, leof, lerr )
         if ( lerr ) goto 20                     !--- stop error ---!
         if ( leof ) then
            write(imp,888)
            kstop = -1
            goto 30                              !--- stop e-o-f ---!
         end if

         k1 = 2
         if ( car(k1:k1) == cfin ) then
            karp = k1
            k1   = k1 + 1
         end if

            ! Add last end-of-sentence symbol in paragraphs if missing
         call gmpads( cfin, sep2, k1 )

            ! Selective conversion of commands (keywords) to capitals
            ! throughout all paragraphs (keyword values not converted)
         call cvucs( car(k1:ncar), com(k1:ncar), '=', sep2 )

         goto 10            ! Back to paragraphs reading

      end if

      ! Prepare bayes save file if used
      ! -------------------------------
      if ( lbayes ) then
            ! Write problem-record on bayes save file
            ! (remember opening and rewinding were done by pacbay if relevant)
         if ( lrwdb .or. newpb ) then
            write(ibayes,33333) bsymp, nprob, titre(:72)
            if ( lrwdb ) then
               irep = 1
            else
               irep = 0
            end if
!!          call chkpg( 5, .false., newp )
            write(lgu,3333) ibayes, rep(irep)
!!          lines = lines + 3
            if ( bayfil(:1) /= ' ' ) then
               j = len_trim( bayfil )
               write(lgu,111) savnam
               write(lgu,111) (bayfil(i:min(i+lrecp-2,j)),i=1,j,lrecp-1)
!!             lines = lines + 1 + (j-1)/(lrecp-1) + 1
            end if
            newpb = .false.
            lrwdb = .false.
         end if
      end if

      if ( lform ) then

         ! Entry by formulas: analyze them and perform the comparisons
         ! -----------------------------------------------------------

            ! Check for duplicated regression coefficients
         if ( nrco < maxrc ) then
            do i = 1, nrco-1
               do j = i+1, nrco
                  if ( rcoef(i) == rcoef(j) ) then
!!                   call chkpg( 3+1+(nrco-1)/lrecp, .false., newp )
                     write(lgu,*) ermrk1
                     write(lgu,44444) (rcoef(k),k=1,nrco)
                     write(lgu,*) ermrk2
!!                   lines = lines + 3 + 1 + (nrco-1)/lrecp
                     if ( linter .and. lec == lec1 ) then
                        call chkpg( 0, .true., newp )
                        write(imp,333)
                        write(imp,444) pfin(:nfin)
                        lines = lines + 2
                        goto 10      ! Back to paragraphs reading
                     end if
                     goto 20                     !--- stop error ---!
                  end if
               end do
            end do
         end if

            ! Allocate part (GT-partition)
         if ( allocated(part) ) deallocate(part)
         allocate ( part(ntrt,ngrp), stat=status )
         if ( status /= 0 ) then   ! fatal error
            write(lgu,*) ermrk1
            write(lgu,101) " (PacDir)."
            write(lgu,*) ermrk2
            if ( allocated(usv) ) deallocate(usv)
            call gmp999            ! release memory space and files
            call pac999( 99 )
            stop
         end if

         do i = 1, maxnf   ! Loop on comparison-formulas
                           ! ---------------------------

               ! If no more formula read next request
            if ( cmpfor(i)(1:1) == noform ) cycle paragr_loop

               ! Compress formula
               ! (Note that closing parentheses, if any, are currently
               ! suppressed. So, although not pertaining to the language
               ! of comparisons, they can be used for within comparisons
               ! but only with an "aesthetic" purpose and they are not
               ! analyzed)
            last = 1
            do k = 1, maxcf
               if ( cmpfor(i)(k:k) /= ' ' .and. cmpfor(i)(k:k) /= ')' ) then
                  cmpfor(i)(last:last) = cmpfor(i)(k:k)
                  last = last + 1
               end if
            end do
            if ( last <= maxcf ) cmpfor(i)(last:last) = ' '
            last = last - 1

            nladd = (last-1)/lrecp + 3
                ! When checking for enough lines, take account of
                ! - message for weighting type (2 lines),
                ! - messages by pacfor (comparison type: 3 lines,
                !   re-using formula or condition: at least 3 lines each),
                ! - messages by paclvo (at least 3 lines)
   !!!!!!   call chkpg( nladd+2+3+3+3+3, i>1, newp )
            call chkpg( nladd+2+3+3+3+3, .false., newp )
            write(imp,5555) entyp(1)
            write(imp,111) (cmpfor(i)(j:min(j+lrecp-2,last)),j=1,last,lrecp-1)
            lines = lines + nladd

               ! Analyze formula (ncv & ncw known at completion of analysis)
            nszu = ngrp
            call pacfor( i, last, oldcnd, noldc, loldc, planb,               &
                         oldfor, noldf, loldf, rcoef, nrco, balang, balant,  &
                         part, cmptyp, rtyp, kdeg, ndf1, npart, nrsavd, lerr )

               ! Delete formula processed
            cmpfor(i)(1:1) = noform
            if ( lerr ) then   ! Skip the comparison in error
               cmptit = ' '
!!             call chkpg( 2, .false., newp )
               write(lgu,*) ermrk1
               write(lgu,9999)
               write(lgu,*) ermrk2
!!             lines = lines + 2
               cycle           ! Examine next comparison (if any)
            end if
            ncomp = ncomp + 1
            planb = .false.

               ! Print weighting type
            call chkpg( 3, .false., newp )
            write(imp,8888) pondg(gpond), pondt(tpond)
            lines = lines + 3
               ! Set up output options
            call paclvo( outlev, maxou, noutlv, ndf1==1, baylev, hyp0, nomv )
               ! Write comparison-record on bayes save file (if used)
            if ( lbayes ) write(ibayes,33333) bsymc, ncomp, cmptit(:72)
               ! Check some results from previous comparison can be used
            samew = prvpla .and. lupvw .and. ncw == nwold
               ! Select appropriate weights
            if ( gpond == 1 ) then
               gweight => vec1   ! Equal weights
            else
               gweight => ag     ! Group-size weights
            end if
            if ( tpond == 1 ) then
               tweight => vec1   ! Equal weights
            else
               tweight => at     ! User-given weights
            end if
               ! Perform comparison-analysis
            call pacy( ndf1, lonce, lform, balang, balant, npart,      &
                       nrsavd, part, gweight(1:ngrp), tweight(1:ntrt), &
                       rtyp, baylev, samew, prvldp, lerr )
               ! Delete comparison-title
            cmptit = ' '
               ! Only the 1st derived protocol is saved
            lderiv = .false.
               ! Update check on availability of results for further use
            nwold  = ncw
            prvpla = .not. lerr .and. lupvw
         end do

      else

         ! Entry by vectors: perform the (only one) comparison
         ! ---------------------------------------------------
         ncomp = ncomp + 1
         if ( lupvw ) then
            j = 1
         else
            j = 2
         end if
            ! Take account of paclvo messages (at least 3 lines)
         call chkpg( 2+3+1+3, .false., newp )
         write(imp,5555) entyp(2)
         write(imp,6666) evtyp(j), ndf1
         lines = lines + 2 + 3
         if ( lupvw .and. .not. lspecf ) then
            write(imp,7777)
            lines = lines + 1
         end if
         if ( ndf1 <= 0 ) then
            write(lgu,*) ermrk1
            write(lgu,9999)
            write(lgu,*) ermrk2
!!          lines = lines + 1
            prvpla = .false.
            cycle paragr_loop
         end if
            ! Print weighting type
         call chkpg( 3, .false., newp )
         write(imp,8888) pondg(gpond), pondt(tpond)
         lines = lines + 3
            ! Set up output options
         call paclvo( outlev, maxou, noutlv, ndf1==1, baylev, hyp0, nomv )
            ! Write comparison-record on bayes save file (if used)
         if ( lbayes ) write(ibayes,33333) bsymc, ncomp, cmptit(:72)
            ! Check some results from previous comparison can be used
         samew = prvpla .and. lupvw .and. ncw == nwold
            ! Perform the comparison
         npart  = 0
         gweight => ag
         tweight => vec1
         rtyp   = 0
         nrsavd = 0
         call pacy( ndf1, lonce, lform, balang, balant, npart,      &
                    nrsavd, part, gweight(1:ngrp), tweight(1:ntrt), &
                    rtyp, baylev, samew, prvldp, lerr )
            ! Derived protocol was saved if required, so reset flag
         lderiv = .false.
            ! Update check on availability of results for further use
         nwold  = ncw
         prvpla = .not. lerr .and. lupvw

      end if

      ! Cycle to read next request
      ! --------------------------
   end do paragr_loop


   ! Error in gmpcmd and batch run (=> stop)
   ! ---------------------------------------
20 continue
   write(lgu,*) ermrk1
   write(lgu,555)
   write(lgu,*) ermrk2
   kstop = -1

   ! Normal and error return
   ! -----------------------
30 continue
   if ( lbayes ) then
!!    call chkpg( 4, .false., newp )
      write(lgu,4444) nbayes
!!    lines = lines + 4
   end if
   if ( allocated(usv) ) deallocate(usv)
   if ( allocated(part) ) deallocate(part)

   INCLUDE 'allocf.'
   INCLUDE 'gmpdirf1.'
   INCLUDE 'pacdirf.'

END SUBROUTINE pacdir
