SUBROUTINE pacfid( plevel, xeff, scal, stdv, df1, df2, l1df1, lstdv,  &
                   koda, mult, xlim )
!***********************************************************************
!     - pacfid -
!     given a Bayes-fiducial probability, computes and prints the
!     Bayes-FIDucial limits for the deviation between true effect and
!     observed effect, notable and negligible effect, and, possibly,
!     a credibility interval for the standard deviation
!     If a limit is entered, the probability for notable and negligible
!     effect, instead of the limit, will be searched for
!***********************************************************************
!
!     PLEVEL- INPUT . Bayes-fiducial probability              - REAL*8
!     XEFF  - INPUT . Effect value                            - REAL*8
!     SCAL  - INPUT . Scale index of Bayes-fiducial           - REAL*8
!                     distribution function (assumed positive)
!     STDV  - INPUT . Standard deviation associated to XEFF   - REAL*8
!     DF1   - INPUT . First  df parameter of Bayes-fiducial   - REAL*8
!                     distribution function
!     DF2   - INPUT . Second df parameter of Bayes-fiducial   - REAL*8
!                     distribution function
!     L1DF1 - INPUT . .true. if DF1 is one, .false. otherwise - LOGICAL
!     LSTDV - INPUT . .true. if inference on STDV is desired  - LOGICAL
!                     (univariate case only)
!     KODA  - INPUT . Whether the distribution fonction used  - INTEGER
!                     here is exact or approximate :
!                     0 = exact distribution fonction
!                     1 = approximate distribution fonction
!     MULT  - INPUT . Multivariate df (0 for univariate case) - INTEGER
!     XLIM  - INPUT . The limit entered                       - REAL*8
!                     (only used in univariate case and if
!                     LIMIT is .true.)
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     pacdtr : xdtr1
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        DFMTC  DTRINV  KSI2CDF  KSI2I  PACCHI  PACL2
!     Fortran  functions called:
!        ABS  NINT  SIGN  SQRT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacdtr
   USE paclgl
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: plevel, xeff, scal, stdv, df1, df2, xlim
   INTEGER, INTENT(IN) :: koda, mult
   LOGICAL, INTENT(IN) :: l1df1, lstdv

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: dtrinv, ksi2cdf, ksi2i, pacchi, pacl2
   CHARACTER(LEN=6), EXTERNAL :: dfmtc

   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8
   REAL(KIND=8), PARAMETER :: tolzer=1.0e-30_8, xapp=1.0e3_8

      ! tolzer is a tolerance for zero used for outputs

   REAL(KIND=8) :: a, a2, b, c, p, scal1, sgne, y, ya
   INTEGER :: iapp, ier, ine, isgne, kodap, mxit

   INCLUDE 'pacfidp.'

   !--------------------------------------------------------------------

      ! Note that no checks are made for enough lines in current page
      ! for outputs, thus it is assumed sufficient lines are provided
      ! (i.e. proper action must have been taken in calling module)

   kodap = koda + koda
   mxit  = maxit / 10
      ! a2 is centrality index of Bayes-fiducial distribution
   if ( l1df1 ) then
      scal1 = scal
      a2    = ( xeff / scal )**2
   else
      scal1 = scal / sqrt(df1)
      a2    = df1 * ( xeff / scal )**2
   end if

   if ( mult <= 0 ) then

      ! Univariate problem
      ! ------------------
      p = plevel
      if ( l1df1 ) then
         fmti(2:3) = fmti1
         fmtv(2:3) = fmtv1
         fmtl(2:3) = '16'
      else
         fmti(2:3) = fmti2
         fmtv(2:3) = fmtv2
         fmtl(2:3) = '24'
      end if

      ! (true effect - obs. effect) limit or credibility interval
      ! ---------------------------------------------------------
      if ( plevel > zero ) then
            ! Use the ksi distribution
         y = ksi2i( plevel, df1, df2, zero, delta, mxit, iapp, ier )
         if ( ier == 0 ) then
            y = sqrt(y) * scal1
            if ( l1df1 ) then
               fmtv(12:17) = dfmtc( 10, -6, xeff-y, tolzer )
               fmtv(33:38) = dfmtc( 10, -6, xeff+y, tolzer )
               write(imp,fmtv) xeff-y, nome(0), xeff+y, plevel, mapp(kodap+iapp)
            else
               fmti(fmtiy:fmtiy+5) = dfmtc( 10, -6, y, tolzer )
               write(imp,fmti) y, plevel, mapp(kodap+iapp)
            end if
         else
            write(imp,111) typer(0), errm(abs(ier))
         end if
         lines = lines + 1
      end if

      ! Negligible effect
      ! -----------------
         ! Use the ksi distribution
      if ( limit ) then
         y = (xlim/scal1)**2
         p = ksi2cdf( y, df1, df2, a2, delta, mxit, iapp, ier )
      else
         y = ksi2i( plevel, df1, df2, a2, delta, mxit, iapp, ier )
      end if
      if ( ier == 0 ) then
         if ( limit ) then
            y = abs( xlim )
         else
            y = sqrt(y) * scal1
         end if
         fmtl(20:25) = dfmtc( 10, -6, y, tolzer )
         write(imp,fmtl) typef(1), nome(1), sineq(-1), y, p, mapp(kodap+iapp)
      else
         write(imp,111) typer(1), errm(abs(ier))
      end if
      lines = lines + 1

      ! Notable effect
      ! --------------
      if ( l1df1 ) then
            ! Non-central Student-t distribution
         ine   = 0
         sgne  = sign( one, xeff )
         isgne = nint( sgne )
         if ( limit ) then
            y = ((xlim-xeff)/scal1)**2
            p = ksi2cdf( y, one, df2, zero, delta, mxit, iapp, ier )
            if ( ier == 0 ) then
               p = (one-p) * half
               if ( xeff == zero ) then
                  isgne = nint( sign( one, xlim ) )
               else if ( sign(one,xlim-xeff) /= sgne ) then
                  p = one - p
               end if
            end if
            y = xlim
         else
            ya = one - (plevel+plevel)
            y  = ksi2i( abs(ya), one, df2, zero, delta, mxit, iapp, ier )
            if ( ier == 0 ) y = xeff + sign(one,ya)*sgne*sqrt(y)*scal1
         end if
      else
            ! Ksi distribution
         ine   = 1
         isgne = 1
         if ( limit ) then
            y = (xlim/scal1)**2
            p = ksi2cdf( y, df1, df2, a2, delta, mxit, iapp, ier )
            if ( ier == 0 ) p = one - p
            y = abs( xlim )
         else
            y = ksi2i( one-plevel, df1, df2, a2, delta, mxit, iapp, ier )
            if ( ier == 0 ) y = sqrt(y) * scal1
         end if
      end if
      if ( ier == 0 ) then
         fmtl(20:25) = dfmtc( 10, -6, y, tolzer )
         write(imp,fmtl) typef(2), nome(ine), sineq(isgne), y, p, mapp(kodap+iapp)
      else
         write(imp,111) typer(2), errm(abs(ier))
      end if
      lines = lines + 1

      ! Credibility interval for standard deviation
      ! -------------------------------------------
      if ( lstdv .and. plevel > zero ) then
         write(imp,222) msgsdv, stdv
         lines = lines + 1
            ! Chi-square distribution
            ! Use xdtr1 in pacdtr to pass df parameter to pacchi
         xdtr1 = df2
         ya    = half + half*plevel
         y     = dtrinv( pacchi, ya, .true., .false., zero,  &
                         xmaxim, xdtr1, sqrt(xdtr1+xdtr1),   &
                         delta, mxit, ier )
         if ( ier == 0 ) then
            a  = xmaxim
            if ( y > zero ) a = stdv * sqrt( df2/y )
            ya = half - half*plevel
            y  = dtrinv( pacchi, ya, .true., .false., zero,  &
                         xmaxim, xdtr1, sqrt(xdtr1+xdtr1),   &
                         delta, mxit, ier )
         end if
         if ( ier == 0 ) then
            b = xmaxim
            if ( y > zero ) b = stdv * sqrt( df2/y )
            if ( xdtr1 <= xapp ) then
               iapp = 0
            else
               iapp = 1
            end if
            fmtv(12:17) = dfmtc( 10, -6, a, tolzer )
            fmtv(33:38) = dfmtc( 10, -6, b, tolzer )
            write(imp,fmtv) a, msgsdv, b, plevel, mapp(iapp)
         else
            write(imp,111) msgsdv//': ', errm(abs(ier))
         end if
         lines = lines + 1
      end if

   else

      ! Multivariate problem
      ! --------------------

      fmti(2:3) = fmti1
      fmtl(2:3) = '16'

      ! (true effect - obs. effect) limit
      ! ---------------------------------
         ! Exact chi-square distribution
         ! Use xdtr1 in pacdtr to pass df parameter to pacchi
      xdtr1 = mult * df1
      y     = dtrinv( pacchi, plevel, .true., .false., zero,  &
                      xmaxim, xdtr1, sqrt(xdtr1+xdtr1),       &
                      delta, mxit, ier )
      if ( ier == 0 ) then
         y = sqrt( y / xdtr1 ) * scal
         if ( xdtr1 <= xapp ) then
            iapp = 0
         else
            iapp = 1
         end if
         fmti(fmtiy:fmtiy+5) = dfmtc( 10, -6, y, tolzer )
         write(imp,fmti) y, plevel, mapp(kodap+iapp)
      else
         write(imp,111) typer(0), errm(abs(ier))
      end if
      lines = lines + 1

         ! Procedures for notable an negligible limits are not yet
         ! available in the general multivariate case
      if ( mult > 1 ) return

         ! Lambda-square distribution
         ! Use xdtr1, xdtr2, xdtr3 in pacdtr to pass parameters to pacl2
      xdtr1 = df1
      xdtr2 = df2
      xdtr3 = a2
      b = a2 / df1
      c = a2 / df2
      y = b + b + one
      a = b + one                   ! Mean
      b = sqrt((two/df1)*(b*c+y))   ! Std deviation

      ! Negligible effect
      ! -----------------
      y = dtrinv( pacl2, plevel, .true., .false., zero,  &
                  xmaxim, a, b, delta, mxit, ier )
      if ( ier == 0 ) then
         if ( y >= zero ) then
            y = sqrt(y)*scal
            fmtl(20:25) = dfmtc( 10, -6, y, tolzer )
            write(imp,fmtl) typef(1), nome(1), sineq(-1), y, plevel, mapp(kodap+iapp)
         else
            write(imp,111) typer(1), errm(5)
         end if
      else
         write(imp,111) typer(1), errm(abs(ier))
      end if
      lines = lines + 1

      ! Notable effect
      ! --------------
      y = dtrinv( pacl2, one-plevel, .true., .false., zero,  &
                  xmaxim, a, b, delta, mxit, ier )
      if ( ier == 0 ) then
         if ( y >= zero ) then
            y = sqrt(y)*scal
            fmtl(20:25) = dfmtc( 10, -6, y, tolzer )
            write(imp,fmtl) typef(2), nome(1), sineq(1), y, plevel, mapp(kodap+iapp)
         else
            write(imp,111) typer(2), errm(5)
         end if
      else
         write(imp,111) typer(2), errm(abs(ier))
      end if
      lines = lines + 1

   end if

   111 format(16x,'* ',2a)
   222 format(8x,a,' =',g14.7)

END SUBROUTINE pacfid
