SUBROUTINE pacfno( nr, nc, df1, j1, rep, yqg, v, ag, supg, ftest, qa, ierr )

!***********************************************************************
!     - pacfno -
!     computes approximate F-test when neither equal variances nor
!     circularity are assumed.
!     Works both for univariate and multivariate cases
!***********************************************************************
!
!     NR    - INPUT . Number of rows in REP and columns in V  - INTEGER
!     NC    - INPUT . Number of columns in REP                - INTEGER
!     DF1   - INPUT . Numerator df (= NR*NC)                  - REAL*8
!     J1    - INPUT . Index of the 1st relevant row in YQG    - INTEGER
!                     (used to restrict to the variable or
!                     subset of variables currently processed)
!     REP   - INPUT . Representant of effect with NR rows and - REAL*8
!                     NC columns
!     YQG   - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     V     - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     SUPG  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     FTEST - OUTPUT. Approximate F statistic                 - REAL*8
!     QA    - OUTPUT. Adjusted denominator df                 - REAL*8
!     IERR  - OUTPUT. Error code: 0 if normal, a non-zero     - INTEGER
!                     integer otherwise
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        MASTAZ  MSINV  MTASAZ
!     External functions called:
!        INDXT
!     Fortran  functions called:
!        PACK  TRANSPOSE
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: df1
   INTEGER, INTENT(IN) :: nr, nc, j1
   REAL(KIND=8), INTENT(IN) :: rep(nr,nc), yqg(nwdctg)
   REAL(KIND=8), INTENT(IN) :: v(ngrp,nr), ag(ngrp)
   INTEGER, INTENT(IN) :: supg(ngsup)
   REAL(KIND=8), INTENT(OUT) :: ftest, qa
   INTEGER, INTENT(OUT) :: ierr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: indxt

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, two=2.0_8, three=3.0_8, fivem=-5.0_8
   REAL(KIND=8) :: aa2, a2, a21, dc2, gg, g2, hg, hh, pp
   INTEGER :: i, ij, ijp, il, ilp, im, j, jl, jm, jmp, jt, j0
   INTEGER :: k, kc1, kc2, ks, ks1, ks2, k1, k2, l, ll, lm, lmp
   INTEGER :: lq, lqg, lq1, lq2, m, n, ncg, ncgc, ncr
   REAL(KIND=8) :: h(1)   ! h is an array used to ensure consistency
                          ! with array argument in mtasaz
   REAL(KIND=8) :: zd(nc*nr)
   REAL(KIND=8) :: zm((nc*nr)*(nc*nr+1)/2)
   REAL(KIND=8) :: zp((nc*ngsup)*(nc*ngsup+1)/2)
   REAL(KIND=8) :: zv(nc*ngsup*nc*nr)

   !--------------------------------------------------------------------

      ! zv is direct product of the part of v restricted to groups
      ! in G-comparison-support by the identity matrix of order nc
      ! Temporary zp is a block-diagonal matrix (in triangular form)
      ! with the yqg's (up to a constant) in the diagonal blocks

   ncr  = nc * nr
   ncg  = nc * ngsup
   ncgc = ncg * nc
   dc2  = -(nc+2)
   j0   = j1 - 1
   jt   = j1*(j1+1) / 2

   ! Initialize zv
   ! -------------
   zv = zero

   if ( nc > 1 ) then

      ! Fill in zv and temporary zp
      ! ---------------------------
      l  = 1
      ll = 0
      lq = jt

      do ks = 1, ngsup
            ! Fill in zv
         k = supg(ks)
         m = l
         do i = 1, nr
            hg = v(k,i)
            n  = m
            do j = 1, nc
               zv(n) = hg
               n = n + ncg + 1
            end do
            m = m + ncgc
         end do
            ! Fill in zp
         aa2 = one / (ag(k)*(ag(k)+dc2))
         lqg = lq
         do i = l, l+nc-1
            zp(ll+1:ll+l-1) = zero
            do j = ll+l, ll+i
               zp(j) = yqg(lqg) * aa2
               lqg = lqg + 1
            end do
            ll  = ll + i
            lqg = lqg + j0
         end do

         l  = l + nc
         lq = lq + nwdct
      end do

      ! Compute symmetric matrix zm = zv' * zp * zv
      ! -------------------------------------------
      call mtasaz( zv, zp, zm, ncg, ncr )

      ! Replace zm by its inverse
      ! -------------------------
      call msinv( zm, ncr, ierr )
      if ( ierr /= 0 ) goto 10

      ! Compute symmetric matrix zp = zv * zm * zv'
      ! -------------------------------------------
      call mastaz( zv, zm, zp, ncg, ncr )

      ! Compute h = zd' * zm * zd (df1 is mean of h)
      ! -------------------------
         ! (where zd is transpose(rep) taken as a columnwise vector)
         ! First rearrange rep in zd
!!    l = 0                     ! version pour rep
!!    m = ncr - nr              ! en 1 seule dimension nr*nc,
!!    do i = 1, nr              ! remplacee par
!!       do j = i, i+m, nr      ! l'instruction suivante
!!          l = l + 1           !
!!          zd(l) = rep(j)      !
!!       end do                 !
!!    end do                    !
      zd = pack( transpose(rep), mask=.true. )
      call mtasaz( zd, zm, h, ncr, 1 )

      ! Compute mean of h**2
      ! --------------------
      hh  = zero
      gg  = zero
      kc1 = 0
      lq1 = 0
      do ks1 = 1, ngsup
         k1  = supg(ks1)
         a2  = ag(k1) + dc2
         aa2 = ag(k1) * a2
         a21 = a2*(a2-one)
         hg  = zero
         do i = 1, nc
            do j = 1, nc
               ij  = indxt( i+j0, j+j0 )
               ijp = indxt( i+kc1, j+kc1 )
               do l = 1, nc
                  il  = indxt( i+j0, l+j0 )
                  ilp = indxt( i+kc1, l+kc1 )
                  jl  = indxt( j+j0, l+j0 )
                  do m = 1, nc
                     im  = indxt( i+j0, m+j0 )
                     jm  = indxt( j+j0, m+j0 )
                     jmp = indxt( j+kc1, m+kc1 )
                     lm  = indxt( l+j0, m+j0 )
                     lmp = indxt( l+kc1, m+kc1 )
                     hg  = hg + ( zp(ijp)*zp(lmp) + two*zp(ilp)*zp(jmp) )   &
                                * ( a21*yqg(lq1+ij)*yqg(lq1+lm) +           &
                                    a2*(yqg(lq1+il)*yqg(lq1+jm)+yqg(lq1+im)*yqg(lq1+jl)) )
                  end do
               end do
            end do
         end do
         hh = hh + hg/(aa2*aa2*(a21-two))

            ! Note that the following do-loop is not executed
            ! when ks1 = ngsup
         hg  = zero
         kc2 = kc1
         lq2 = lq1
         do ks2 = ks1+1, ngsup
            k2  = supg(ks2)
            kc2 = kc2 + nc
            lq2 = lq2 + nwdct
            g2  = zero
            do i = 1, nc
               do j = 1, nc
                  ij  = indxt( i+j0, j+j0 )
                  ijp = indxt( i+kc1, j+kc1 )
                  do l = 1, nc
                     il  = indxt( i+kc1, l+kc2 )
                     do m = 1, nc
                        jm  = indxt( j+kc1, m+kc2 )
                        lm  = indxt( l+j0, m+j0 )
                        lmp = indxt( l+kc2, m+kc2 )
                        g2  = g2 + (zp(ijp)*zp(lmp)+two*zp(il)*zp(jm)) &
                                   * yqg(lq1+ij) * yqg(lq2+lm)
                     end do
                  end do
               end do
            end do
            hg = hg + g2/(ag(k2)*(ag(k2)+dc2))
         end do
         gg = gg + hg/aa2

         kc1 = kc1 + nc
         lq1 = lq1 + nwdct
      end do

   else
         ! Simplifications occur: rep doesn't need to be re-arranged
         ! in zd and the do-loop's i,j,l,m to nc are no longer needed

      ! Fill in zv and temporary zp
      ! ---------------------------
      ll = 0
      lq = jt
      do l = 1, ngsup
            ! Fill in zv
         k = supg(l)
         m = l
         do i = 1, nr
            zv(m) = v(k,i)
            m = m + ngsup
         end do
            ! Fill in zp
         zp(ll+1:ll+l-1) = zero
         ll = ll + l
         zp(ll) = yqg(lq) / (ag(k)*(ag(k)+dc2))
         lq = lq + nwdct
      end do

      ! Compute symmetric matrix zm = zv' * zp * zv
      ! -------------------------------------------
      call mtasaz( zv, zp, zm, ncg, ncr )

      ! Replace zm by its inverse
      ! -------------------------
      call msinv( zm, ncr, ierr )
      if ( ierr /= 0 ) goto 10

      ! Compute symmetric matrix zp = zv * zm * zv'
      ! -------------------------------------------
      call mastaz( zv, zm, zp, ncg, ncr )

      ! Compute h = rep' * zm * rep (df1 is mean of h)
      ! ---------------------------
         ! (where rep is taken as a columnwise vector)
      call mtasaz( pack(rep,mask=.true.), zm, h, ncr, 1 )

      ! Compute mean of h**2
      ! --------------------
      hh  = zero
      gg  = zero
      ijp = 0
      lq1 = jt
      do ks1 = 1, ngsup
         k1  = supg(ks1)
         aa2 = ag(k1) * (ag(k1)+dc2)
         ijp = ijp + ks1
         hh  = hh  +  zp(ijp)*zp(ijp)*yqg(lq1)*yqg(lq1)  &
                      / (ag(k1)*aa2*(ag(k1)+fivem))

            ! Note that the following do-loop is not executed
            ! when ks1 = ngsup
         hg  = zero
         lmp = ijp
         lq2 = lq1
         do ks2 = ks1+1, ngsup
            k2  = supg(ks2)
            lmp = lmp + ks2
            lq2 = lq2 + nwdct
            il  = indxt(ks1,ks2)
            hg  = hg  +  (zp(ijp)*zp(lmp)+two*zp(il)*zp(il))  &
                         * yqg(lq2) / (ag(k2)*(ag(k2)+dc2))
         end do
         gg = gg + (yqg(lq1)/aa2)* hg

         lq1 = lq1 + nwdct
      end do
      hh = three * hh

   end if

      ! (hh+2*gg is mean of h**2)
   g2 = df1 * df1
   hh = hh + gg + gg - g2
   pp = df1
   gg = pp*hh - (g2+g2)

   ! Compute approximate F-test and adjusted denominator df
   ! ------------------------------------------------------
   if ( gg > zero ) then
      qa    = two * (one + pp*(hh+g2)/gg)
      ftest = qa/((qa-two)*df1) * h(1)
   else
      qa    = zero
      ftest = zero
   end if

   ! Normal return
   ! -------------
   return

   ! Error message for matrix inversion
   ! ----------------------------------
10 continue
      ! (It is assumed there's enough lines in page for output)
   write(imp,111) ierr
   lines = lines + 2

   INCLUDE 'pacfnof.'

END SUBROUTINE pacfno
