SUBROUTINE pacmvt( ierr, nw, nvar, nr, n1, multv, baylev, kova, copy,     &
                   noeq, prtc, iuw, boik, cstdf, cstb, namv, nnamv, namb, &
                   bmat, lbmat, rep, ys2, yqg, v, ag, supg, amat, wrk1 )
!***********************************************************************
!     - pacmvt -
!     computes and prints MultiVariate statistics and Tests for a
!     subset of variables
!     . If there is only one variable, no test is performed and return
!     is made immediately after computation of square root of inverse
!     of within-groups covariances matrix for the relevant subset
!     (under equal covariances assumption but no circularity)
!     . If both LUPVW and NOEQ are .false. then nothing is done
!***********************************************************************
!
!     IERR  - OUTPUT. Error code: 0 if square root of inverse - INTEGER
!                     of subset of YS2 has been successfully
!                     computed, a non-zero integer otherwise
!                     and then no saving on IUW
!                     (relevant only if comparison is product
!                     form)
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     NVAR  - INPUT . Number of dependent variables in subset - INTEGER
!     NR    - INPUT . Number of rows in REP and AMAT          - INTEGER
!     N1    - INPUT . First row/col in YS2 of the symmetric   - INTEGER
!                     subset of relevant variables
!                     So the last relevant row/col in YS2 is
!                     N1 + NW*NVAR - 1
!     MULTV - INPUT . code for what is to be done:            - INTEGER
!                     1 : only square root of inverse of
!                         relevant subset of YS2, and
!                         diagonalization if reduction
!                         process is requested
!                         (assuming equal covariances
!                         matrices but no circularity)
!                     >1: complete procedure
!     BAYLEV- INPUT . Bayes-probabilty level                  - REAL*8
!     KOVA  - INPUT . Status code regarding covariates:       - INTEGER
!                       0 : procedure applies to dep. var.
!                           with no covariates
!                       1 : procedure applies to dep. var.
!                           adjusted by covariates
!                      -1 : procedure applies to slopes of
!                           dep. var. for covariates
!     COPY  - INPUT . .true. if the relevant subset in YS2 is - LOGICAL
!                     not yet stored in WRK1 and thus must be
!                     copied into it prior to any computation
!     NOEQ  - INPUT . .true. if test not assuming equal       - LOGICAL
!                     covariances matrices nor circularity
!                     may be computed
!     PRTC  - INPUT . .true. if AMAT matrix is to be printed  - LOGICAL
!                     N.B. To be effective printing requires
!                     both PRTC is .true. and NR = NW = 1
!     IUW   - INPUT . Unit-number for saving square root of   - INTEGER
!                     inverse of subset of YS2
!                     (only if comparison is product form)
!                     If negative, no saving
!     BOIK  - INPUT . Boik's epsilon (1991) for circularity   - REAL*8
!                       = 0.0 if irrelevant,
!                       < 0.0 if unavailable
!     CSTDF - INPUT . The within df associated to the         - REAL*8
!                     comparison on relevant groups
!     CSTB  - INPUT . A coefficient involved in computation   - REAL*8
!                     of scale index of Bayes distribution
!                     It is the square of component of norm
!                     of G-contrast vectors (or 1.0 when the
!                     procedure applies to slopes of dep. var.
!                     for covariates) (May be multiplied by a
!                     constant if by covariates)
!                     Note: if adjustment by covariates and
!                     NR is 1, then it was divided by BMAT(1)
!                     Used only if comparison is product form
!     NAMV  - INPUT . Names of the NVAR dependent variables   - CHAR
!                     Used only if PRTC is .true.
!     NNAMV - INPUT . Dimension of NAMV                       - INTEGER
!     NAMB  - INPUT . A label used to represent the set of    - CHAR
!                     dependent variables when a Bayes save
!                     file is used (must be of same length as
!                     the variable names NOMV)
!     BMAT  - INPUT . A symmetric corrective matrix, of order - REAL*8
!                     NR, stored in triangular form
!                     (proceeding from covariates adjustment)
!                     Used only if KOVA .ne. 0
!     LBMAT - INPUT . Actual dimension of BMAT                - INTEGER
!                     (NR*(NR+1)/2 or 1 if not used)
!     REP   - INPUT . Representant of effect with NR rows and - REAL*8
!                     NW*NVAR columns
!     YS2   - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     YQG   - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!                     Used only if NOEQ is.true.
!     V     - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!                     Used only if NOEQ is.true.
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!                     Used only if NOEQ is.true.
!     SUPG  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!                     Used only if NOEQ is.true.
!     AMAT  - OUTPUT. Calibrated representant of effect       - REAL*8
!                     with NR rows and NW*NVAR columns
!     WRK1  - INPUT . Contains relevant subset of YS2 if COPY - REAL*8
!                     is .false.
!                     of length (NW*NVAR)*(NW*NVAR+1)/2
!             OUTPUT. Destroyed
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  MASC  MPOWER  MTASAZ  PACFNO  PACMV2
!     External functions called:
!        BETACDF
!     Fortran  functions called:
!        DOT_PRODUCT  MIN  NINT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacdtr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: baylev, boik, cstdf, cstb
   INTEGER, INTENT(IN) :: nw, nvar, nr, n1, multv, kova, iuw, nnamv, lbmat
   LOGICAL, INTENT(IN) :: copy, noeq, prtc
   CHARACTER(LEN=*), INTENT(IN) :: namb
   REAL(KIND=8), INTENT(IN) :: ys2(nwdct), yqg(nwdctg), v(ngrp,ncv)
   REAL(KIND=8), INTENT(IN) :: bmat(lbmat)
   REAL(KIND=8), INTENT(IN) :: rep(nr,nw*nvar), ag(ngrp)
   INTEGER, INTENT(IN) :: supg(ngsup)
   CHARACTER(LEN=*), INTENT(IN) :: namv(nnamv)
   REAL(KIND=8), INTENT(INOUT) :: amat(nr,nw*nvar), wrk1(nw*nvar*(nw*nvar+1)/2)
   INTEGER, INTENT(OUT) :: ierr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: betacdf
   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8
   REAL(KIND=8), PARAMETER :: epsl=1.0e-12_8
      ! epsl is a small real constant used to compute zero tolerance
      !      for eigenvalues as epsl * trace(matrix) / order(matrix)

   REAL(KIND=8) :: btmp, cst, cstc, df1m, df2m, ftest, prob
   REAL(KIND=8) :: sumx, test, tolz, trk, wdbl
   INTEGER :: i, ierb, ii, ij, j, jj, jjj, jt, jw, k, kj, kl, kstdf
   INTEGER :: l, lt, nc, nladd, nrc, nrw
   LOGICAL :: bcor, lmult, lrduw, newp, prt

   INCLUDE 'pacmvtp.'

   !--------------------------------------------------------------------
      ! NOTE. This routine may be called even when the order of the
      ! matrix to be diagonalized is 1 (thus trivial): this can only
      ! happen when data reduction is requested and nvar*nw=1 and nr>1

   ierr = 0
   if ( multv <= 0 ) return

      ! nc is the order of the relevant subset in ys2
   nc    = nvar * nw
   nrw   = nr * nw
   nrc   = nr * nc
   kstdf = nint( cstdf )
   prt   = prtc .and. nrw == 1
   lmult = multv > 1 .and. nvar > 1
   lrduw = lrduce .and. iuw >= 0
   if ( lupvw ) cstc = one / (cstdf*cstb)
   bcor  = kova /= 0 .and. nr > 1
   if ( kova /= 0 .and. nr == 1 ) then
      btmp = cstb / bmat(1)
   else
      btmp = cstb
   end if

   ! If required, copy the relevant symmetric subset of ys2 in wrk1
   ! --------------------------------------------------------------
   if ( copy ) then
      tolz = zero
      ii = n1*(n1-1)/2
      k = 0
      do i = n1, n1+nc-1
         do j = ii+n1, ii+i
            k = k + 1
            wrk1(k) = ys2(j)
         end do
         ii = ii + i
         tolz = tolz + wrk1(k)
      end do
   end if

         ! 1) Assuming equal covariances matrices
         ! ======================================

   if ( lupvw ) then

            ! 1-a) Not assuming circularity
            ! =============================

      if ( lmult ) then
         nladd = 7 + min0( 2, nw )
         if ( nr == 1 ) nladd = nladd + 6
         call chkpg( nladd, .false., newp )
         if ( ngsup > 1 ) then
            write(imp,111) with, uwith
            lines = lines + 2
         end if
         if ( nw > 1 ) then
            write(imp,222) witho
            lines = lines + 1
         end if
      end if

      if ( kstdf < nc ) then
         ierr = 9999
         write(imp,777)
         lines = lines + 1
         if ( .not. lmult ) return
         goto 10
      end if

         ! Compute zero tolerance for eigenvalues
      if ( .not. copy ) then
         tolz = zero
         ii = 0
         do i = 1, nc
            ii = ii + i
            tolz = tolz + wrk1(ii)
         end do
      end if
      tolz = epsl * tolz / nc

      ! Compute inverse of square-root of subset of ys2, in wrk1
      ! --------------------------------------------------------
      call mpower( wrk1, -half, nc, tolz, ierr )

      if ( ierr /= 0 ) then
         write(imp,666) ierr
         lines = lines + 2
         if ( .not. lmult ) return

      else
            ! If required save wrk1 on iuw
         if ( iuw >= 0 ) then
            rewind iuw
            write(iuw) wrk1
         end if

         ! That's all if multivariate proc. not needed or irrelevant
         ! ---------------------------------------------------------
         if ( .not. (lrduw .or. lmult) ) return

         ! Compute matrix of calibrated effect (amat)
         ! ------------------------------------------
         call masc( rep, wrk1, amat, nr, nc )
            ! If needed for data reduction step, save amat on iuw
         if ( lrduw ) write(iuw) amat

         ! Compute symmetric matrix amat'*(bmat*)amat (stored in wrk1)
         ! -----------------------------------------------------------
         if ( bcor ) then
               ! Covariates are used thus corrective bmat must be used
               ! Compute amat'*bmat*amat in wrk1
            call mtasaz( amat, bmat, wrk1, nr, nc )

         else
               ! No covariate
            ij = 0
            do i = 1, nc
               do j = 1, i
                  sumx = dot_product( amat(:,i), amat(:,j) )
                  ij = ij + 1
                  wrk1(ij) = sumx
               end do
            end do

         end if

         if ( lrduw .and. nc == 1 ) then
               ! Skip diagonalization, as trivial in this case, and
               ! save core matrix (also trace and eigenvalue), and exit
            write(iuw) wrk1(1)
            return
         end if

         ! Compute and print multivariate test-statistics
         ! ----------------------------------------------
         ij = min(nc,nnamv)
         call pacmv2( wrk1(:nc*(nc+1)/2), epsl,                   &
                      nc, nr, nr, iuw, lmult, cstdf, btmp, cstc,  &
                      prt, amat(:,1:nc), namv(1:ij), ij,          &
                      baylev, kova, 1, 0, one+one, namb, ierb )
            ! Is that all ?
         if ( .not. lmult ) return
      end if

   10 continue

            ! 1-b) Assuming circularity (if nw > 1)
            ! =========================

      if ( nw > 1 ) then

         call chkpg( nladd, .false., newp )
         write(imp,*) '  '
         write(imp,222) with
         lines = lines + 2

         if ( kstdf < nvar ) then
            ierb = 9999
            write(imp,777)
            lines = lines + 1
            if ( .not. lmult ) return
            goto 20
         end if

         wdbl = nw

            ! First restore the relevant subset of ys2 in wrk1
         ii = n1*(n1-1)/2
         k = 0
         do i = n1, n1+nc-1
            do j = ii+n1, ii+i
               k = k + 1
               wrk1(k) = ys2(j)
            end do
            ii = ii + i
         end do
            ! Compute the traces/w of the symmetric sub-matrices nw*nw
            ! in wrk1, for each pair of variables
            ! The resultant symmetric matrix is also stored in wrk1
            ! in triangular form with length nvar*(nvar+1)/2
         tolz = zero
         jjj = 0
         jw = 0
         do j = 1, nvar
            jj = jw*(jw+1)/2
            do k = 1, j
               ii = jj - (j-k)*nw
               trk = zero
               do i = jw+1, jw+nw
                  ii = ii + i
                  trk = trk + wrk1(ii)
               end do
               jjj = jjj + 1
               wrk1(jjj) = trk / wdbl
            end do
            tolz = tolz + trk
            jw  = jw + nw
         end do
         tolz = epsl * tolz / nvar

         ! Compute matrix of calibrated effect (amat)
         ! ------------------------------------------
            ! First, compute inverse of square-root of wrk1, resultant
            ! symmetric matrix stored in triangular form in wrk1
         call mpower( wrk1, -half, nvar, tolz, ierb )

         if ( ierb /= 0 ) then
            write(imp,666) ierb
            lines = lines + 2

         else
               ! Compute amat = rep * (direct product of wrk1 by iw)
            lt = 0
            kl = 0
            do l = 1, nvar
               do k = 1, nw
                  kl = kl + 1
                  do i = 1, nr
                     kj = k
                     sumx = zero
                     do j = 1, l
                        sumx = sumx + rep(i,kj)*wrk1(lt+j)
                        kj = kj + nw
                     end do
                     jt = lt + l
                     do j = l+1, nvar
                        sumx = sumx + rep(i,kj)*wrk1(jt+l)
                        kj = kj + nw
                        jt = jt + j
                     end do
                     amat(i,kl) = sumx
                  end do
               end do
               lt = lt + l
            end do

            ! Compute symmetric matrix amat'*(bmat*)amat (in wrk1)
            ! ----------------------------------------------------
            if ( bcor ) then
                  ! Corrective bmat is used as covariates are involved
                  ! compute amat'*bmat*amat in wrk1
               call mtasaz( amat, bmat, wrk1, nr, nc )
            else
                  ! No covariate
               ij = 0
               do i = 1, nc
                  do j = 1, i
                     sumx = dot_product( amat(:,i), amat(:,j) )
                     ij = ij + 1
                     wrk1(ij) = sumx
                  end do
               end do
            end if
               ! Compress wrk1 for circularity:
               ! Compute traces of the symmetric sub-matrices nw*nw
               ! in wrk1, for each pair of variables
               ! The resultant symmetric matrix is also stored in wrk1
               ! in triangular form with length nvar*(nvar+1)/2
            jjj = 0
            jw = 0
            do j = 1, nvar
               jj = jw*(jw+1)/2
               do k = 1, j
                  ii = jj - (j-k)*nw
                  cst = zero
                  do i = jw+1, jw+nw
                     ii = ii + i
                     cst = cst + wrk1(ii)
                  end do
                  jjj = jjj + 1
                  wrk1(jjj) = cst
               end do
               jw = jw + nw
            end do

            ! Compute and print multivariate test-statistics
            ! ----------------------------------------------
            call pacmv2( wrk1(:nvar*(nvar+1)/2), epsl,                           &
                         nvar, nr, nrw, -1, lmult, cstdf*wdbl, btmp, cstc/wdbl,  &
                         prt, amat(:,1:nvar), namv(1:nvar), nvar,                &
                         baylev, kova, 1, 1, boik, namb, ierb )
         end if

      end if

   end if

20 continue

         ! 2) Not assuming equal covariances matrices nor circularity
         ! ==========================================================

   if ( noeq .or. .not. lupvw ) then
      call chkpg( 3+1+4, .false., newp )
      write(imp,*) '  '
      write(imp,111) witho, uwitho
      lines = lines + 3
      if ( nw > 1 ) then
         write(imp,222) witho
         lines = lines + 1
      end if
   end if

   if ( .not. noeq ) then
      if ( .not. lupvw ) then
         write(imp,777)
         lines = lines + 1
      end if
      return
   end if
   if ( kstdf < nrc ) then
      write(imp,777)
      lines = lines + 1
      return
   end if

   df1m = nrc
      ! Compute approximate F-test
   call pacfno( nr, nc, df1m, n1, rep, yqg, v, ag, supg, ftest, df2m, ierb )
   if ( ierb /= 0 ) return
      ! Compute significance level of ftest
   if ( df2m > zero ) then
      test = df1m * ftest
      prob = betacdf( test/(test+df2m), half*df1m, half*df2m, ierb )
      if ( ierb == 0 ) then
         prob = one - prob
      else
         prob = -one
      end if
   end if

   ! Print results
   ! -------------
   write(imp,*) '  '
   if ( df2m > zero ) then
      write(imp,333)
      write(imp,444) ftest, df1m, df2m, prob, ' A'
      lines = lines + 4
   else
      write(imp,555)
      lines = lines + 2
   end if

   INCLUDE 'pacmvtf.'

END SUBROUTINE pacmvt
