SUBROUTINE pacovw( x, xpar, ax, ndim, ncx, nxp1, nxno0, k1p, nprod,   &
                   balall, arith, lerr )
!***********************************************************************
!     - pacovw -
!     Othogonalizes and re-norms contrasts (V or W) according to the
!     fundamental measure (product type comparisons only).
!***********************************************************************
!
!     X     - INPUT . Contrast-coefficients matrix to be      - REAL*8
!             OUTPUT. orthogonalized and properly normed,
!                     of length NDIM*NCX
!     XPAR  - INPUT . X partition-vector, of length NDIM      - INTEGER
!     AX    - INPUT . Vector of weights, of length NDIM       - REAL*8
!     NDIM  - INPUT . Dimension of the set involved (G or T)  - INTEGER
!     NCX   - INPUT . Number of contrasts in X                - INTEGER
!     NXP1  - INPUT . Number of parts in XPAR                 - INTEGER
!     NXNO0 - INPUT . Number of non-zero elements in XPAR     - INTEGER
!     K1P   - INPUT . Index of first non-zero element in XPAR - INTEGER
!     NPROD - INPUT . > 1 if there has been product of        - INTEGER
!                     contrast vectors; 0 or 1 otherwise
!     BALALL- INPUT . .true. if equal marginal weights        - LOGICAL
!     ARITH - INPUT . .true. if arithmetic mean is used to    - LOGICAL
!                     derived metric terms for
!                     orthogonalization;
!                     .false. if harmonic mean is used
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        BASORT  GMP999  MABC  MTADAS  PAC999  TQL2X
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ALLOCATE  ALLOCATED  DEALLOCATE  MAX  SUM  SQRT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ndim, ncx, nxp1, nxno0, k1p, nprod
   LOGICAL, INTENT(IN) :: balall, arith
   REAL(KIND=8), INTENT(IN) :: ax(ndim)
   INTEGER, INTENT(IN) :: xpar(ndim)
   REAL(KIND=8), INTENT(INOUT) :: x(ndim*ncx)
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8), PARAMETER :: epsl=1.0e-12_8
      ! epsl is a small real constant used to calculate zero tolerance
      ! for eigenvalues as epsl * trace(matrix) / order(matrix)

   REAL(KIND=8) :: c, cl, tolz
   INTEGER :: i, ierr, j, k, l, lbc, lbl, lbx, m, status

   REAL(KIND=8) :: xwgh(ndim)
   REAL(KIND=8), ALLOCATABLE :: bwrk(:)
      ! bwrk is work array of maximal length 1 + nxp1*max(nxp1-1,1)
      ! or 2*nxp1*ncx + nxp1 + ncx*(ncx+1)/2

   !--------------------------------------------------------------------

   lerr = .false.

      ! A sufficient (not necessary) property for orthogonality
      ! is that the product-partition is balanced.
      ! If x already orthogonal skip orthogonalization

      ! Fill in xwgh with relevant part-weights
   xwgh(1:nxp1) = zero
   if ( arith ) then
      do k = k1p, ndim
         j = xpar(k)
         if ( j > 0 ) xwgh(j) = xwgh(j) + ax(k)
      end do
         ! See if the product-partition is balanced
      do j = 2, nxp1
         if ( xwgh(j) /= xwgh(1) ) goto 10   ! Unbalanced
      end do
         ! Balancing holds, thus skip orthonormalization
      return
      10 continue
   end if
      ! Allocate bwrk
   if ( nprod > 1 ) then
      j = 2*nxp1*ncx + nxp1 + ncx*(ncx+1)/2
   else
      j = 1 + nxp1*max(nxp1-1,1)
   end if
   allocate ( bwrk(1:j), stat=status )
   if ( status /= 0 ) then   ! fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacOVW)."
      write(lgu,*) ermrk2
      call gmp999            ! release memory space and files
      call pac999( 99 )
      stop
   end if

   bwrk(1:nxp1) = zero

   if ( .not. arith ) then   ! Harmonic mean is used
      do k = k1p, ndim
         j = xpar(k)
         if ( j > 0 ) then
            xwgh(j) = xwgh(j) + one/ax(k)
            bwrk(j) = bwrk(j) + one
         end if
      end do
      xwgh(1:nxp1) = bwrk(1:nxp1)*bwrk(1:nxp1) / xwgh(1:nxp1)
   end if

   if ( nprod > 1 ) then

         ! Locations in bwrk
      lbc = 1   + nxp1*ncx
      lbl = lbc + nxp1*ncx
      lbx = lbl + nxp1
         ! Store 1/xwgh in bwrk(lbl)
      bwrk(lbl:lbl-1+nxp1) = one / xwgh(1:nxp1)
         ! Store x (parts numbered 0 being compressed) in bwrk(lbc)
      l = 0
      i = lbc - 1
      do m = 1, ncx
         do k = k1p, ndim
            j = xpar(k)
            if ( j > 0 ) bwrk(i+j) = x(l+k)
         end do
         i = i + nxp1
         l = l + ndim
      end do
         ! Compute bc'*bl*bc in bwrk(lbx)
      j = ncx*(ncx+1)/2
      k = ncx*ncx
      call mtadas( bwrk(lbc:lbc-1+nxp1*ncx), bwrk(lbl:lbl-1+nxp1), bwrk(lbx:lbx-1+j), nxp1, ncx )
         ! Diagonalize bwrk(lbx): eigenvalues returned in bwrk(lbl),
         ! eigenvectors in x(1); x(ncx*ncx+1) used as work array
      call tql2x( ncx, bwrk(lbx:lbx-1+j), bwrk(lbl:lbl-1+ncx), x(k+1:k+ncx), x(1:k), .true., ierr )
      lerr = ierr /= 0
      if ( lerr ) then
         if ( allocated(bwrk) ) deallocate(bwrk)
         return
      end if
         ! Calculate tolerance for zero for eigenvalues
      tolz = sum( bwrk(lbl:lbl+ncx-1) ) * epsl / ncx
         ! Calculate desired common squared-norm for x
         ! (harmonic mean of eigenvalues)
      c = zero
      do j = lbl, lbl+ncx-1
         if ( bwrk(j) > tolz ) c = c + one/bwrk(j)
      end do
      c = ncx / c
         ! Orthogonalize: bwrk(lbc)*x, store in bwrk(1)
      call mabc( bwrk(lbc:lbc-1+nxp1*ncx), x(1:ncx*ncx), bwrk(1:nxp1*ncx), nxp1, ncx, ncx )
         ! Re-norm and store in x: bwrk(1)j * sqrt(c/eigenvalue j)
      l = 0
      i = 0
      do m = 1, ncx
         cl = sqrt( c / bwrk(lbl-1+m) )
         do k = 1, ndim
            l = l + 1
            j = xpar(k)
            if ( j > 0 ) then
               x(l) = bwrk(i+j) * cl
            else
               x(l) = zero
            end if
         end do
         i = i + nxp1
      end do

   else if ( .not. balall .or. ncx /= nxno0 ) then
         ! No x-product, get x-contrasts with correct metric
         ! Find an orthonormal basis set of contasts for x,
         ! Store it in bwrk(2)
         ! Note that basort only concerns real contrast vectors
         ! when the partition has more than 1 part; thus the mean
         ! component of a regression should be excluded. But
         ! there's no problem as 1 x-vector comparisons were
         ! excluded from this process by a former logical test
      call basort( nxp1, nxp1, .false., xwgh(:nxp1), bwrk(2:1+nxp1*max(nxp1-1,1)) )
         ! Fill in x-contrast vectors
      l = 0
      i = 1
      do j = 1, max( nxp1-1, 1 )
         bwrk(i) = zero
         do k = 1, ndim
            x(l+k) = bwrk( i+xpar(k) )
         end do
         i = i + nxp1
         l = l + ndim
      end do
   end if

   if ( allocated(bwrk) ) deallocate(bwrk)

   INCLUDE 'allocf.'

END SUBROUTINE pacovw
