SUBROUTINE pacumt( nw, nvar, nr, df1, l1df1, iuss, iuw,    &
                   cstdf, cstb, racst, baylev, nokod,      &
                   lschef, noeqm, boik, kova, msgv, flag,  &
                   rep, cale, dfc, bmat, lbmat, lerr )
!***********************************************************************
!     - pacumt -
!     directs computation and printing of Univariate and Multivariate
!     statistics and Tests
!***********************************************************************
!
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     NVAR  - INPUT . Number of dependent variables           - INTEGER
!     NR    - INPUT . Number of rows in the matrix of         - INTEGER
!                     representant of effect (see REP)
!     DF1   - INPUT . Df of the comparison                    - REAL*8
!     L1DF1 - INPUT . .true. if DF1 is one                    - LOGICAL
!     IUSS  - INPUT . Unit number for ANOVA table saving      - INTEGER
!                     Only used if ANOVA table is requested
!                     and the comparison is product form
!     IUW   - INPUT . Unit number for saving multivariate     - INTEGER
!                     results (no save if negative)
!                     Only used if LREDUC is requested and
!                     the comparison is product form
!     CSTDF - INPUT . The within df associated to the         - REAL*8
!                     comparison on relevant groups
!     CSTB  - INPUT . A coefficient involved in computation   - REAL*8
!                     of scale index of Bayes distribution
!                     It is the square of component of norm
!                     of G-contrast vectors (or 1.0 when the
!                     procedure applies to slopes of dep. var.
!                     for covariates)
!                     Used only if comparison is product form
!     RACST - INPUT . A scalar involved in computation of sum - REAL*8
!                     of squares:
!                     the reciprocal of the scalar "a" such
!                     that W'*W = a*Iw  (Iw being the identity
!                     matrix of order NCW)
!                     Used only if comparison is product form
!     BAYLEV- INPUT . Bayes-probability level                 - REAL*8
!     NOKOD - INPUT . Code for lack of approximation for      - INTEGER
!                     univariate test without assumption:
!                     < 0 : approximation is available
!                       0 : not defined
!                       1 : insufficient group-counts
!     LSCHEF- INPUT . .true. if Scheffe's method for multiple - LOGICAL
!                     comparisons is to be applied
!     NOEQM - INPUT . .true. if multivariate test without     - LOGICAL
!                     assuming equal covariances nor
!                     circularity may be computed
!     BOIK  - INPUT . Boik's epsilon (1991) for circularity   - REAL*8
!                       = 0.0 if irrelevant,
!                       < 0.0 if unavailable
!     KOVA  - INPUT . Status code regarding covariates:       - INTEGER
!                       0 : procedure applies to dep. var.
!                           with no covariates
!                       1 : procedure applies to dep. var.
!                           adjusted by covariates
!                      -1 : procedure applies to slopes of
!                           dep. var. for covariates
!     MSGV  - INPUT . Message to be printed before results    - CHAR
!                     of multivariate step, regarding the
!                     set of variables used
!     FLAG  - INPUT . if .false. MSGV is not printed          - LOGICAL
!     REP   - INPUT . Matrix of representant of effect        - REAL*8
!                     2-dimensioned (NR,NW*NVAR)
!             OUTPUT. May be rescaled
!                     CAUTION: may be an array of module mem2
!     CALE  - OUTPUT. Matrix of multivariate representant of  - REAL*8
!                     calibrated effect
!                     2-dimensioned (NR,NW*NVAR)
!     DFC   - INPUT . Vector of departure from circularity    - REAL*8
!                     of length NVAR
!                     (only used if NW > 1)
!     BMAT  - INPUT . Symmetric corrective matrix of order NR - REAL*8
!                     stored in triangular form
!                     (proceeding from covariates adjustment)
!                     Unused if KOVA = 0
!     LBMAT - INPUT . Actual dimension of BMAT                - INTEGER
!                     (NR*(NR+1)/2 or 1 if not used)
!     LERR  - OUTPUT. .true. if an error occurred in PACMVT   - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACANV  PACMVT  PACUVT
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM  MAX  MIN  MOD  NINT SQRT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst2
   USE memstb
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: df1, cstdf, cstb, racst, baylev, boik
   INTEGER, INTENT(IN) :: nw, nvar, nr, iuss, iuw, nokod, kova, lbmat
   LOGICAL, INTENT(IN) :: l1df1, lschef, noeqm, flag
   CHARACTER(LEN=*), INTENT(IN) :: msgv
   REAL(KIND=8), INTENT(IN) :: bmat(lbmat)
   REAL(KIND=8), INTENT(IN) :: dfc(nvar)
   REAL(KIND=8), INTENT(INOUT) :: rep(nr,nw*nvar)
   LOGICAL, INTENT(OUT) :: lerr
   REAL(KIND=8), INTENT(OUT) :: cale(nr,nw*nvar)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: badj, baytmp, btmp
   INTEGER :: ierr, j, lcmpt, ll, multv, nladd, nwt
   LOGICAL :: leru, newp, nomeq, noueq, prtc
   REAL(KIND=8) :: wk1(nw*nvar*(nw*nvar+1)/2)

   INCLUDE 'pacumtp.'

   !--------------------------------------------------------------------

   lerr = .false.
   leru = .false.

      ! Length of comparison-title for printing
   lcmpt = min( lrecp-1, len_trim(cmptit) )

   noueq = nokod < 0

   ! If covariates adjustment, make corrections
   ! ------------------------------------------
   if ( kova /= 0 ) then
      baytmp = zero
   else
      baytmp = baylev
   end if
   if ( kova /= 0 .and. nr > 1 ) then
         ! Compute corrective constant (badj)
      badj = zero
      ll = 0
      do j = 1, nr
         ll = ll + j
         badj = badj + bmat(ll)
      end do
      badj = nr / badj
      btmp = cstb * badj
      badj = sqrt( badj )
         ! Rescale representant of effect using corrective constant
      rep = rep * badj
   else
      btmp = cstb
   end if

   ! Univariate procedures
   ! ---------------------
   if ( luniv ) then

      nladd = 20
      if ( noueq ) nladd = nladd + 8
      call chkpg( nladd, .false., newp )
      if ( newp ) then
         if ( .not. linter ) then
            write(imp,111) cmptit(:lcmpt)
            write(imp,111) '  '
            lines = lines + 2
         end if
      else
         write(imp,111) '  '
         lines = lines + 1
      end if
      if ( nw == 1 ) then
         write(imp,1111) '*', '*', ' ', ' ', '*', '*'
      else
         write(imp,1111) uwith, umsgci, with, msgci, uwith, umsgci
      end if
      lines = lines + 4

      call pacuvt( kova, nokod, nvar, nr, df1, l1df1,     &
                   cstdf, btmp, racst, iuss, baylev,      &
                   nomv, hyp0, xlim, bmat, lbmat, rep,    &
                   ys2, yqg, v, ag, supg, ene, qne, l_ene )

      ! Anova table computations if relevant
      ! ------------------------------------
      if ( lupvw ) then
         if ( lanova ) then
            call chkpg( 8, .false., newp )
            if ( newp ) then
               write(imp,111) cmptit(:lcmpt)
               lines = lines + 1
            end if
            if ( nw > 1 ) then
               write(imp,333)
            else
               write(imp,444)
            end if
            lines = lines + 3
         end if
         call pacanv( nvar, nw, dfc, lschef, iuss )
      end if

      ! Multivariate approach to repeated measures (if relevant)
      ! ------------------------------------------
      if ( nw > 1 ) then

         nladd = 16
         if ( noueq ) nladd = nladd + 4
         call chkpg( nladd+5, .false., newp )
         if ( newp ) then
            if ( .not. linter ) then
               write(imp,111) cmptit(:lcmpt)
               write(imp,111) '  '
               lines = lines + 2
            end if
         else
            write(imp,111) '  '
            lines = lines + 1
         end if
         write(imp,1111) uwitho, umsgci, witho, msgci, uwitho, umsgci
         lines = lines + 4

         if ( nint(cstdf) >= nw ) then

            if ( .not. noueq .and. ngsup > 1 ) then
               write(imp,555)
               lines = lines + 1
               if ( nokod == 1 ) then
                  write(imp,666)
                  lines = lines + 2
               else
                  write(imp,111) '  '
                  lines = lines + 1
               end if
            end if

            nwt = nw*(nw+1)/2
            leru = .false.
            ll = 1
            do j = 1, nvar
               call chkpg( nladd, .false., newp )
               if ( newp .and. .not. linter ) then
                  write(imp,111) cmptit(:lcmpt)
                  write(imp,111) '  '
                  lines = lines + 2
               end if
               write(imp,222) nomv(j)
               lines = lines + 2
               call pacmvt( ierr, 1, nw, nr, ll, 2,                                 &
                            baytmp, kova, .true., noueq, .false., -1,               &
                            boik, cstdf, btmp, nomv(1:1), 1, nomv(j),               &
                            bmat, lbmat, rep(:,ll:ll-1+nw), ys2, yqg, v, ag, supg,  &
                            cale(:,ll:ll-1+nw), wk1(1:nwt) )
               ll = ll + nw
               leru = leru .or. (ierr /= 0)
            end do

         else
            write(imp,777)
            lines = lines + 2
            leru = .true.
         end if

      end if

   end if

   ! Multivariate procedures (if relevant)
   ! -----------------------
      ! If only data reduction is requested, no output at this stage
   if ( lrduce .or. (lmulti .and. nvar > 1) ) then
      multv = 1
      nomeq = .false.
      prtc  = .false.

      if ( lmulti .and. nvar > 1 ) then
         multv = 2
         nomeq = noeqm
         prtc  = .true.
         nladd = 19
         if ( nomeq ) nladd = 23
         if ( flag ) nladd = nladd + 2
         call chkpg( nladd, .false., newp )
         if ( newp ) then
            if ( .not. linter ) then
               write(imp,111) cmptit(:lcmpt)
               write(imp,111) '  '
               lines = lines + 2
            end if
         else
            write(imp,111) '  '
            lines = lines + 1
         end if
         write(imp,2222)
         lines = lines + 4
         if ( flag ) then
            write(imp,111) msgv, '  '
            lines = lines + 2
         end if
      end if

      call pacmvt( ierr, nw, nvar, nr, 1, multv,                      &
                   baytmp, kova, .true., nomeq, prtc, iuw,            &
                   boik, cstdf, btmp, nomv(1:nvar), nvar, '&multiv.', &
                   bmat, lbmat, rep, ys2, yqg, v, ag, supg, cale, wk1 )
      lerr = ierr /= 0

   end if

   lerr = lerr .or. leru

   INCLUDE 'pacumtf.'

END SUBROUTINE pacumt
