SUBROUTINE pacuvt( kova, nokod, nvar, nr, df1, l1df1,       &
                   cstdf, cstb, racst, iuss, baylev, nomv,  &
                   hyp0, xlim, bmat, lbmat, rep, ys2, yqg,  &
                   v, ag, supg, ene, qne, l_ne )
!***********************************************************************
!     - pacuvt -
!     computes and prints UniVariate Tests, assuming circularity if more
!     than 1 W-contrast
!***********************************************************************
!
!     KOVA  - INPUT . Status code regarding covariates:       - INTEGER
!                       0 : procedure applies to dep. var.
!                           with no covariates
!                       1 : procedure applies to dep. var.
!                           adjusted by covariates
!                      -1 : procedure applies to slopes of
!                           dep. var. for covariates
!     NOKOD - INPUT . Code for test without equal variances   - INTEGER
!                     (considered only if NCW=1 as otherwise
!                     redundant with multivariate approach to
!                     repeated measures)
!                     <0: available;  >=0: not available:
!                      0: not (yet) defined
!                      1: insufficient group-counts
!     NVAR  - INPUT . Number of variables                     - INTEGER
!     NR    - INPUT . Number of rows in REP                   - INTEGER
!     DF1   - INPUT . Df of the comparison                    - REAL*8
!     L1DF1 - INPUT . .true. if DF1 is one                    - LOGICAL
!     CSTDF - INPUT . The within df associated to the         - REAL*8
!                     comparison on relevant groups
!                     (if equal variances assumed)
!     CSTB  - INPUT . A coefficient involved in computation   - REAL*8
!                     of scale index of Bayes distribution
!                     It is the square of component of norm
!                     of G-contrast vectors (or 1.0 when the
!                     procedure applies to slopes of dep. var.
!                     for covariates) (May be multiplied by
!                     a constant when covariates)
!                     Used only if comparison is product form
!     RACST - INPUT . A scalar involved in computation of sum - REAL*8
!                     of squares:
!                     the reciprocal of the scalar "a" such
!                     that W'*W = a*Iw  (Iw being the identity
!                     matrix of order NCW)
!                     Used only if comparison is product form
!     IUSS  - INPUT . Scratch unit number used to store       - INTEGER
!                     ANOVA table information when requested
!                     (saving will be effective only if the
!                     comparison is product form)
!     BAYLEV- INPUT . Bayes-probability level                 - REAL*8
!     NOMV  - INPUT .    see definition of MEMB arrays        - CHAR
!     HYP0  - INPUT .    see definition of MEM2 arrays        - REAL*8
!     XLIM  - INPUT .    see definition of MEM2 arrays        - REAL*8
!     BMAT  - INPUT . A corrective symmetric matrix, of order - REAL*8
!                     NR, stored in triangular form (arising
!                     from adjustment by covariates)
!                     Used only if KOVA .ne. 0
!     LBMAT - INPUT . Actual dimension of BMAT                - INTEGER
!                     (NR*(NR+1)/2 or 1 if not used)
!     REP   - INPUT . Representant of effect with NR rows     - REAL*8
!                     and NCW*NVAR columns
!     YS2   - INPUT .    see definition of MEM2 arrays        - REAL*8
!                     --- THE FOLLOWING ARRAYS ARE USED
!                     --- ONLY IF NOKOD IS NEGATIVE
!     YQG   - INPUT .    see definition of MEM2 arrays        - REAL*8
!     V     - INPUT .    see definition of MEM2 arrays        - REAL*8
!     AG    - INPUT .    see definition of MEM2 arrays        - REAL*8
!     SUPG  - INPUT .    see definition of MEM2 arrays        - INTEGER
!     ENE   - INPUT .    see definition of MEM2 arrays        - REAL*8
!                     Used only if 1 df and NOKOD < 0
!     QNE   - INPUT .    see definition of MEM2 arrays        - REAL*8
!                     Used only if 1 df and NOKOD < 0
!     L_NE  - INPUT . Size of arrays ENE and QNE              - INTEGER
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     pacint : nbayes
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  FANNP  PACFEQ  PACFID  PACFNO
!     External functions called:
!        BETACDF  DFMTC
!     Fortran  functions called:
!        MAX  MIN  SQRT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacdtr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: df1, cstdf, cstb, racst, baylev
   INTEGER, INTENT(IN) :: kova, nokod, nvar, nr, iuss, lbmat, l_ne
   LOGICAL, INTENT(IN) :: l1df1
   REAL(KIND=8), INTENT(IN) :: bmat(lbmat), rep(nr,ncw*nvar)
   REAL(KIND=8), INTENT(IN) :: hyp0(nvar), xlim(nvar)
   REAL(KIND=8), INTENT(IN) :: ys2(nwdct), yqg(nwdctg), v(ngrp,ncv)
   REAL(KIND=8), INTENT(IN) :: ag(ngrp), ene(l_ne), qne(l_ne)
   INTEGER, INTENT(IN) :: supg(ngsup)
   CHARACTER(LEN=*), INTENT(IN) :: nomv(nvar)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: betacdf
   CHARACTER(LEN=6), EXTERNAL :: dfmtc

   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8
   REAL(KIND=8), PARAMETER :: tolzer=1.0e-30_8
      ! tolzer is a tolerance for zero used for outputs

   REAL(KIND=8) :: blvl, btmp, cm1, dff, df2, ecal, efmg
   REAL(KIND=8) :: fmin, ftst, prob, prof
   REAL(KIND=8) :: qadj, scal, scstb, stdv, test
   REAL(KIND=8) :: xneg, xnot
   INTEGER :: i, iapp, ierr, j, jj, jr, kapp, kap2, kodno
   INTEGER :: nkova, nladb1, nladb2, nladc, nladd, nladf, nvv, nw
   LOGICAL :: bcor, fapp, fint, flmt, lblev, newp, noeq, sanova, sbayes
   CHARACTER(LEN=60) :: hyplim
   CHARACTER(LEN=13) :: txc
   CHARACTER(LEN=9) :: txf
   CHARACTER(LEN=8) :: txp
   CHARACTER(LEN=4) :: nob

   INCLUDE 'pacuvtp.'

   !--------------------------------------------------------------------

      ! If df1 is one, then Student's t rather than Fisher-Snedecor's F
      ! is printed
      ! The probabilities are computed using the Beta distribution
      !
      ! The 2 integer values preceding nomv(j) on Bayes-save-file code
      ! respectively whether or not equal variances are assumed and
      ! whether or not the comparison has multiple df's:
      ! 1 means yes, 0 means no
      !
      ! df1/2 are numerator/denominator df
      ! As it assumes equal variances, ANOVA table is saved only
      ! if the comparison is product form
      !
      ! nlad's are line counters:
      ! nladd  is no. of lines printed for traditional inference
      !        statements, when equal variances are not assumed (4 or 6
      !        depending on whether the variable name already printed)
      ! nladb1 is no. of lines printed for bayesian statements, when
      !        equal variances are assumed (with inference on std.dev.)
      ! nladb2 is same but when equal variances are not assumed
      !        (thus no Bayesian inference on std.dev.)
      ! nladf  one line must be added for frequentist approach to the
      !        negligible/notable problem if given
      ! nladc  is no. of lines transmitted to chkpg, prior to outputs,
      !        only used to avoid a new page between outputs with and
      !        without assumptions on variances
      !
      ! hyplim is printed along with the variable name and reports
      !        entered null hypothesis (1 df) and/or negligible/notable
      !        limits

   nw     = ncw
   nvv    = ncv
   df2    = cstdf * nw
   noeq   = nokod < 0 .and. ngsup > 1
   kodno  = min( nokod, 2 )
!! sanova = lanova .and. lupvw
   sanova = lupvw   ! Always save ANOVA-table info when product form
   bcor   = kova /= 0 .and. nr > 1
   if ( kova /= 0 ) then
      nkova = ncova
      if ( kova == -1 ) nvv = ncv * ncw
   else
      nkova = 0
   end if
   if ( kova /= 0 .and. nr == 1 ) then
      btmp = cstb / bmat(1)
   else
      btmp = cstb
   end if
   scstb = sqrt( btmp )
   if ( kova == 1 .and. nr > 1 ) then
      blvl  = zero
      lblev = .false.
      fapp  = .false.
      flmt  = .false.
      fint  = .false.
   else
      blvl  = baylev
      lblev = limit .or. blvl > zero
      fapp  = lfrapp .and. lblev
      flmt  = fapp .and. limit
      fint  = fapp .and. .not. limit
   end if

   ! Prepare outputs
   ! ---------------
      ! Note that it is assumed a new page was got by calling
      ! module so that chkpg need not to be called at this point
   if ( l1df1 ) then
      write(imp,111)
      lines = lines + 1
   end if

      ! Take Bayes statements in account if relevant
   if ( lblev ) then
      if ( blvl > zero ) then
         nladb1 = 5
         nladb2 = 3
      else
         nladb1 = 2
         nladb2 = 2
      end if
   else
      nladb1 = 0
      nladb2 = 0
   end if
   nladf = 0
   if ( flmt ) nladf = 1
   if ( fint ) nladf = 2

      ! Currently no Bayes when variances not assumed equal and
      ! df > 1 (see nladc)
   nladd = 6
   nladc = 6 + nladf + nladb1
   if ( ngsup > 1 ) then
      if ( lupvw ) then
            ! Product form comparison
         if ( noeq .and. nw == 1 ) then
               ! Approximation is available when no assumption on var.
            nladd = 4
            nladc = nladc + nladd
            if ( l1df1 ) nladc = nladc + nladf + nladb2
            write(imp,222) case1//case1x
            write(imp,222) case2//case2x
            lines = lines + 2
         else
               ! Approximation not available or redundant
            write(imp,222) case1x
            lines = lines + 1
            if ( nw == 1 ) then
               if ( kodno == 2 ) write(msgno(2)(30:35),'(i6)') nokod
               write(imp,1111) msgno(kodno)
               lines = lines + 1
            end if
         end if
      else
            ! Non-product form comparison (variances cannot be
            ! reasonably assumed equal)
         if ( noeq ) then
            nladc = 6
            if ( l1df1 ) nladc = nladc + nladf + nladb2
            write(imp,222) case2x
            lines = lines + 1
         else
               ! Nothing can be computed...
            write(imp,2222)
            lines = lines + 2
            return
         end if
      end if
   end if

      ! Prepare approximation-flag (obsolete: no more used)
   kapp = 0
   kap2 = 2

      ! Prepare Bayes saving and numbering
   sbayes = lbayes .and. lblev

      ! Prepare saving of ANOVA-table information
   if ( sanova ) rewind iuss

   hyplim = ' '
   if ( flmt ) hyplim(39:46) = msglim

   ! Compute and print
   ! -----------------

   if ( l1df1 ) then
      ! One d.f. : Student's t
      ! ----------------------

      fmt0(2:3) = ' 8'
      fmt1(2:3) = ' 8'

      jj = 0
      do j = 1, nvar
         jj = jj + j

         hyplim(:32) = ' '
         if ( hyp0(j) /= zero ) write(hyplim(7:32),3333) hyp0(j)
         if ( flmt ) write(hyplim(47:60),4444) xlim(j)

         if ( lupvw ) then
            ! Variances assumed equal
            ! -----------------------
               ! Compute
            if ( ys2(jj) > zero ) then
               stdv = sqrt( ys2(jj) )
               scal = scstb * stdv
               ecal = rep(1,j) / stdv
               ftst = rep(1,j) / scal
               test = ftst * ftst
               test = test / (test+df2)
               prob = betacdf( test, half, half*df2, ierr )
               if ( ierr == 0 ) then
                  prob = half * (one-prob)
               else
                  prob = -one
               end if
                  ! Frequentist approach to negligible/notable problem
               if ( fapp ) call fannp( rep(1,j), scal, one, df2, flmt, &
                                       xlim(j), blvl, prof, fmin, dff, &
                                       i, xneg, xnot, iapp, ierr )
               txc = ' '
               txf = ' '
               txp = ' '
               write(txc,11111) ecal
               write(txf,22222) ftst
               write(txp,33333) prob
            else
               stdv = zero
               ecal = zero
               scal = zero
               ftst = zero
               prob = -one
               fmin = zero
               prof = -one
               txc = '    /        '
               txf = '    /    '
               txp = '     /  '
            end if
               ! Save ANOVA information
            if ( sanova ) then
               cm1 = rep(1,j)*rep(1,j)*racst/btmp
               write(iuss) nomv(j), cm1, racst*ys2(jj),   &
                           df1, df2, ftst*ftst, prob+prob, kapp
            end if
               ! Save Bayes information
            if ( sbayes ) then
               nbayes = nbayes + 1
               write(nob,9999) nbayes
               write(ibayes,8888) nob, 1, nkova, kova, 1, 0, nomv(j), &
                                  rep(1,j), stdv, scal, scstb, cstdf, nvv, ncw
            else
               nob = '    '
            end if
               ! Print
               ! (4 lines must be reserved for pacfid outputs)
            call chkpg( nladc, .false., newp )
            write(imp,333) nomv(j), hyplim
            write(imp,444) nob, callab, calund
            write(imp,555) rep(1,j), txc, scal, txf, df2, txp, mapp(kapp)
            if ( flmt ) write(imp,5555) msgfrq, typef(i), sqrt(fmin), df2, prof, mapp(kapp)
            if ( fint ) then
               xneg = max( xneg, zero )
               fmt0(15:20) = dfmtc( 10, -6, xneg, tolzer )
               write(imp,fmt0) msgfrq, typef(0), msgeff, xneg, blvl, mapp(kapp)
               xnot = max( xnot, zero )
               fmt1(15:20) = dfmtc( 10, -6, xnot, tolzer )
               write(imp,fmt1) msgfrq, typef(1), msgeff, xnot, blvl, mapp(kapp)
            end if
            lines = lines + 6 + nladf
               ! Compute and print Bayes-fiducial limits
            if ( lblev .and. scal > zero )      &
               call pacfid( blvl, rep(1,j), scal, stdv, one, df2, l1df1, .true., 0, 0, xlim(j) )
         end if

         if ( noeq ) then
            ! Variances not assumed equal
            ! ---------------------------
               ! Compute
            if ( qne(j) > zero ) then
               ftst = rep(1,j) / ene(j)
               test = ftst * ftst
               test = test / (test+qne(j))
               prob = betacdf( test, half, half*qne(j), ierr )
               kap2 = kapp + 2
               if ( ierr == 0 ) then
                  prob = half * (one-prob)
               else
                  prob = -one
               end if
                  ! Frequentist approach to negligible/notable problem
               if ( fapp ) call fannp( rep(1,j), ene(j), one, qne(j), flmt,  &
                                       xlim(j), blvl, prof, fmin, dff,       &
                                       i, xneg, xnot, iapp, ierr )
               txf = ' '
               txp = ' '
               write(txf,22222) ftst
               write(txp,33333) prob
            else
               ftst = zero
               prob = -one
               fmin = zero
               prof = -one
               txf = '    /    '
               txp = '     /  '
            end if
               ! Save Bayes information
            if ( sbayes ) then
               nbayes = nbayes + 1
               write(nob,9999) nbayes
               write(ibayes,8888) nob, 1, nkova, kova, 0, 0, nomv(j),  &
                                  rep(1,j), zero, ene(j), scstb, qne(j), nvv, ncw
            else
               nob = '    '
            end if
               ! Print
               ! (3 lines must be reserved for pacfid outputs)
            if ( lupvw ) then
               write(imp,*)   '  '
            else
               call chkpg( nladd+nladf+nladb2, .false., newp )
               write(imp,333) nomv(j), hyplim
            end if
            write(imp,444) nob, calblk, calblk
            write(imp,666) rep(1,j), ene(j), txf, qne(j), txp, mapp(kap2)
            if ( flmt ) write(imp,5555) msgfrq, typef(i), sqrt(fmin), qne(j), prof, mapp(kap2)
            if ( fint ) then
               xneg = max( xneg, zero )
               fmt0(15:20) = dfmtc( 10, -6, xneg, tolzer )
               write(imp,fmt0) msgfrq, typef(0), msgeff, xneg, blvl, mapp(kap2)
               xnot = max( xnot, zero )
               fmt1(15:20) = dfmtc( 10, -6, xnot, tolzer )
               write(imp,fmt1) msgfrq, typef(1), msgeff, xnot, blvl, mapp(kap2)
            end if
            lines = lines + nladd + nladf
               ! Compute and print Bayes-fiducial limits
            if ( lblev .and. ene(j) > zero )                          &
               call pacfid( blvl, rep(1,j), ene(j), zero,             &
                            one, qne(j), l1df1,.false., 1, 0, xlim(j) )
         end if

      end do

   else
      ! Multiple d.f. : Fisher-Snedecor's F
      ! -----------------------------------

      if ( noeq ) then
            ! jr points to relevant subset in rep for current variable
         jr = 1
      end if
      fmt0(2:3) = '16'
      fmt1(2:3) = '16'

      do j = 1, nvar

         if ( flmt ) write(hyplim(47:60),4444) xlim(j)

         if ( lupvw ) then
            ! Variances assumed equal
            ! -----------------------
               ! Compute test statistics
            call pacfeq( bcor, btmp, bmat, lbmat, ys2, rep, j, nr, nw, efmg, stdv, ftst, scal )
            if ( scal > zero ) then
               ecal = efmg / stdv
               test = df1 * ftst
               test = test / (test+df2)
               prob = betacdf( test, half*df1, half*df2, ierr )
               if ( ierr == 0 ) then
                  prob = one - prob
               else
                  prob = -one
               end if
                  ! Frequentist approach to negligible/notable problem
               if ( fapp ) call fannp( efmg, scal, df1, df2, flmt,     &
                                       xlim(j), blvl, prof, fmin, dff, &
                                       i, xneg, xnot, iapp, ierr )
               txc = ' '
               txf = ' '
               txp = ' '
               write(txc,11111) ecal
               write(txf,22222) ftst
               write(txp,33333) prob
            else
               ecal = zero
               prob = -one
               fmin = zero
               dff  = zero
               prof = -one
               txc = '    /        '
               txf = '    /    '
               txp = '     /  '
            end if
               ! Save ANOVA information
            if ( sanova ) then
               cm1 = efmg*efmg*racst/btmp
               write(iuss) nomv(j), cm1, racst*stdv*stdv, df1, df2, ftst, prob, kapp
            end if
               ! Save Bayes information
            if ( sbayes ) then
               nbayes = nbayes + 1
               write(nob,9999) nbayes
               write(ibayes,8888) nob, 1, nkova, kova, 1, 1, nomv(j),  &
                                  efmg, stdv, scal, scstb, cstdf, nvv, ncw
            else
               nob = '    '
            end if
               ! Print
               ! (4 lines must be reserved for pacfid outputs)
            call chkpg( nladc, .false., newp )
            write(imp,333) nomv(j), hyplim
            write(imp,777) nob, callab, calund
            write(imp,888) efmg, txc, scal, txf, df1, df2, txp, mapp(kapp)
            if ( flmt ) write(imp,6666) msgfrq, typef(i), fmin, dff, df2, prof, mapp(kapp)
            if ( fint ) then
               xneg = max( xneg, zero )
               fmt0(15:20) = dfmtc( 10, -6, xneg, tolzer )
               write(imp,fmt0) msgfrq, typef(0), msgeff, xneg, blvl, mapp(kapp)
               xnot = max( xnot, zero )
               fmt1(15:20) = dfmtc( 10, -6, xnot, tolzer )
               write(imp,fmt1) msgfrq, typef(1), msgeff, xnot, blvl, mapp(kapp)
            end if
            lines = lines + 6 + nladf
               ! Compute and print Bayes-fiducial limits
            if ( lblev .and. scal > zero )                         &
               call pacfid( blvl, efmg, scal, stdv,                &
                            df1, df2, l1df1, .true., 0, 0, xlim(j) )
         end if

         if ( noeq .and. nw == 1 ) then
            ! Variances not assumed equal
            ! ---------------------------
            ! (The case nw>1 is treated by the multivariate approach
            ! to repeated measures)
            ! (Remember that non-product forms with multiple df's are
            ! not currently allowed, so that efmg is already defined)
!!!! ??????????????? a verifier  QADJ SCAL pour Bayes
               ! Compute test statistics
            call pacfno(nr, nw, df1, jr, rep(1:,jr), yqg, v, ag, supg, ftst, qadj, ierr )
            jr = jr + nw
            if ( ftst > zero .and. qadj > zero ) then
               scal = efmg / sqrt(ftst)
               test = df1 * ftst
               test = test / (test+qadj)
               prob = betacdf( test, half*df1, half*qadj, ierr )
               kap2 = kapp + 2
               if ( ierr == 0 ) then
                  prob = one - prob
               else
                  prob = -one
               end if
                  ! Frequentist approach to negligible/notable problem
!!             if ( fapp ) call fannp( efmg, scal, df1, qadj, flmt,    &
!!                                     xlim(j), blvl,                  &
!!                                     prof, fmin, dff, i, xneg, xnot, &
!!                                     iapp, ierr )
               txf = ' '
               txp = ' '
               write(txf,22222) ftst
               write(txp,33333) prob
            else
               scal = zero
               ftst = zero
               prob = -one
               fmin = zero
               dff  = zero
               prof = -one
               txf = '    /    '
               txp = '     /  '
            end if
               ! Save Bayes information
!!          if ( sbayes ) then
!!             nbayes = nbayes + 1
!!             write(nob,9999) nbayes
!!             write(ibayes,8888) nob, 1, nkova, kova, 0, 1, nomv(j), &
!!                                efmg, zero, scal, scstb, qadj,      &
!!                                nvv, ncw
!!          else
!!             nob = '    '
!!          end if
               ! Print
               ! (3 lines must be reserved for pacfid outputs)
            if ( lupvw ) then
               write(imp,*)   '  '
            else
!!             call chkpg( nladd+nladf+nladb2, .false., newp )
!!             write(imp,333) nomv(j), hyplim
               call chkpg( nladd, .false., newp )
               write(imp,333) nomv(j), ' '
            end if
!!          write(imp,777) nob, calblk, calblk
!!          write(imp,999) efmg, scal, txf, df1, qadj, txp, mapp(kap2)
!!          if ( flmt ) write(imp,6666) msgfrq, typef(i), fmin, dff, qadj, prof, mapp(kap2)
!!          if ( fint ) then
!!             xneg = max( xneg, zero )
!!             fmt0(15:20) = dfmtc( 10, -6, xneg, tolzer )
!!             write(imp,fmt0) msgfrq, typef(0), msgeff, xneg, blvl, mapp(kap2)
!!             xnot = max( xnot, zero )
!!             fmt1(15:20) = dfmtc( 10, -6, xnot, tolzer )
!!             write(imp,fmt1) msgfrq, typef(1), msgeff, xnot, blvl, mapp(kap2)
!!          end if
!!          lines = lines + nladd + nladf
            write(imp,777) '    ', calblk, calblk
            write(imp,999) efmg, scal, txf, df1, qadj, txp, mapp(kap2)
            lines = lines + nladd
               ! Compute and print Bayes-fiducial limits
!!          if ( lblev .and. scal > zero )            &
!!             call pacfid( blvl, efmg, scal, zero,   &
!!                          df1, qadj, l1df1,.false., 1, 0, xlim(j) )
         end if

      end do

   end if

      ! Print warning if ANOVA-table request has been cancelled
   if ( lanova .and. .not. sanova ) then
      call chkpg( 2, .false., newp )
      write(imp,7777)
      lines = lines + 2
   end if

   INCLUDE 'pacuvtf.'

END SUBROUTINE pacuvt
