SUBROUTINE pacy( ndf1, lonce, lform, balang, balant, npart, nrsavd,  &
                 part, gwei, twei, rtyp, baylev, samew, prvldp, lerr )
!***********************************************************************
!     - pacy -
!     directs computations on relevant derived protocol "Y"
!***********************************************************************
!
!     NDF1  - INPUT . Degrees of freedom of the comparison    - INTEGER
!     LONCE - INPUT . Whether the covariates were measured    - LOGICAL
!                     only once or for all treatments
!     LFORM - INPUT . Whether the comparison was entered by   - LOGICAL
!                     a formula or by vectors
!     BALANG- INPUT . .true. if equal group counts            - LOGICAL
!                     (design balanced for groups)
!     BALANT- INPUT . .true. if equal treatment weights       - LOGICAL
!                     (design balanced for treatments)
!     NPART - INPUT . Number of parts in the partition matrix - INTEGER
!                     PART resulting from formula analysis
!                     (0 means that PART is not available)
!     NRSAVD- INPUT . Number of regression-variable vectors   - INTEGER
!                     saved on scratch unit IU4
!                     (relevant only if NPART > 0)
!     PART -  INPUT . PART matrix                             - INTEGER
!                     (relevant only if NPART > 0)
!             OUTPUT. Note that this array will be destroyed
!                     on output
!     GWEI  - INPUT . Vector of group weights, according      - REAL*8
!                     to the weighting option,
!                     of length NGRP
!                     (relevant only if NPART > 0)
!     TWEI  - INPUT . Vector of treatment weights, according  - REAL*8
!                     to the weighting option,
!                     of length NTRT
!                     (relevant only if NPART > 0)
!     RTYP  - INPUT . Regression type:                        - INTEGER
!                        0 = no regression requested
!                        1 = degree type
!                        2 = cumulative type
!                     (a negative value for type codes the
!                     corresponding residual comparison)
!     BAYLEV- INPUT . Bayes-probability level                 - REAL*8
!     SAMEW - INPUT . Flags results from previous comparison  - LOGICAL
!                     can be used for current one
!                     N.B. Must be completed by checking same
!                     G-comparison-support and same W-contrast
!             OUTPUT. Updated
!     PRVLDP- INPUT . Whether previous derived protocol was   - LOGICAL
!                     printed
!             OUTPUT. Reset to .true. if derived protocol
!                     was printed and no data reduction step;
!                     reset to .false. if derived protocol
!                     was requested and data reduction step,
!                     or if derived protocol not requested
!                     and SAMEW is .false. on output
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!                     (thus indicating no results available
!                     for the next comparison)
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     memst2 : udpw  supg  ybar  ys2  eff  ene  qne  yqg
!              l_udpw  l_ene  l_qne
!     pacint : gcount  nwdc  nwdct  nwdctg  ranka
!     paclgl : lrduce
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  FILOPN  GMP999  MABC  MASTAZ  MSINV  MSSZ  MSXC  MTASAZ
!        MXBS  PACADJ  PACCST  PACDFC  PACDPA  PACDPB  PACDPL  PACMVT
!        PACPPM  PACPRY  PACUMT  PACYDS  PACYSS  PACZ  PAC999  PRTMAT
!        PRTTAM
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ALLOCATE  ALLOCATED  DEALLOCATE  HUGE  LEN_TRIM  MAX  MIN  MOD
!        NINT  RESHAPE  SQRT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memst2
   USE memsta
   USE memstb
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: baylev
   INTEGER, INTENT(IN) :: ndf1, npart, nrsavd, rtyp
   LOGICAL, INTENT(IN) :: lonce, lform, balang, balant
   REAL(KIND=8), INTENT(IN) :: gwei(ngrp), twei(ntrt)
   INTEGER, INTENT(INOUT) :: part(ntrt,ngrp)
   LOGICAL, INTENT(INOUT) :: samew, prvldp
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, two=2.0_8
   REAL(KIND=8) :: agmin, bcst, boik, csta, df1, qadj, qcst, racst, rqb
   REAL(KIND=8) :: sumv, sumw, temp
   INTEGER :: i, ierr, ii, ij, iout, iudp, iupr, iuss, iuw
   INTEGER :: j, jk, judp, k, ks, lcmpt, ldc, leffc, ll, lll, minag, multv
   INTEGER :: ncasin, ngprt, nokod, nssav, nv, nvar, nvt
   INTEGER :: nw, nwcot, nwcov, nwcwd, nwdep, nwdet, status
   LOGICAL :: ldpcov, ldptmp, lerr1, lschef, l1df1
   LOGICAL :: needdp, newp, noeq, noeqm, noequ

   REAL(KIND=8), ALLOCATABLE :: wrk0(:), wrk1(:), wrk2(:), wrk3(:)
   REAL(KIND=8), ALLOCATABLE :: zbar(:,:)
   CHARACTER(LEN=8) :: blank(1)=(/ '        ' /)
   CHARACTER(LEN=nocw) :: labv(ncv), labw(ncw), nomwr(ncw*nvtot)
   CHARACTER(LEN=nocw), ALLOCATABLE :: eign(:)

   INCLUDE 'pacyp.'

   !--------------------------------------------------------------------

   !          - PACY - Length and use of numerical work arrays
   !  Notation:
   !  x = set of dependent variables   z = set of covariates
   !  nd  = ndep       nc   = ncova    ndc = nvtot
   !  nv  = ncv        nw   = ncw      nwd = nw*ndep
   !  nwc = nw*ncova   nwdc = nw*ndc   n$  = n(n+1)/2
   !  in(out) = array is an input(output) of the procedure
   !
   !                                                     |===============
   ! |    WRK0    |    WRK1    |    WRK2    |   WRK3     |  STEP 1
   ! +------------+------------+------------+------------+===============
   ! |            |         out|            |            |  PACDFC
   ! |            |     ndc    |            |            |  if nw>1
   ! +------------+------------+------------+------------+
   ! |         out|     "   in |         in |            |  PACUMT
   ! | nv.nwdc    | ndc  (nw>1)|   1        |            |
   ! +------------+------------+------------+------------+===============
   ! |                                                   |  STEP 2
   ! +------------+--------------------------------------+===============
   ! |     "      |  covariance matrix for dep.var.      |
   ! |            |    nwd$                              |
   ! +------------+------------+------------+------------+
   ! |         in |      in/out|         in |            |  PACMVT
   ! | nv.nwd     |    nwd$    |   1        |            |
   ! +------------+------------+------------+------------+===============
   ! |                                                   |  STEP 3
   ! +------------+--------------------------------------+===============
   ! |     "      |  covariance matrix for covariates    |
   ! |            |    nwc$                              |
   ! +------------+------------+------------+------------+
   ! |         in |      in/out|         in |            |  PACMVT
   ! | nv.nwc     |    nwc$    |   1        |            |
   ! +------------+------------+------------+------------+===============
   ! |                                                   |  STEP 4
   ! +------------+------------+------------+------------+===============
   ! |            |  YQGzz**-1 |            |            |  MSINV
   ! |            |    nwc$    |            |            |  regression
   ! +------------+------------+------------+------------+  coefficients
   ! |  nwc.nwd   |     "      |            |            |  per group
   ! |(write IUSS)|            |            |            |  MSXC
   ! +------------+------------+------------+------------+
   ! |            |            |  Zz=(YS2zz)**(-1/2)     |
   ! |            |            |   nwc$  (read IUW)      |
   ! +------------+------------+------------+------------+
   ! |            |  Sz=Zz**2  |     "  in  |            |  MSSZ
   ! |            |    nwc$    |            |            |
   ! +------------+------------+------------+------------+
   ! |         in |     "   in |  Dz.Sz.Dz'              |  MASTAZ
   ! |     nwc    |            |   nv$  (write IUW)      |
   ! +------------+------------+------------+------------+
   ! |Pzx=Sz.YS2zx|     "   in |            |            |  MSXC
   ! |  nwc.nwd   |            |            |            |
   ! +------------+------------+------------+------------+
   ! |     "   in |   Dz.Pzx   |            |            |  MABC
   ! |            |    nv.nwd  |            |            |
   ! +------------+------------+------------+------------+
   ! |     "   in |  YS2xz.Pzx |            |            |  MXBS
   ! |            |    nwd$    |            |            |
   ! +------------+------------+------------+------------+
   ! |     "      |     "      |  Qgz=YQgzz |            | )
   ! |            |            |   nwc$     |            | ) if printing
   ! +------------+------------+------------+------------+ )
   ! |     "      |Pzx'.Qgz.Pzx|     "      |            | ) MTASAZ
   ! |            |    nwd$    |            |            | )
   ! +------------+------------+------------+------------+ )
   ! |     "      |     "      |  YQgxz.Pzx |            | ) MXBS
   ! |            |            |   nwd$     |            | )
   ! +------------+------------+------------+------------+
   ! |     "   in |         out|            |            |  PACADJ
   ! |            |    nwc     |            |            | if deriv/print
   ! +------------+------------+------------+------------+
   ! |     "   in |     "   in |            |            |  PACDPB
   ! |            |            |            |            | if deriv/print
   ! +------------+------------+------------+------------+
   ! |            |            |            |         out|  PACDFC
   ! |            |            |            |    nd      |  if nw>1
   ! +------------+------------+------------+------------+
   ! |            |            |(read IUSS) |            |  if printing
   ! |            |            |  nwc.nwd   |            |
   ! +------------+------------+------------+------------+
   ! |            | Qzz=qYS2zz |            |            |
   ! |            |  nwc$      |            |            |
   ! +------------+------------+------------+------------+
   ! |     "   in |            |            |     "   in |  PACYSS
   ! +------------+------------+------------+------------+
   ! |     "   in |     "   in |     "   out|     "   in |  PACUMT
   ! +------------+------------+------------+------------+
   ! | B (readIUW)|            |            |            |
   ! |    nv$     |            |            |            |
   ! +------------+------------+------------+------------+
   ! |B=Iv+B/(qb2)|            |            |            |
   ! |    nv$     |            |            |            |
   ! +------------+------------+------------+------------+
   ! |   B**-1    |            |            |            |  MSINV
   ! |    nv$     |            |            |            |
   ! +------------+------------+------------+------------+
   ! |     "      |            |            |            |  PACYSS
   ! |            |            |            |            |
   ! +------------+------------+------------+------------+
   ! |     "   in |            |         out|     "   in |  PACUMT
   ! |            |            |  nv.nwd    |            |
   ! +------------+------------+------------+------------+

   lerr = .false.

   ! Copy variables (to avoid double argument-transmission with modules)
   ! --------------
   iout = imp
   nv = ncv
   nw = ncw
   ncasin= nindvu

   ! Some usefull constants
   ! ----------------------
   nwdc = ncw * nvtot
   nwdct = nwdc*(nwdc+1) / 2
   nvt = ncv*(ncv+1)/2
   ranka = 1

      ! Length of comparison-title for printing
   lcmpt = min( lrecp-1, len_trim(cmptit) )

   ! Print head lines
   ! ----------------
   call chkpg( 10, .false., newp )
   if ( .not. newp ) then
      write(imp,333) '  '
      lines = lines + 1
   end if
   write(imp,111) nprob, ncomp
   write(imp,333) cmptit(:lcmpt)
   write(imp,'(1x,78a1)') ('<',i=1,78)
   lines = lines + 3

   ! Set flag for data reduction step
   ! --------------------------------
   lrduce = lreduc .and. lupvw .and. ncova == 0 .and. ncv*nwdc > 1

   ! Degrees of freedom of the comparison
   ! ------------------------------------
   df1 = ndf1
   l1df1 = ndf1 == 1

   ! Scratch units used
   ! ------------------
      ! iupr is scratch-unit-number for storage of previous results
      !      (set to iu3 that was opened in pacin1)
      ! iuss is scratch-unit-number for various purposes:
      !      . When lanova: ANOVA-table saving
      !      . When lpmean: temporary storage of results from pacreg
      !        and polreg, and is then used in pacppm
      !      . When ncova > 0 and several groups are involved:
      !        temporary storage of covariate regression coefficients
      !        for each group
      !      (set to iu4 that was opened in pacin1)
      ! iuw  is scratch-unit-number for saving by pacmvt or pacmv2
      !      when ncova > 0 or lrduce
      ! iudp is scratch-unit-number for derived procotol when temporary
      !      storage is needed (ncova > 0 or printing requested but
      !      no user's save file)
      !      it is set to proper value on return from paccst
      ! judp is additional scratch-unit-number for derived procotol
      !      when temporary storage is needed (ncova > 0 and printing
      !      requested but no user's save file)
      !      It is set to proper value in step 4

      ! Following informations will be saved on iuw in that order:
      !      - Square root of inverse of relevant section of ys2
      !           and if reduction step is requested and relevant:
      !      - Matrix of calibrated effect (from which the "core"
      !        matrix to be diagonalized is built)
      !      - Order of core matrix, its theoretical rank, return code
      !        from diagonalization, trace of core, re-computed trace,
      !        a zero-tolerance for eigenvalues
      !      - All eigenvalues of core  (if return code is 0)
      !      - All eigenvectors of core (if return code is 0)

   iupr = iu3
   iuss = iu4
   iuw  = -1
   iudp = -1
   judp = -1
   if ( lrduce .or. (lupvw .and. ncova > 0) ) then
      iuw = junit1
      call filopn( 0, iuw, ' ', 'SCRATCH', 'UNFORMATTED', .false., lerr )
      if ( lerr ) goto 20
   end if

   ! Print partition matrix and associated means
   ! -------------------------------------------
   if ( lpmean .and. npart > 0 ) then
      call pacppm( balang, balant, npart, rtyp, nrsavd, max(1,nrsavd), &
                   part, gapp, xbar, at, gwei, twei, labg, labt, nomv )
   end if

   ! Get constants (ngsup and gsup known on return from paccst)
   ! -------------
      ! Allocate mem2 array udpw
   if ( .not. lupvw ) then
      l_udpw = ngrp
   else
      l_udpw = 1
   end if
   if ( allocated(udpw) ) deallocate(udpw)
   allocate ( udpw(l_udpw), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacY)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if
      ! Prepare check for previous results utilization
   if ( samew ) rewind iupr
   call paccst( gsup, ag, at, v, w, udpw, l_udpw, qcst, bcst, racst,  &
                .not. lform .and. ncova > 0, sumv, sumw, samew, iupr )

   gcount = nint(qcst) + ngsup

      ! ngprt is number of groups involved in printing of means and
      ! standard deviations per group: ngsup if these printings are
      ! requested and 0 otherwise
   if ( levout(3) ) then
      ngprt = ngsup
   else
      ngprt = 0
   end if

   ! Flag for Scheffe's method for multiple comparisons
   ! --------------------------------------------------
   lschef = lupvw .and. lanova .and. .not. lspecf .and. ncv < ngrp .and. sumv == zero

   ! Check covariates may be used
   ! ----------------------------
   if ( ncova > 0 .and. .not. lupvw ) then
      call chkpg( 4, .false., newp )
      write(imp,999)
      lines = lines + 4
      write(lgu,*) ermrk1//ermrk1
      write(lgu,999)
      write(lgu,*) ermrk2//ermrk2
      lerr = .true.
      goto 20
   end if

   ! Is temporary storage of derived protocol needed ?
   ! -------------------------------------------------
   needdp = lderiv .or. listdp
   ldpcov = needdp .and. ncova > 0 .and. (.not. lonce .or. sumw /= zero)
   ldptmp = ldpcov .or. (needdp .and. .not. lderiv)
      ! If so, assign proper unit to iudp and open it
   if ( ldptmp ) then
      iudp = junit1
      if ( iudp == iuw ) iudp = junit2
      call filopn( 0, iudp, ' ', 'SCRATCH', 'UNFORMATTED', .false., lerr )
      if ( lerr ) goto 20
   end if

   ! Allocate mem2 arrays
   ! --------------------
   nwdctg = ngsup*nwdct
   if ( l1df1 ) then
      l_ene = nvtot
   else
      l_ene = 1
   end if
   if ( allocated(supg) ) deallocate(supg)
   if ( allocated(ybar) ) deallocate(ybar)
   if ( allocated(ys2) ) deallocate(ys2)
   if ( allocated(eff) ) deallocate(eff)
   if ( allocated(ene) ) deallocate(ene)
   if ( allocated(qne) ) deallocate(qne)
   if ( allocated(yqg) ) deallocate(yqg)
   allocate ( supg(ngsup), ybar(nwdc,ngsup), ys2(nwdct), eff(ncv,nwdc),  &
              ene(l_ene), qne(l_ene), yqg(nwdctg), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacY)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if

   ! Set supg, agmin
   ! ---------------
      ! agmin is the minimum group-count among the G-support groups
   agmin = huge(zero)
   ks = 1
   do k = 1, ngrp
      if ( gsup(k) > 0 ) then
         supg(ks) = k
         ks = ks + 1
         if ( ag(k) < agmin ) agmin = ag(k)
      end if
   end do
   minag = nint( agmin )

      ! If needed, create Variable-W-contrast names
   if ( nw > 1 ) then
      k = 1
      do i = 1, nvtot
         nomwr(k) = nomv(i)
         k = k + 1
         do j = 2, nw
            write(nomwr(k),222) '...w', j, ' '
            k = k + 1
         end do
      end do
   else
      nomwr(1:nvtot) = nomv
   end if
      ! V/W-contrast names (maybe used further)
   labv(1) = blank(1)
      ! If needed, create V-contrasts names
   if ( (levout(1) .or. levout(2)) .and. nv > 1 ) then
      do i = 1, nv
         write(labv(i),222) '*v', i,'   '
      end do
   end if
   labw(1) = blank(1)

   ! Flag for test without equal variances or circul. assumptions
   ! ------------------------------------------------------------
      ! noequ and noeqm code for univariate/multivariate tests
      ! Negative nokod codes for availability of approximation;
      ! positive nokod codes for reason for lack of approximation:
      !    0: not defined,  1: insufficient group-counts
   noeq = ngsup > 1
   noequ = noeq  .and. minag > nw+4
   if ( noequ ) then
      nokod = -1
   else
      nokod = 1
   end if

   ! Print contrast-vectors if requested
   ! -----------------------------------
   if ( levout(1) ) then
      if ( lupvw ) then
         if ( ntrt > 1 ) then
            if ( nw > 1 ) then
                  ! Create W-contrasts names
               do i = 1, nw
                  write(labw(i),222) '*w', i, '   '
               end do
            end if
            call chkpg( 5+nw, .false., newp )
            if ( newp ) then
               write(imp,333) cmptit(:lcmpt)
               lines = lines + 1
            end if
            write(imp,444) msgdcf(2), undlin(:zdcf2)
            lines = lines + 3
            call prttam( iout, .true., w, ntrt, ntrt, nw, labt, labw )
         end if
         if ( ngrp > 1 ) then
            call chkpg( 5+nv, .false., newp )
            if ( newp ) then
               write(imp,333) cmptit(:lcmpt)
               lines = lines + 1
            end if
            write(imp,444) msgdcf(1), undlin(:zdcf1)
            lines = lines + 3
            call prttam( iout, .true., v, ngrp, ngrp, nv, labg, labv )
         end if
      else
         if ( l1df1 ) then
            write(imp,444) msgdcf(3), undlin(:zdcf3)
            lines = lines + 3
            call prttam( iout, .true., w, ntrt, ntrt, ngrp, labt, labg )
         end if
      end if
   end if

   ! Print norms of contrast-vectors
   ! -------------------------------
   csta = one / sqrt( racst )
   call chkpg( 1+4, .false., newp )
   if ( newp ) then
      write(imp,333) cmptit(:lcmpt)
      lines = lines + 1
   end if
   write(imp,333) '  '
   lines = lines + 1
   if ( lupvw ) then
      temp = sqrt( bcst )
      write(imp,33) ncv, ncw, ndep, ncova, qcst
      write(imp,777) msgnmc, msgdcf(2), csta
      write(imp,777) msgnmc, msgdcf(1), temp
      write(imp,888) racst
      lines = lines + 4
   else
      call chkpg( 6, .false., newp )
      write(imp,44) ndf1, ndep, ncova, qcst
      write(imp,777) msgnm, msgdcf(3), csta
      write(imp,888)
      lines = lines + 3
      blank(1) = dpwnam
      call pacpry( iout, .true., blank, udpw, 1, ngsup, 1, labg, supg )
      blank(1) = '        '
   end if

   ! STEP 1. ANALYZE ALL VARIABLES
   ! =============================

   !===============================================================!
   ! THIS STEP IS COMMON TO ANALYSES WITH AND WITHOUT COVARIATES:  !
   ! COVARIATES, IF ANY, ARE NOT DISTINGUISHED FROM DEPENDENT VAR. !
   !===============================================================!

   if ( samew ) then
      call chkpg( 7, .false., newp )
      if ( newp ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,555)
      lines = lines + 7
   end if
   if ( ncova > 0 ) then
      call chkpg( 18, .false., newp )
      if ( newp ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,1111)
      lines = lines + 5
   end if

   ! Number of dependent variables for this step
   ! -------------------------------------------
   nvar = nvtot

   ! Derive relevant protocol "Y" and Y-statistics ("Y" possibly saved)
   ! ------------------------------------------------------------------
      ! Remember W is actually W or U depending on whether the
      ! comparison is product form, and thus it's always the
      ! appropriate array for derivation-coefficients

   call pacdpa( iu2, iudp, iupr, xbar, gnum, gapp, gsup, supg, ag, hyp0,                &
                w, v, ybar, yqg, ys2, eff, ene, qne, l_ene, qcst, racst, udpw, l_udpw,  &
                nvar, ncasin, nssav, ldptmp, ldpcov, samew, l1df1, noequ )

   ! Print y descripive statistics
   ! -----------------------------
   if ( levout(2) .and. ncova == 0 ) then
      call chkpg( 13, .false., newp )
      if ( newp ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
   end if
   if ( lupvw ) then
      temp = racst / bcst
   else
      temp = racst
   end if
   call pacyds( samew, ngprt, nvar, labg, labv, nomwr(1:ncw*nvar), nomv, .true., temp )

   ! Check it's worth continuing (more than 1 count)
   ! -----------------------------------------------
   if ( qcst <= zero ) then
      call chkpg( 3, .false., newp )
      write(imp,22)
      lines = lines + 3
      write(lgu,*) ermrk1
      write(lgu,22)
      write(lgu,*) ermrk2
         ! Set error flag to indicate no results available for next comparison
      lerr = .true.
      if ( ldptmp ) goto 20
      lrduce = .false.
      goto 10
   end if

   ! Check covariates are irrelevant (so only dep.var. are analyzed)
   ! ---------------------------------------------------------------
   if ( ncova > 0 .and. (lonce .and. sumw == zero) ) then
      call chkpg( 4, .false., newp )
      if ( newp .and. .not. linter ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,666)
      lines = lines + 4
      write(lgu,*) ermrk1//ermrk1
      write(lgu,666)
      write(lgu,*) ermrk2//ermrk2
      nvar = ndep
   end if

   ! Allocate work arrays
   ! --------------------
   if ( allocated(wrk0) ) deallocate(wrk0)
   if ( allocated(wrk1) ) deallocate(wrk1)
   if ( allocated(wrk2) ) deallocate(wrk2)
   allocate ( wrk0(nv*nw*nvar), wrk1(nvar), wrk2(1), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacY)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if

   ! Univariate and multivariate procedures
   ! --------------------------------------
      ! Departure from circularity
   if ( nw > 1 ) call pacdfc( ys2, nw, nvar, nomv, boik, wrk1(1:nvar) )

   noeqm = noeq .and. minag > nwdc+4

   if ( lrduce ) then
      ll = iuw
   else
      ll = -1
   end if

   call pacumt( nw, nvar, nv, df1, l1df1, iuss, ll, qcst, bcst, racst,      &
                baylev, nokod, lschef, noeqm, boik, 0, msgv(1), ncova > 0,  &
                eff, wrk0(1:nv*nw*nvar), wrk1(1:nvar), wrk2(1:1), 1, lerr1 )

   ! If no covariates (or irrelevant), go to final step
   ! --------------------------------------------------
   if ( ncova <= 0 .or. (lonce .and. sumw == zero) ) then
      if ( lupvw ) then
            ! Save arrays for possible use in next comparison
         rewind iupr
         write(iupr) gsup(1:ngrp)
         write(iupr) w(:,1:nw)
         write(iupr) ys2(1:nwdct)
         write(iupr) yqg(1:nwdctg)
      end if
      goto 10
   end if

             !==============================!
             ! COVARIATES -- SPECIFIC STEPS !
             !==============================!

      ! No data reduction analysis when covariates are used
   lrduce = .false.

      ! Some usefull constants
   nwdep = nw * ndep
   nwdet = nwdep*(nwdep+1)/2
   nwcov = nw * ncova
   nwcot = nwcov*(nwcov+1)/2
   nwcwd = nwcov * nwdep
      ! Some usefull sub-arrays
   leffc = nwdep + 1      ! Points to sub-array of magnitude of effect for covariates
   ldc = nwdet + 1        ! Points to sub-matrix of pooled-within-groups covariances
                          ! between cova. and dep.var.

   ! Allocate work arrays for all specific steps
   ! -------------------------------------------
   if ( allocated(wrk0) ) deallocate(wrk0)
   if ( allocated(wrk1) ) deallocate(wrk1)
   if ( allocated(wrk2) ) deallocate(wrk2)
   if ( allocated(wrk3) ) deallocate(wrk3)
   ii = max( nv*nwdep, nv*nwcov, nwcwd, nvt )
   ij = max( nwdet, nwcot, nv*nwdep )
   jk = max( ij, nwcwd )
   allocate ( wrk0(ii), wrk1(ij), wrk2(jk), wrk3(ndep), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacY)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if

   ! STEP 2. ANALYZE DEPENDENT VARIABLES ONLY
   ! ========================================

   if ( lmulti .and. ndep > 1 ) then

      call chkpg( 17, .false., newp )
      if ( newp .and. .not. linter ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,333) '  ', msgv(2), '  '
      lines = lines + 3

      ! Number of dependent variables for this step
      ! -------------------------------------------
      nvar = ndep

      ! Multivariate procedures
      ! -----------------------
      noeqm = noeq .and. minag > nwdep+4

         ! First, copy relevant subset of ys2 in wrk1
      wrk1(1:nwdet) = ys2(1:nwdet)
      call pacmvt( ierr, nw, nvar, nv, 1, 2, baylev, 0, .false., noeqm,      &
                   .true., -1, boik, qcst, bcst, nomv(1:nvar), nvar, tmpdep, &
                   wrk2(1:1), 1, eff, ys2, yqg, v, ag, supg,                 &
                   wrk0(1:nv*nw*nvar), wrk1(1:nwdet) )

   end if

   ! STEP 3. ANALYZE COVARIATES AS DEPENDENT VARIABLES
   ! =================================================

   if ( nwcov > 1 ) then

         ! Note that even if multivariate procedures are not required
         ! by user, pacmvt must be called in order to build the square
         ! root of inverse of sub-matrix of ys2 for covariates (needed
         ! in step 4), unless this sub-matrix reduces to a scalar

      if ( ncova > 1 ) then
         call chkpg( 17, .false., newp )
         if ( newp .and. .not. linter ) then
            write(imp,333) cmptit(:lcmpt)
            lines = lines + 1
         end if
         if ( lmulti ) then
            write(imp,333) '  ', msgv(3), '  '
            lines = lines + 3
         end if
      end if

      ! Number of dependent variables for this step
      ! -------------------------------------------
      nvar = ncova

      ! Multivariate procedures
      ! -----------------------
      noeqm = noeq .and. minag > nwcov+4
      if ( lmulti ) then
         multv = 2
      else
         multv = 1
      end if
      call pacmvt( ierr, nw, nvar, nv, nwdep+1, multv, baylev, 0,       &
                   .true.,  noeqm, .true., iuw,                         &
                   boik, qcst, bcst, nomv(ndep+1:nvtot), ncova, tmpcov,  &
                   wrk2(1:1), 1, eff(:,leffc:), ys2, yqg, v, ag, supg,   &
                   wrk0(1:nv*nw*nvar), wrk1(1:nwcot) )
      lerr = ierr /= 0

   else
        ! The matrix reduces to a scalar, check for non-zero variance
      lerr = ys2(nwdct) <= zero

   end if
   if ( lerr ) then
      call chkpg( 4, .false., newp )
      write(imp,5555)
      lines = lines + 4
      write(lgu,*) ermrk1//ermrk1
      write(lgu,5555)
      write(lgu,*) ermrk2//ermrk2
      goto 20
   end if

   ! STEP 4. ANALYZE DEPENDENT VARIABLES ADJUSTED FOR COVARIATES
   ! ===========================================================

   call chkpg( 22, .false., newp )
   if ( newp ) then
      write(imp,333) cmptit(:lcmpt)
      lines = lines + 1
   end if
   write(imp,2222)
   lines = lines + 4

   ! Some usefull constants
   ! ----------------------
   rqb = one / (qcst*bcst)
   qadj = qcst - nwcov
   if ( qadj <= zero ) then
      lerr = .true.
      call chkpg( 4, .false., newp )
      write(imp,6666)
      lines = lines + 4
      write(lgu,*) ermrk1//ermrk1
      write(lgu,6666)
      write(lgu,*) ermrk2//ermrk2
      goto 20
   end if

      ! Tests without equal variances assumption not implemented
   nokod = 0
   noeqm = .false.

   ! Number of dependent variables for this step
   ! -------------------------------------------
   nvar = ndep

   ! Covariate regression coefficients per group
   ! -------------------------------------------
      ! Now wrk0 is covariate regression coefficients array
      ! For each group the matrix of coefficients is calculated and
      ! stored on iuss, to be printed later on (those matrices are not
      ! actually used, but are printed in order one can make up his
      ! mind about the assumption of equal slope in all groups)
   if ( levout(5) .and. ngsup > 1 ) then
      rewind iuss
      ll = 0
      do ks = 1, ngsup
            ! Copy yqg<cov> into wrk1
         ii = nwdet
         k = 0
         do i = nwdep+1, nwdc
            do j = ii+nwdep+1, ii+i
               k = k + 1
               wrk1(k) = yqg(ll+j)
            end do
            ii = ii + i
         end do
            ! Replace wrk1 by its inverse
         call msinv( wrk1(1:nwcot), nwcov, ierr )
         if ( ierr /= 0 ) exit
            ! Compute regression coefficients in wrk0 and save on iuss
         lll = ll + nwdet
         call msxc( wrk1(1:nwcot), yqg(lll+1:), wrk0(1:nwcwd), nwcov, nwdep )
         write(iuss) wrk0(1:nwcwd)
         ll = ll + nwdct
      end do
   end if

   ! Inverse of pooled within-groups covariances matrix
   ! --------------------------------------------------
   if ( nwcov > 1 ) then
         ! Compute wrk1 = s**-2 <for cova> = (s**-1<cov>)**2
         ! (first, copy s**-1, saved on iuw by pacmvt, in wrk2)
      rewind iuw
      read(iuw) wrk2(1:nwcot)
      call mssz( wrk2(1:nwcot), wrk1(1:nwcot), nwcov )
   else
         ! Compute 1/(ys2)<cov>
      wrk1(1) = one / ys2(nwdct)
   end if

   if ( nv*nwcwd > 1 ) then

      ! Corrective matrix component = eff<cov> * s**-2<cov> * eff'<cov>
      ! ---------------------------
      call mastaz( eff(:,leffc:), wrk1(1:nwcot), wrk2(1:nvt), nv, nwcov )
         ! Save it on iuw for further use
      rewind iuw
      write(iuw) wrk2(1:nvt)

      ! Covariate regr. coeff. = s**-2<cov> * ys2<cov,dep>
      ! ----------------------
         ! Read them from iupr unit, if available
      if ( samew ) then
         read(iupr) wrk0(1:nwcwd)
      else
         call msxc( wrk1(1:nwcot), ys2(ldc:), wrk0(1:nwcwd), nwcov, nwdep )
      end if

      ! Save arrays for possible use in next comparison
      ! -----------------------------------------------
      rewind iupr
      write(iupr) gsup(1:ngrp)
      write(iupr) w(:,1:nw)
      write(iupr) ys2(1:nwdct)
      write(iupr) yqg(1:nwdctg)

      ! Adjusted effects = eff<dep> - eff<cov>*slopes
      ! ----------------
      call mabc( eff(:,leffc:), wrk0(1:nwcwd), wrk1(1:nv*nwdep), nv, nwcov, nwdep )
      eff(:,1:nwdep) = eff(:,1:nwdep) - reshape( wrk1(1:nv*nwdep), (/nv,nwdep/) )

      ! Adjusted ys2 = (ys2<dep>-ys2<dep,cov>*wrk0) * q/adj.q
      ! ------------
         ! (first  wrk1 = ys2<dep,cov> * wrk0; i.e. the matrix of
         ! sum of products for the reg. coeff, up to a constant)
      call mxbs( ys2(ldc:), wrk0(1:nwcwd), wrk1(1:nwdet), nwcov, nwdep )
      temp = qcst / qadj
      ys2(1:nwdet) = ( ys2(1:nwdet) - wrk1(1:nwdet) ) * temp

      ! Adjusted yqg per group (if requested)
      ! ----------------------
         ! yqg = yqg<dep> + wrk0'*yqg<cov>*wrk0 - 2*yqg<dep,cov>*wrk0
      if ( levout(3) .or. levout(6) ) then
         ll = 1
         do ks = 1, ngsup
               ! Copy yqg<cov> in wrk2
            ii = 1
            jk = nwdet + nwdep
            do i = 1, nwcov
               do j = 1, i
                  wrk2(ii) = yqg(ll+jk)
                  ii = ii + 1
                  jk = jk + 1
               end do
               jk = jk + nwdep
            end do
               ! wrk1 = wrk0'*yqg<cov>*wrk0
            call mtasaz( wrk0(1:nwcwd), wrk2(1:nwcot), wrk1(1:nwdet), nwcov, nwdep )
               ! wrk2 = yqg<dep,cov>*wrk0
            call mxbs( yqg(ll+nwdet:), wrk0(1:nwcwd), wrk2(1:nwdet), nwcov, nwdep )
               ! yqg<dep> = yqg<dep> + wrk1 - 2*wrk2
            do i = 1, nwdet
               yqg(ll-1+i) = yqg(ll-1+i) + wrk1(i) - (wrk2(i)+wrk2(i))
            end do
            ll = ll + nwdct
         end do
      end if

   else
      ! Same computations as above, but with scalars
      ! --------------------------------------------
         ! Save arrays for possible use in next comparison
      rewind iupr
      write(iupr) gsup(1:ngrp)
      write(iupr) w(:,1:nw)
      write(iupr) ys2(1:nwdct)
      write(iupr) yqg(1:nwdctg)
         ! eff<cov>**2 * s**-2<cov>
      wrk2(1) = eff(1,leffc) * eff(1,leffc) * wrk1(1)
         ! Save it on iuw for further use
      rewind iuw
      write(iuw) wrk2(1)
         ! Slope of covariate
      wrk0(1) = wrk1(1) * ys2(ldc)
         ! Adjusted effect
      eff(1,1) = eff(1,1) - eff(1,leffc)*wrk0(1)
         ! Wrk1 = ys2<dep,cov> * slope; i.e. sum-of-squares for slope
      wrk1(1) = ys2(ldc)*wrk0(1)
         ! Adjusted YS2
      ys2(1) = (ys2(1)-wrk1(1))* qcst/qadj
         ! Adjusted yqg per group (if requested)
      if ( levout(3) .or. levout(6) ) then
        ll = 1
        do ks = 1, ngsup
           yqg(ll) = yqg(ll) + yqg(ll+2)*wrk0(1)**2 - two*yqg(ll+1)*wrk0(1)
           ll = ll + nwdct
        end do
      end if

   end if

      ! Save regr. coeff. for possible use in next comparison
   write(iupr) wrk0(1:nwcwd)

   ! Compute adjusted y statistics and save derived protocol, if needed
   ! ------------------------------------------------------------------
   if ( needdp .or. ((levout(2).or.levout(5)).and..not.samew) ) then

         ! Compute adjusted y-means (in ybar)
         ! (on return wrk1 will contain general means of covariates
         ! multiplied by sumw)
      call pacadj( ybar, wrk1(1:nwcov), wrk0(1:nwcwd), ag, supg, sumw, nwdep, nwcov )
         ! Compute derived protocol and save it if required
      if ( needdp ) then
         judp = junit2
         if ( judp == iudp ) judp = junit3
         call filopn( 0, judp, ' ', 'SCRATCH', 'UNFORMATTED', .false., lerr )
         if ( lerr ) goto 20
         call pacdpb( iudp, judp, wrk0(1:nwcwd), wrk1(1:nwcov), ncasin, nwdep, nwcov )
            ! Reset iudp for possible further utilization by pacdpl
         iudp = judp
      end if
   end if

   ! Print adjusted y descriptive statistics
   ! ---------------------------------------
   call pacyds( samew, ngprt, nvar, labg, labv, nomwr(1:ncw*nvar), nomv, .false., zero )

      ! Departure from circularity
   if ( nw > 1 ) call pacdfc( ys2(1:nwdet), nw, nvar, nomv, boik, wrk3(1:nvar) )
      ! (wrk3 now contains departure from circularity for the nvar dependent variables)

   if ( .not. samew ) then

      if ( levout(5) ) then
            ! Print regression coefficients
         call chkpg( 6, .false., newp )
         if ( newp .and. .not. linter ) then
            write(imp,333) cmptit(:lcmpt)
            lines = lines + 1
         end if
         write(imp,333) '  ', msgr, undlin(:zmsgr)
         lines = lines + 3
         call prtmat( iout, .true., wrk0(1:nwcwd), nwcov, nwcov, nwdep, nomwr(nwdep+1:), nomwr )
         if ( ngsup > 1 ) then
               ! Print regression coefficients per group
            if ( ierr == 0 ) then
               rewind iuss
               do ks = 1, ngsup
                  call chkpg( 6, .false., newp )
                  if ( newp .and. .not. linter ) then
                     write(imp,333) cmptit(:lcmpt)
                     lines = lines + 1
                  end if
                  k = supg(ks)
                  regrp = regrg // labg(k)
                  write(imp,333) '  ', msgr//regrp, undlin(:zmsgr+zreg+8)
                  lines = lines + 3
                  read(iuss) wrk2(1:nwcwd)
                  call prtmat( iout, .true., wrk2(1:nwcwd), nwcov, nwcov, nwdep, nomwr(nwdep+1:), nomwr )
               end do
            else
               call chkpg( 2, .false., newp )
               if ( newp .and. .not. linter ) then
                  write(imp,333) cmptit(:lcmpt)
                  lines = lines + 1
               end if
               write(imp,333) '  ', msgrx
               lines = lines + 2
            end if
         end if

      end if

      ! Univariate & multivariate procedures on regression coefficients
      ! ---------------------------------------------------------------
      call chkpg( 20, .false., newp )
      if ( newp .and. .not. linter ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,7777) msgpr
      lines = lines + 3

         ! Store corrective matrix: qcst * ys2<cov> in wrk1
      ii = 1
      ll = nwdet + nwdep + 1
      do i = 1, nwcov
         do j = 1, i
            wrk1(ii) = qcst * ys2(ll)
            ii = ii + 1
            ll = ll + 1
         end do
         ll = ll + nwdep
      end do

         ! Compute and print sum of squares for the slopes
      if ( levout(4) )  &
         call pacyss( nvar, nwcov, wrk0(1:nwcwd), racst, .true.,  &
                      wrk1(1:nwcot), nwcot, nomwr(1:ncw*nvar), nomv )

      j = nwcov * nw
      temp = j
      call pacumt( nw, nvar, nwcov, temp, j == 1, iuss, -1, qadj, one, racst,             &
                   baylev, nokod, lschef, noeqm, boik, -1, msgv(1), .false.,              &
                   wrk0(1:nwcwd), wrk2(1:nwcwd), wrk3(1:nvar), wrk1(1:nwcot), nwcot, lerr )

   end if

   ! Take inverse of corrective matrix
   ! ---------------------------------
      ! Restore its component, previously saved on iuw
   rewind iuw
   read(iuw) wrk0(1:nvt)
   if ( nv > 1 ) then
         ! First compute corrective matrix = Iv + wrk0/(qcst*bcst)
      ij = 0
      do i = 1, nv
         do j = 1, i
            ij = ij + 1
            wrk0(ij) =  rqb * wrk0(ij)
         end do
         wrk0(ij) = one + wrk0(ij)
      end do
         ! Replace corrective matrix by its inverse
      call msinv( wrk0(1:nvt), nv, ierr )
      if ( ierr /= 0 ) then
         lerr = .true.
         goto 10
      end if
   else
         ! The matrix reduces to a scalar
      wrk0(1) = one / (one + rqb*wrk0(1))
   end if

   ! Univariate and multivariate procedures
   ! --------------------------------------
   if ( .not. samew ) then
      call chkpg( 20, .false., newp )
      if ( newp .and. .not. linter ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,7777) msgpv
      lines = lines + 3
   end if

      ! Compute and print sum of squares of the comparison
   if ( levout(4) ) call pacyss( nvar, nv, eff, racst/bcst, .true.,        &
                                 wrk0(1:nvt), nvt, nomwr(1:ncw*nvar), nomv )

   call pacumt( nw, nvar, nv, df1, l1df1, iuss, -1, qadj, bcst, racst,        &
                baylev, nokod, lschef, noeqm, boik, 1, msgv(1), .false.,      &
                eff, wrk2(1:nv*nw*nvar), wrk3(1:nvar), wrk0(1:nvt), nvt, lerr )


10 continue

   ! STEP 5. DATA REDUCTION PROCEDURE
   ! ================================

   if ( lrduce .and. .not. lerr1 ) then
         ! Allocate local arrays
      if ( allocated(zbar) ) deallocate(zbar)
      if ( allocated(eign) ) deallocate(eign)
      allocate ( zbar(ranka,ngsup), eign(ranka), stat=status )
      if ( status /= 0 ) then   ! Fatal error
         write(lgu,*) ermrk1
         write(lgu,101) " (PacY)."
         write(lgu,*) ermrk2
         call gmp999            ! Release memory space and files
         call pac999( 99 )
         stop
      end if
      ii = iu2   ! Input data unit (to avoid double transmission with gmpin)
      call pacz( iuw, qcst, bcst, sumv, ii, iudp, ncasin, nssav, zbar, eign, ierr )
      lerr1 = ierr /= 0
   end if

   ! Prompt for successfull saving of derived protocol
   ! -------------------------------------------------
   if ( lderiv .and. .not. (lrduce .and. lerr1) ) then
      call chkpg( 3+1, .false., newp )
      if ( newp ) then
         write(imp,333) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,8888) ideriv, nssav
      lines = lines + 3
      if ( lrduce ) then
         write(imp,9999)
         lines = lines + 1
      end if
   end if

   ! Print derived protocol if requested
   ! -----------------------------------
   if ( .not. samew ) prvldp = .false.
   if ( listdp ) then
      if ( .not. lrduce ) then
         if ( samew .and. prvldp ) then
            call chkpg( 6, .false., newp )
            write(imp,3333)
            lines = lines + 6
         else
            ll = nwdc
            lll = ncw * nvar
            call pacdpl( iudp, gsup, supg, ag, ybar, ll, ys2, labg, nomwr(1:lll), lll )
            prvldp = .true.
         end if
      else if ( .not. lerr1 ) then
         lll = ranka
         call pacdpl( iudp, gsup, supg, ag, zbar, lll, ys2, labg, eign, lll )
         prvldp = .false.
      end if
   end if

20 continue

      ! Free local arrays
   if ( allocated(udpw) ) deallocate(udpw)
   if ( allocated(supg) ) deallocate(supg)
   if ( allocated(ybar) ) deallocate(ybar)
   if ( allocated(ys2) ) deallocate(ys2)
   if ( allocated(eff) ) deallocate(eff)
   if ( allocated(ene) ) deallocate(ene)
   if ( allocated(qne) ) deallocate(qne)
   if ( allocated(yqg) ) deallocate(yqg)
   if ( allocated(zbar) ) deallocate(zbar)
   if ( allocated(eign) ) deallocate(eign)
   if ( allocated(wrk0) ) deallocate(wrk0)
   if ( allocated(wrk1) ) deallocate(wrk1)
   if ( allocated(wrk2) ) deallocate(wrk2)
   if ( allocated(wrk3) ) deallocate(wrk3)
      ! Close local scratch units
   if ( iuw  > 0 ) close(iuw)
   if ( iudp > 0 ) close(iudp)
   if ( judp > 0 ) close(judp)

   INCLUDE 'allocf.'
   INCLUDE 'pacyf.'

END SUBROUTINE pacy
