SUBROUTINE polreg( klast, kdeg, rtyp, prop, intcp, np, z, pa,   &
                   lrsav, ursav, nrsavd, c )
!***********************************************************************
!     - polreg -
!     computes the matrix of contrast-coefficients associated to a
!     POLynomial REGression of given degree
!***********************************************************************
!
!     KLAST - INPUT . Maximum degree involved in computation  - INTEGER
!                     of contrasts
!     KDEG  - INPUT . Requested degree of the polynomial      - INTEGER
!     RTYP  - INPUT . Type of the regression                  - INTEGER
!                     1 = only a specific degree requested
!                     2 = cumulative degrees
!                     A negative type codes for the residual
!                     comparison
!     PROP  - INPUT . .true. if proportionality is requested  - LOGICAL
!                     (implies KDEG = 1 and RTYP = 1 or -1)
!     INTCP - INPUT . .true. if intercept is requested        - LOGICAL
!                     (This special case is not directly
!                     calculated, but is got by equivalencing
!                     to the residual to proportionality case
!                     and keeping only one contrast. Thus it
!                     implies following arguments were set to:
!                     PROP=.true. and KDEG=1 and RTYP=-1)
!     NP    - INPUT . Number of points on which applies the   - INTEGER
!                     polynomial regression
!     Z     - INPUT . Vector of the regression coefficients,  - REAL*8
!                     of length NP
!     PA    - INPUT . Vector of part-weights, of length NP,   - REAL*8
!                     the reciprocal of diagonal terms of
!                     the metric used (i.e. contrasts will be
!                     orthogonal with respect to the PA**-1
!                     metric)
!     LRSAV - INPUT . .true. if the vector PA & the variables - LOGICAL
!                     (vectors) temporary built during the
!                     computations must be saved on scratch
!                     unit URSAV
!     URSAV - INPUT . Scratch unit number for this saving     - INTEGER
!     NRSAVD- OUTPUT. Number of variables (vectors of length  - INTEGER
!                     NP) saved on scratch unit URSAV
!                     Relevant only if LRSAV is .true.
!                     (set to 0 otherwise)
!     C     - OUTPUT. Matrix of the contrast-coefficients for - REAL*8
!                     each degree, 2-dimensioned (NP,0:KLAST)
!                     (columns are contrasts)
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MIN  SUM  SQRT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: klast, kdeg, rtyp, np, ursav
   LOGICAL, INTENT(IN) :: prop, intcp, lrsav
   REAL(KIND=8), INTENT(IN) :: z(np), pa(np)
   INTEGER, INTENT(OUT) :: nrsavd
   REAL(KIND=8), INTENT(OUT) :: c(np,0:klast)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: a10, q, r, s, t, tpa
   INTEGER :: i, ii, ip, j, ji, jj, k, km1, l
   LOGICAL :: only1, sava
   INTEGER :: ib(0:klast)
   REAL(KIND=8) :: a(0:klast), h(0:(1+klast)*(2+klast)/2-2)

   !--------------------------------------------------------------------
      ! h is used to store the upper triangle of a symmetric matrix of
      ! order 0:klast, less the last diagonal cell which is not used.
      ! The common square norm of the contrasts will be, with m being
      ! the number of contrasts involved:
      ! - if not residual type and m > 1 :  (m/(np-1)) * (2/r)
      !   where r = mean(pa) - ( corrected std.dev.(pa) / sum(pa) );
      !   and if m = 1 the norm of the contrast, once derived from the
      !   corresponding variable, is left unchanged;
      ! - if residual type: m / sum(pa)  whatever the value of m.
      ! The a coefficients will be saved only if type is not residual
      ! and degree is greater than 0 (they will be needed to compute
      ! the coefficients of the adjusted polynomial).

   nrsavd = 0
   sava = lrsav .and. rtyp > 0 .and. kdeg > 0
   if ( lrsav ) then
      rewind ursav
      write(ursav) pa
      write(ursav) klast, kdeg, sava
   end if

   ! Initialize & get denominator of the norm**2 for the contrasts (r)
   ! -----------------------------------------------------------------
   t = zero
   s = zero
   do ip = 1, np
      c(ip,0) = one
      t = t + pa(ip)
      s = s + pa(ip)*pa(ip)
   end do
   tpa = t
   if ( rtyp > 0 ) then

      if ( kdeg == 0 ) then
            ! Simplification occurs for non-residual degree 0
            ! Save variable (unit vector) if requested
         if ( lrsav ) then
            write(ursav) (one,ip=1,np)
            nrsavd = 1
         end if
         t = one / t
         c(:,0) = pa * t
         return
      end if

      if ( prop ) then
            ! Special case for proportionality
            ! Save (constrained) variables if requested
         if ( lrsav ) then
               ! Save a coefficients (as sava is obviously .true.)
            write(ursav) one
            write(ursav) zero, one
               ! Save (constrained) variables
               ! N.B. Variable at degree 0 is here forced to 0, so that
               ! standard procedure for calculating polynomial coeff.
               ! (in routine pacppm) will give 0 for degree 0
            write(ursav) (zero,ip=1,np), z
            nrsavd = 2
         end if
         s = sum( pa * z * z )
         c(:,1) = (pa * z) / s
         return
      end if

      q = np
      s = (s - t*t/q) / (np-1)
      r = t/q - s/t

   else

      r = t
   end if

   a(0) = one
      ! Save a coefficients if necessary
   if ( sava ) write(ursav) a(0)

   ! Compute the c(ip,k)
   ! -------------------
   do k = 1, klast
      km1 = k - 1

         ! Compute the h(i,k-1)
      ii = km1*k / 2
         ! Take account of special case i=0, where 0.**0 may occur
      if ( km1 == 0 ) then
         h(ii) = t
         ii = ii + 1
         do i = k, klast
            h(ii) = sum( (z**i) * pa )
            ii = ii + i + 1
         end do
      else
         do i = km1, klast
            h(km1+ii) = sum( c(:,km1) * (z**i) * pa )
            ii = ii + i + 1
         end do
      end if
         ! Compute the a(j)
      a(k) = one
      jj = k*(k+3) / 2
      do j = km1, 0, -1
         jj = jj - j - 2
         ji = jj
         s  = zero
         do i = j+1, k
            ji = ji + i
            s  = s + a(i)*h(ji)
         end do
         a(j) = -s / h(jj)
      end do
      if ( k == 1 ) a10 = a(0)
         ! Save a coefficients if necessary
      if ( sava ) write(ursav) (a(i),i=0,k)
         ! Compute the variables c(ip,k)
      do ip = 1, np
         s = a(0)
         do i = 1, k
            s = s + a(i)*(z(ip)**i)
         end do
         c(ip,k) = s
      end do

   end do

   if ( prop ) then
         ! Transform c(.,0) and c(.,1)
      s = sum( pa * c(:,1) * c(:,1) )
      c(:,0) = (s + a10*c(:,1)*tpa) / (s + a10*a10*tpa)
      c(:,1) = z
   end if

      ! Now the c(ip,k) are the variables; replace them by contrasts
      ! (measures) with a common norm if more than one contrast

      ! Fill ib with indices of relevant variables/contrasts according
      ! to type, l being their number.
      ! only1 flags only one contrast and not residual, or special
      ! intercept case.
   if ( rtyp > 0 ) then
      if ( rtyp == 1 ) then
         l = 1
         ib(1) = kdeg
      else
         l = kdeg
         do k = 1, kdeg
            ib(k) = k
         end do
      end if
         ! Save variables if requested
      if ( lrsav ) then
         write(ursav) (one,ip=1,np), ((c(ip,ib(j)),ip=1,np),j=1,l)
         nrsavd = l + 1
      end if
      only1 = l == 1
   else
      only1 = intcp
      l = 0
      if ( rtyp == -1 .and. kdeg > 0 ) then
         l = min( kdeg-1, klast )
         do k = 1, l
            ib(k) = k
         end do
      end if
      if ( prop ) then
         l = 1
         ib(1) = 0
      end if
      if ( .not. intcp ) then
         do k = kdeg+1, klast
            l = l + 1
            ib(l) = k
         end do
      end if
         ! Save variables if requested (without the 0-degree one,
         ! unless proportionality applies)
      if ( lrsav ) then
         write(ursav) ((c(ip,ib(j)),ip=1,np),j=1,l)
         nrsavd = l
      end if
   end if
      ! Get contrasts from variables, with unit norm if more than one
      ! or residual type, with the final desired norm otherwise
   do j = 1, l
      k = ib(j)
      s = sum( pa * c(:,k) * c(:,k) )
      if ( only1 ) then
         s = one / s
      else
         s = one / sqrt(s)
      end if
      c(:,k) = pa * c(:,k) * s
   end do
   if ( only1 ) return

      ! Now contrasts have unit norm; complete calculation of the
      ! desired common norm (r) and renorm them.
      ! For residual type, multiply the norm by -1 so that contrast
      ! coefficients will have their sign changed and then will
      ! correspond to the order (observed values - predicted values).
   if ( rtyp > 0 ) then
      r = sqrt( (l+l) / ((np-1)*r) )
   else
      r = -sqrt( l / r )
   end if
   do j = 1, l
      k = ib(j)
      c(:,k) = c(:,k) * r
   end do

END SUBROUTINE polreg
