SUBROUTINE proset( set1, set2, nrow, ns1, ns2 )

!***********************************************************************
!     - proset -
!     computes a set of contrasts which is the PROduct of 2 SETs of
!     contrasts in a same space
!***********************************************************************
!
!     SET1  - INPUT . The first set with its NS1 contrast     - REAL*8
!                     vectors in the NS1 first columns
!                     2-dimensioned (NROW, NS1*NS2)
!             OUTPUT. Product set, with vectors of 1st set
!                     varying faster:
!                     let p1, p2, p3 be the 1st set input
!                     vectors, q1, q2 those of the 2nd set,
!                     then output columns in set1 will be:
!                     p1*q1,p2*q1,p3*q1,p1*q2,p2*q2,p3*q2
!     SET2  - INPUT . The second set                          - REAL*8
!                     2-dimensioned (NROW, NS2)
!                     Note that SET2(1,1) may be same address
!                     as SET1(1,NS1+1), in which case input
!                     SET2 values will be destroyed
!     NROW  - INPUT . Number of rows in SET1 and SET2,        - INTEGER
!                     i.e. dimension of the common space
!     NS1   - INPUT . Number of contrast vectors in SET1      - INTEGER
!                     on input
!     NS2   - INPUT . Number of contrast vectors in SET2      - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nrow, ns1, ns2
   REAL(KIND=8), INTENT(IN) :: set2(nrow,ns2)
   REAL(KIND=8), INTENT(INOUT) :: set1(nrow,ns1*ns2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: x, y
   INTEGER :: i, jj, j1, j12, j2

   !--------------------------------------------------------------------
      ! Vectors in set1 are crossed with those in set2
      ! For any resulting vector, any coordinate is simply the product
      ! of homologous coordinates of the two vectors in set1 and set2
      ! set1 is filled in such a way that set2(1,1) may be set1(1,ns1+1)

   ! Special cases
   ! -------------
   if ( ns2 == 1 ) then
      do j1 = 1, ns1
         set1(1:nrow,j1) = set1(1:nrow,j1) * set2(1:nrow,1)
      end do
      return
   end if
   if ( ns1 == 1 ) then
      do i = 1, nrow
         x = set1(i,1)
         set1(i,1:ns2) = set2(i,1:ns2) * x
      end do
      return
   end if

   ! Take product for last set2 vectors up to the 3rd one
   ! ----------------------------------------------------
   if ( ns2 > 2 ) then
      j12 = ns1 * ns2
      do j2 = ns2, 3, -1
         do j1 = ns1, 1, -1
            set1(1:nrow,j12) = set1(1:nrow,j1) * set2(1:nrow,j2)
            j12 = j12 - 1
         end do
      end do
   end if

   ! Now proceede for the first 2 set2 vectors
   ! -----------------------------------------
   jj = ns1 + ns1
   do i = 1, nrow
      j12 = jj
      y = set2(i,2)
      x = set2(i,1)
      do j1 = ns1, 1, -1
         set1(i,j12) = set1(i,j1) * y
         set1(i,j1)  = set1(i,j1) * x
         j12 = j12 - 1
      end do
   end do

END SUBROUTINE proset
