SUBROUTINE rescal( type, ng, nt, nv, nw, gag, tat, k1, i1, resv, resw,  &
                   gpar, tpar, vwgh, wwgh, v, maxv, w, maxw, part, pwgh )
!***********************************************************************
!     - rescal -
!     rescales to the G&T set the V and/or W contrast-coefficients
!     associated to the parts of the comparison, according to the
!     G and T-weights.
!***********************************************************************
!
!     TYPE  - INPUT . Type of the comparison:                 - INTEGER
!                        1 = W/G1 type    (product form)
!                        2 = V/T1 type    (product form)
!                        3 = V.W  type    (product form)
!                        4 = general type (not product form)
!     NG    - INPUT . Number of groups                        - INTEGER
!     NT    - INPUT . Number of treatments                    - INTEGER
!     NV    - INPUT . Number of V-contrasts                   - INTEGER
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     GAG   - INPUT . Vector of group-weights, of length NG   - REAL*8
!     TAT   - INPUT . Vector of treatment-weights,            - REAL*8
!                     of length NT
!                     --- FOLLOWING ARGUMENTS ARE
!                     --- IRRELEVANT IF TYPE IS 4
!     K1    - INPUT . Index of 1st group involved in the      - INTEGER
!                     comparison
!     I1    - INPUT . Index of 1st treatment involved in the  - INTEGER
!                     comparison
!     RESV  - INPUT . .false. if rescaling on V is useless    - LOGICAL
!                     (coefficient is 1.), .true. otherwise
!                     Irrelevant if type is 1
!     RESW  - INPUT . .false. if rescaling on W is useless    - LOGICAL
!                     (coefficient is 1.), .true. otherwise
!                     Irrelevant if type is 2
!     GPAR  - INPUT . G-partition vector, of length NG        - INTEGER
!     TPAR  - INPUT . T-partition vector, of length NT        - INTEGER
!     VWGH  - INPUT . Vector of G-part weights                - REAL*8
!     WWGH  - INPUT . Vector of T-part weights                - REAL*8
!     V     - INPUT . Matrix of V-contrast coefficients       - REAL*8
!                     2-dimensioned (NG,MAXV)
!             OUTPUT.
!     MAXV  - INPUT . Maximum number of columns of matrix V   - INTEGER
!     W     - INPUT . Matrix of W-contrast coefficients       - REAL*8
!                     2-dimensioned (NT,MAXW)
!             OUTPUT.
!     MAXW  - INPUT . Maximum number of columns of matrix W   - INTEGER
!                     --- FOLLOWING ARGUMENTS ARE
!                     --- IRRELEVANT IF TYPE IS NOT 4
!     PART  - INPUT . Partition matrix, 2-dimensioned (NT,NG) - INTEGER
!     PWGH  - INPUT . Vector of GT-part-weights               - REAL*8
!     U     - INPUT . Matrix of contrast coefficients on the  - REAL*8
!                     G&T set, 2-dimensioned (NT,NG)
!             OUTPUT.
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        GMP999  PAC999
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: type, nt, ng, nv, nw, k1, i1, maxv, maxw
   LOGICAL, INTENT(IN) :: resv, resw
   INTEGER, INTENT(IN) :: gpar(ng), tpar(nt), part(nt,ng)
   REAL(KIND=8), INTENT(IN) :: gag(ng), tat(nt), vwgh(*), wwgh(*), pwgh(*)
   REAL(KIND=8), INTENT(INOUT) :: v(ng,maxv), w(nt,maxw)

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: p
   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   INTEGER :: i, j, k

   !--------------------------------------------------------------------

   if ( type == 1 .or. type == 3 ) then
      ! W/g1       or  V.W type
      ! -----------------------
      if ( resw ) then
            ! Rescale W
         do i = i1, nt
            if ( tpar(i) > 0 ) then
               p = tat(i) / wwgh(tpar(i))
               w(i,1:nw) = w(i,1:nw) * p
            end if
         end do
      end if
      if ( type == 1 ) then
            ! W/g1 type, compute unique V and exit
         p = sum( gag(k1:ng), mask=gpar(k1:ng)>0 )
         p = one / p
         do k = 1, ng
            if ( gpar(k) > 0 ) then
               v(k,1) = gag(k) * p
            else
               v(k,1) = zero
            end if
         end do
         return
      end if
   end if

   if ( type == 2 .or. type == 3 ) then
      ! V/t1       or  V.W type
      ! -----------------------
      if ( resv ) then
            ! Rescale V
         do k = k1, ng
            if ( gpar(k) > 0 ) then
               p = gag(k) / vwgh(gpar(k))
               v(k,1:nv) = v(k,1:nv) * p
            end if
         end do
      end if
      if ( type == 2 ) then
            ! V/t1 type, compute unique W
         p = zero
         do i = i1, nt
            if ( tpar(i) > 0 ) p = p + tat(i)
         end do
         p = one / p
         do i = 1, nt
            if ( tpar(i) > 0 ) then
               w(i,1) = tat(i) * p
            else
               w(i,1) = zero
            end if
         end do
      end if
      return
   end if

   if ( type == 4 ) then
      ! General type (not product form) (currently limited to 1 df)
      ! -------------------------------
         ! Rescale W coefficients
      do k = 1, ng
         p = gag(k)
         do i = 1, nt
            j = part(i,k)
            if ( j > 0 ) w(i,k) = w(i,k) * p * tat(i) / pwgh(j)
         end do
      end do
   else     ! Error
      call gmp999()
      call pac999( 99 )
      stop 'Rescal'
   end if

END SUBROUTINE rescal
